/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.db.rmi;

import java.rmi.RemoteException;
import java.util.Date;

import de.willuhn.datasource.rmi.DBObject;

/**
 * Interface of the business object for history. According to the SQL table, we
 * define some getter an setter here.
 * 
 * <pre>
 * CREATE TABLE history (
 * id NUMERIC default UNIQUEKEY('history'),
 * aktien_id int(4) NOT NULL,
 * tradingDate DATE,
 * tradePrice double,
 * UNIQUE (id),
 * PRIMARY KEY (id)
 * );
 * </pre>
 * 
 * <br>
 * Getters and setters for the primary key (id) are not needed. Every one of the
 * following methods has to throw a RemoteException. <br>
 */
public interface ValueHistory extends DBObject {

	/**
	 * Liefert die Wertpapier.
	 * 
	 * @return die Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	Wertpapier getAktie() throws RemoteException;

	/**
	 * Liefert das TradingDate des ValueHistory.
	 * 
	 * @return tradingDate des ValueHistory.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	Date getTradingDate() throws RemoteException;

	/**
	 * Liefert den TradingPrice des ValueHistory.
	 * 
	 * @return tradingPrice des ValueHistory.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	double getTradingPrice() throws RemoteException;

	/**
	 * Setzt die Wertpapier
	 * 
	 * @param wertpapier
	 *            die Wertpapier
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	void setAktie(Wertpapier wertpapier) throws RemoteException;

	/**
	 * Setzt das tradindDate des ValueHistory.
	 * 
	 * @param tradingDate
	 *            des ValueHistory.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	void setTradingDate(Date tradingDate) throws RemoteException;

	/**
	 * Setzt den TradingPrice des ValueHistory.
	 * 
	 * @param tradingPrice
	 *            des ValueHistory.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	void setTradingPrice(double tradingPrice) throws RemoteException;

}
