/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.db.rmi;

import java.rmi.RemoteException;

import de.willuhn.datasource.rmi.DBObject;

/**
 * Interface fuer einen Datensatz in der Versionstabelle.
 * 
 * <pre>
 * CREATE TABLE version (
 *  id NUMERIC default UNIQUEKEY('version'),
 *  name varchar(255) NOT NULL,
 *  version int(5) NOT NULL,
 *  UNIQUE (id),
 *  PRIMARY KEY (id)
 * );
 * </pre>
  * <br>
 * Getters und Setters f&uuml;r den Primary Key (id) sind nicht n&ouml;tig. Jede Methode muss
 * zwingend eine RemoteException werfen.
 * <br>
 * * 
 */
public interface Version extends DBObject {

	/**
	 * Liefert den Namen der Version.
	 * 
	 * @return Name der Version.
	 * @throws RemoteException
	 */
	String getName() throws RemoteException;

	/**
	 * Speichert den Namen der Version.
	 * 
	 * @param name
	 *            Name der Version.
	 * @throws RemoteException
	 */
	void setName(String name) throws RemoteException;

	/**
	 * Liefert den aktuellen Stand der Version.
	 * 
	 * @return Stand der Version.
	 * @throws RemoteException
	 */
	 int getVersion() throws RemoteException;

	/**
	 * Legt die neue Versionsnummer fest.
	 * 
	 * @param newVersion
	 *            die neue Versionsnummer.
	 * @throws RemoteException
	 */
	 void setVersion(int newVersion) throws RemoteException;

}

