/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.db.server;

import java.io.File;
import java.rmi.RemoteException;
import java.sql.Connection;

import de.frankmuenster.jameica.depot.JaDepot;
import de.frankmuenster.jameica.depot.db.rmi.JaDepotDBService;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.datasource.db.EmbeddedDBServiceImpl;
import de.willuhn.datasource.db.EmbeddedDatabase;
import de.willuhn.jameica.messaging.QueryMessage;
import de.willuhn.jameica.plugin.Manifest;
import de.willuhn.jameica.plugin.PluginResources;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.sql.version.UpdateProvider;
import de.willuhn.sql.version.Updater;
import de.willuhn.util.ApplicationException;

/**
 * @author Frank Mnster
 *
 */
public class JaDepotDBServiceImpl extends EmbeddedDBServiceImpl implements JaDepotDBService {

    /**
     * Serial UID
     */
    private static final long serialVersionUID = 2458711982794157288L;

    /**
     * @throws RemoteException
     *             rmi-exception
     */
    public JaDepotDBServiceImpl() throws RemoteException {
	super(Application.getPluginLoader().getPlugin(JaDepot.class).getResources().getWorkPath() + "/db/db.conf", //$NON-NLS-1$
		"aktien", "aktview"); //$NON-NLS-1$ //$NON-NLS-2$

	// We have to define jameicas classfinder.
	// otherwise, the db service will not be able to find
	// implementors by their interfaces.
	this.setClassFinder(Application.getClassLoader().getClassFinder());
    }

    /**
     * @see JaDepotDBService#checkConsistency()
     */
    @Override
    public void checkConsistency() throws RemoteException, ApplicationException {
	Logger.info(I18N.tr("JaDepotDBServiceImpl_startUpdateProviderInfo")); //$NON-NLS-1$
	final UpdateProvider provider = new JaDepotUpdateProvider(getConnection(), VersionUtil.getVersion(this, "db")); //$NON-NLS-1$
	final Updater updater = new Updater(provider, "iso-8859-1"); //$NON-NLS-1$
	updater.execute();
	Logger.info(I18N.tr("JaDepotDBServiceImpl_finishUpdateProviderInfo")); //$NON-NLS-1$

    }

    /**
     * @see JaDepotDBService#install()
     */
    @Override
    public void install() throws RemoteException {

	// Let's create an embedded Database
	final PluginResources res = Application.getPluginLoader().getPlugin(JaDepot.class).getResources();
	final Manifest manifest = Application.getPluginLoader().getManifest(JaDepot.class);
	EmbeddedDatabase database;

	try {
	    database = new EmbeddedDatabase(res.getWorkPath() + "/db", //$NON-NLS-1$
		    "aktien", "aktview"); //$NON-NLS-1$ //$NON-NLS-2$

	    // create the sql tables.
	    database.executeSQLScript(new File(manifest.getPluginDir() + File.separator + "sql", "create.sql")); //$NON-NLS-1$ //$NON-NLS-2$

	} catch (final Exception e) { // NOPMD
	    throw new RemoteException(e.getMessage(), e.getCause()); // NOPMD
	}

    }

    /**
     * @see de.willuhn.datasource.db.DBServiceImpl#getConnection()
     */
    @Override
    protected Connection getConnection() throws RemoteException {
	try {
	    return super.getConnection();
	} catch (final RemoteException re) {
	    // Wir benachrichtigen Jameica ueber den Fehler, damit beim Shutdown
	    // kein Backup erstellt wird
	    Application.getMessagingFactory().getMessagingQueue("jameica.error") //$NON-NLS-1$
		    .sendMessage(new QueryMessage(re));
	    throw re;
	}
    }
}
