/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.db.server;

import java.rmi.RemoteException;

import de.frankmuenster.jameica.depot.db.rmi.Version;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.util.ApplicationException;

/**
 * Utility class fr die Datenbank Versionierung.
 *
 * @author Frank Mnster
 */
public final class VersionUtil {

    /**
     * Liefert die Version der Komponenten. Ist diese noch nicht in der DB, wird
     * sie mit Version 1 angelegt.
     * 
     * @param service
     * 
     * @param name
     *            Name der Komponenten deren Version geliefert wwerden soll.
     * @return die Version
     * @throws RemoteException
     * @throws ApplicationException
     */
    public static Version getVersion(final DBService service, final String name)
	    throws RemoteException, ApplicationException {
	if (name == null || name.length() == 0) {
	    throw new ApplicationException(I18N.tr("VersionUtil.versionNullOrEmpty")); //$NON-NLS-1$
	}
	Version version;
	try {
	    final DBIterator<?> list = service.createList(Version.class);
	    list.addFilter("name = ?", name); //$NON-NLS-1$
	    if (list.hasNext()) {
		version = (Version) list.next();
	    } else {

		// Neue Version erstellen
		version = (Version) service.createObject(Version.class, null);
		version.setName(name);
		version.store();
	    }
	} catch (final RemoteException e) {
	    // Version 0 erstellen, da Versionstabelle noch nicht existiert.
	    version = new VersionImpl();
	    version.setName("db"); //$NON-NLS-1$
	    version.setVersion(0);
	}
	return version;
    }

    private VersionUtil() {
	// Utility class
    }
}
