/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.boerseffm;

import java.io.IOException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

import de.frankmuenster.jameica.depot.finanzen.SearchWertpapier;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.json.InstrumentType;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.json.SearchResponse;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 */
public final class FfmSearchWertpapier implements SearchWertpapier<FfmStockInfoBean> {

    /**
     * Unsere Instanz
     */
    private static FfmSearchWertpapier instance;

    /**
     * Liefert die FfmSearchWertpapier Instanz.
     *
     * @return die FfmSearchWertpapier Instanz.
     */
    public static FfmSearchWertpapier getIntance() {
	synchronized (FfmSearchWertpapier.class) {
	    if (FfmSearchWertpapier.instance == null) {
		FfmSearchWertpapier.instance = new FfmSearchWertpapier();
	    }
	}
	return FfmSearchWertpapier.instance;
    }

    /**
     * privater Standardkonstruktor
     */
    private FfmSearchWertpapier() {
	// absichtlich leer
    }

    /**
     * @see de.frankmuenster.jameica.depot.finanzen.SearchWertpapier#search(java.lang.String)
     */
    @Override
    public List<FfmStockInfoBean> search(final String name) {
	final List<FfmStockInfoBean> list = new ArrayList<>();
	URL url = null;
	try {
	    final String encodedName = URLEncoder.encode(name, java.nio.charset.StandardCharsets.UTF_8.name());
	    StringBuilder urlSB = new StringBuilder(130);
	    urlSB.append(FfmWebAPI.SEARCH_URL).append(encodedName).append(FfmWebAPI.SEARCH_URL_PARAMS);
	    final URL searchUrl = new URL(urlSB.toString());
	    final String responseString = FfmWebAPI.getInstance().getResonseGETRequest(searchUrl, true);
	    final SearchResponse searchResponse = FfmWebAPI.MAPPER.readValue(responseString, SearchResponse.class);
	    urlSB = new StringBuilder();
	    final List<InstrumentType> inst = searchResponse.getInstruments();
	    FfmStockInfoBean bean;
	    for (final InstrumentType instrument : inst) {
		Logger.debug(instrument.getName());
		Logger.debug("ID : " + instrument.getInstrumentID()); //$NON-NLS-1$
		final String urlType = InstrumentTypeEnum.getUrl(instrument.getInstrumentType());
		if (urlType != null) {
		    urlSB.setLength(0);
		    urlSB.append(urlType).append('/').append(instrument.getUrlName());
		    if (urlSB.length() > 0) {
			urlSB.insert(0, FfmWebAPI.MAIN_URL);
			url = new URL(urlSB.toString()); // NOPMD
			bean = new FfmStockInfoBean(); // NOPMD
			bean.setExchange("Brse FFM"); //$NON-NLS-1$
			bean.setName(instrument.getName());
			bean.setType(instrument.getInstrumentType());
			bean.setWkn(instrument.getWKN());
			bean.setIsin(instrument.getISIN());
			bean.setInstrumentId(Integer.toString(instrument.getInstrumentID()));
			bean.setUrl(url);
			list.add(bean);
		    }
		}
	    }
	} catch (IOException | FinanceException e) {
	    Logger.error(I18N.tr("FfmWertpapierHistory.createSearchURLFailed"), e); //$NON-NLS-1$
	}
	return list;
    }
}
