/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.boerseffm;

import org.apache.commons.lang.StringUtils;

/**
 * Mapper f&uuml;r dei URL eines InstrumentTyps.
 *
 * @author Frank Mnster
 */
public enum InstrumentTypeEnum {

    /**
     * Typ Aktie
     */
    AKTIEN("Aktien", "aktien"), //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * Typ Anleihe
     */
    ANLEIHEN("Anleihen", "anleihen"), //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * Typ ETF/ETP
     */
    ETF_ETP("ETF/ETP", "etp"), //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * Typ Index
     */
    INDIZES("Indizes", "index"), //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * Typ Fond
     */
    FONDS("Fonds", "fonds"), //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * Typ Rohstoffe
     */
    ROHSTOFFE("Rohstoffe", "rohstoffe"), //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * Typ W&aumlhrung
     */
    WAEHRUNG("Whrung", "devisen"), //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * Typ Zertifikat
     */
    ZERTIFIKATE("Zertifikate", "zertifikate"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * Der Name des Instrumenttyps
     */
    private String name;

    /**
     * Der Name in der URL
     */
    private String url;

    /**
     * Liefert InstrumentTyp zu einem Namen
     *
     * @param typeName
     *            der InstrumentTypName
     * @return den InstrumentTyp oder NULL, wenn der InstrumentTypName nicht bekannt ist.
     */
    public static InstrumentTypeEnum getInstrumentType(final String typeName) {

        InstrumentTypeEnum type = null;
        if (StringUtils.trimToNull(typeName) != null) {
            for (final InstrumentTypeEnum instrument : InstrumentTypeEnum.values()) {
                if (instrument.getName().equals(typeName)) {
                    type = instrument;
                    break;
                }
            }
        }
        return type;
    }

    /**
     * Liefert die URL eines InstrumentTypNames.
     *
     * @param typeName
     *            InstrumentTypName
     * @return die URL oder NULL, wenn der InstrumentTypName nicht bekannt ist.
     */
    public static String getUrl(final String typeName) {

        String url = null;
        if (StringUtils.trimToNull(typeName) != null) {
            for (final InstrumentTypeEnum instrument : InstrumentTypeEnum.values()) {
                if (instrument.getName().equals(typeName)) {
                    url = instrument.getUrl();
                    break;
                }
            }
        }
        return url;
    }

    InstrumentTypeEnum(final String name, final String url) {

        this.name = name;
        this.url = url;
    }

    /**
     * Liefert den InstrumentTypNamen
     *
     * @return den InstrumentTypNamen
     */
    public String getName() {

        return this.name;
    }

    /**
     * Liefert die URL
     *
     * @return die URL
     */
    public String getUrl() {

        return this.url;
    }

}
