/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.boerseffm.json;

import java.io.IOException;
import java.time.Instant;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.List;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.ObjectCodec;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonDeserializer;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;

/**
 * Deserialiser f&uuml; die ChartDaten.
 *
 * @author Frank Mnster
 */
public class DataDeserialiser extends JsonDeserializer<List<ChartData>> {

    /**
     * Standardkonstruktor
     */
    public DataDeserialiser() {
	super();
    }

    /**
     * @see com.fasterxml.jackson.databind.JsonDeserializer#deserialize(com.fasterxml.jackson.core.JsonParser,
     *      com.fasterxml.jackson.databind.DeserializationContext)
     */
    @Override
    public List<ChartData> deserialize(final JsonParser jsonParser, final DeserializationContext context)
	    throws IOException {
	final List<ChartData> dataList = new ArrayList<>();
	final ObjectCodec objectCodec = jsonParser.getCodec();
	final JsonNode jsonNode = objectCodec.readTree(jsonParser);
	final ArrayNode nodes = (ArrayNode) jsonNode;
	for (int i = 0; i < nodes.size(); i++) {
	    final JsonNode node = nodes.get(i);
	    final Object[] values = objectCodec.treeToValue(node, Object[].class);
	    final ChartData data = new ChartData(); // NOPMD
	    // Datum
	    if (values[0] instanceof Double) {
		final Double dateD = (Double) values[0];
		final LocalDate date = this.doubleToLocalDate(dateD);
		data.setDate(date); // NOPMD
	    }
	    // Erffnungkurs
	    if (values[1] instanceof Double) {
		data.setOpen((Double) values[1]);
	    }
	    // Tageshoch
	    if (values[2] instanceof Double) {
		data.setHigh((Double) values[2]);
	    }
	    // Tagestief
	    if (values[3] instanceof Double) {
		data.setLow((Double) values[3]);
	    }
	    // Schlusskurs (evtl angepasst)
	    if (values[4] instanceof Double) {
		data.setAdjClose((Double) values[4]);
	    }
	    // Volumen
	    if (values[5] instanceof Integer) {
		data.setVolume((Integer) values[5]);
	    } else {
		data.setVolume(0);
	    }
	    // Und nun zur Liste hinzufgen
	    dataList.add(data);
	}
	return dataList;
    }

    private LocalDate doubleToLocalDate(final Double date) {
	final Long dateLong = date.longValue();
	return Instant.ofEpochMilli(dateLong).atZone(ZoneId.systemDefault()).toLocalDate();
    }
}
