/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.boerseffm.json;

import java.util.List;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

/**
 * Das Instrument Objekt der FFM (warum auch immer das Instrument heit???
 *
 * @author Frank Mnster
 *
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "identifier", "hasPreviousDayClose", "hasOHLC", "data" })
public class Instrument {

    /**
     * Der ID String
     */
    @JsonProperty("identifier")
    private String identifier;

    /**
     * Gibt es den Schlusskurs vom Handelstag davor
     */
    @JsonProperty("hasPreviousDayClose")
    private Boolean hasPreviousDayClose; // NOPMD

    /**
     * OLHC Werte vorhanden
     */
    @JsonProperty("hasOHLC")
    private Boolean hasOHLC;

    /**
     * Und die Daten
     */
    @JsonProperty("data")
    @JsonDeserialize(using = DataDeserialiser.class)
    private List<ChartData> data;

    /**
     * Standardkonstruktor
     */
    public Instrument() {
	// Absichtlich leer
    }

    /**
     * Liste der vorhandenen Daten
     *
     * @return Liste der vorhandenen Daten
     */
    public List<ChartData> getData() {
	return this.data;
    }

    /**
     * Liefert <i>true</i>, wenn OHLC Daten vorhanden sind.
     *
     * @return OHLC Daten vorhanden
     */
    @JsonProperty("hasOHLC")
    public Boolean getHasOHLC() {
	return this.hasOHLC;
    }

    /**
     * Liefert <i>true</i>, wenn Schlusskursdaten vom letzten Handelstag
     * vorhanden sind.
     *
     * @return Schlusskursdaten vorhanden
     */
    @JsonProperty("hasPreviousDayClose")
    public Boolean getHasPreviousDayClose() {
	return this.hasPreviousDayClose;
    }

    /**
     * Liefert den ID-String
     *
     * @return den ID-String
     */
    @JsonProperty("identifier")
    public String getIdentifier() {
	return this.identifier;
    }

}
