/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.boerseffm.json;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;

/**
 * JSON Beschreibung des InstrumentType.
 *
 * @author Frank Mnster
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "InstrumentID", "Name", "Category", "WKN", "ISIN", "Valor", "Symbol", "LocalIdentifier", "Bias",
	"InstrumentType", "IssuerCompany", "UnderlyingWKN", "UnderlyingISIN", "UnderlyingName", "UrlName", "IsETF",
	"TKExchangeId", "TKCurrencyId", "ImageUrl", "Text", "PublishDate", "PublishDateAsString" })
@SuppressWarnings("PMD.TooManyFields")
public class InstrumentType {

    /**
     * Brese FFM BIAS der Wertpapieres
     */
    @JsonProperty("Bias")
    private Integer bias;

    /**
     * Kategorie
     */
    @JsonProperty("Category")
    private String category;

    /**
     * Bild URL
     */
    @JsonProperty("ImageUrl")
    private Object imageUrl;

    /**
     * Instrument-ID
     */
    @JsonProperty("InstrumentID")
    private Integer instrumentID;

    /**
     * Instrument-Typ
     */
    @JsonProperty("InstrumentType")
    private String instType; // NOPMD

    /**
     * ETF
     */
    @JsonProperty("IsETF")
    private Boolean isETF;

    /**
     * ISIN
     */
    @JsonProperty("ISIN")
    private String iSIN;

    /**
     * Herausgeber
     */
    @JsonProperty("IssuerCompany")
    private Object issuerCompany;

    /**
     * Lakale-ID
     */
    @JsonProperty("LocalIdentifier")
    private Object localIdentifier;

    /**
     * Name
     */
    @JsonProperty("Name")
    private String name;

    /**
     * Erscheinungsdatum
     */
    @JsonProperty("PublishDate")
    private String publishDate;

    /**
     * Erscheinungsdatum als Text
     */
    @JsonProperty("PublishDateAsString")
    private Object publishDateAsString; // NOPMD

    /**
     * Symbol
     */
    @JsonProperty("Symbol")
    private Object symbol;

    /**
     * Text???
     */
    @JsonProperty("Text")
    private Object text;

    /**
     * Whrungs-ID
     */
    @JsonProperty("TKCurrencyId")
    private Integer tKCurrencyId;

    /**
     * Wechselkurs-ID
     */
    @JsonProperty("TKExchangeId")
    private Integer tKExchangeId;

    /**
     * ISIN des zu grundeliegenden Papiers
     */
    @JsonProperty("UnderlyingISIN")
    private Object underlyingISIN;

    /**
     * Name des zu grundeliegenden Papiers
     */
    @JsonProperty("UnderlyingName")
    private Object underlyingName;

    /**
     * WKN des zu grundeliegenden Papiers
     */
    @JsonProperty("UnderlyingWKN")
    private Object underlyingWKN;

    /**
     * URL Name
     */
    @JsonProperty("UrlName")
    private String urlName;

    /**
     * Valor ???
     */
    @JsonProperty("Valor")
    private Integer valor;

    /**
     * WKN
     */
    @JsonProperty("WKN")
    private String wKN;

    /**
     * Standardkonstruktor
     */
    public InstrumentType() {
	// Absichtlich leer
    }

    /**
     * Liefert das BIAS
     *
     * @return das BIAS
     */
    @JsonProperty("Bias")
    public Integer getBias() {
	return this.bias;
    }

    /**
     * Liefert die Kategorie
     *
     * @return die Kategorie
     */
    @JsonProperty("Category")
    public String getCategory() {
	return this.category;
    }

    /**
     * Liefert die Bild-URL
     *
     * @return die Bild-URL
     */
    @JsonProperty("ImageUrl")
    public Object getImageUrl() {
	return this.imageUrl;
    }

    /**
     * Liefert die Instrument-ID
     *
     * @return die Instrument-ID
     */
    @JsonProperty("InstrumentID")
    public Integer getInstrumentID() {
	return this.instrumentID;
    }

    /**
     * Liefert den Instrument-Typ
     *
     * @return den Instrument-Typ
     */
    @JsonProperty("InstrumentType")
    public String getInstrumentType() {
	return this.instType;
    }

    /**
     * Ist EFT-Wert
     *
     * @return <i>true</i> wenn EFT
     */
    @JsonProperty("IsETF")
    public Boolean getIsETF() {
	return this.isETF;
    }

    /**
     * Liefert die ISIN
     *
     * @return die ISIN
     */
    @JsonProperty("ISIN")
    public String getISIN() {
	return this.iSIN;
    }

    /**
     * Liefert den Herausgeber
     *
     * @return den Herausgeber
     */
    @JsonProperty("IssuerCompany")
    public Object getIssuerCompany() {
	return this.issuerCompany;
    }

    /**
     * Liefert die lokale ID
     *
     * @return die lokale ID
     */
    @JsonProperty("LocalIdentifier")
    public Object getLocalIdentifier() {
	return this.localIdentifier;
    }

    /**
     * Liefert den Name
     *
     * @return den Name
     */
    @JsonProperty("Name")
    public String getName() {
	return this.name;
    }

    /**
     * Liefert das Erscheinungsdatum
     *
     * @return das Erscheinungsdatum
     */
    @JsonProperty("PublishDate")
    public String getPublishDate() {
	return this.publishDate;
    }

    /**
     * Liefert das Erscheinungsdatum als Text
     *
     * @return das Erscheinungsdatum als Text
     */
    @JsonProperty("PublishDateAsString")
    public Object getPublishDateAsString() {
	return this.publishDateAsString;
    }

    /**
     * Liefert das Symbol
     *
     * @return das Symbol
     */
    @JsonProperty("Symbol")
    public Object getSymbol() {
	return this.symbol;
    }

    /**
     * Liefert den Text (Beschreibung?)
     *
     * @return den Text (Beschreibung?)
     */
    @JsonProperty("Text")
    public Object getText() {
	return this.text;
    }

    /**
     * Liefert die Whrungs-ID
     *
     * @return die Whrungs-ID
     */
    @JsonProperty("TKCurrencyId")
    public Integer getTKCurrencyId() {
	return this.tKCurrencyId;
    }

    /**
     * Liefert die Wechselkurs-ID
     *
     * @return die Wechselkurs-ID
     */
    @JsonProperty("TKExchangeId")
    public Integer getTKExchangeId() {
	return this.tKExchangeId;
    }

    /**
     * Liefert die ISIN des zugrundeliegenden Papiers
     *
     * @return die ISIN des zugrundeliegenden Papiers
     */
    @JsonProperty("UnderlyingISIN")
    public Object getUnderlyingISIN() {
	return this.underlyingISIN;
    }

    /**
     * Liefert den Namen des zugrundeliegenden Papiers
     *
     * @return den Namen des zugrundeliegenden Papiers
     */
    @JsonProperty("UnderlyingName")
    public Object getUnderlyingName() {
	return this.underlyingName;
    }

    /**
     * Liefert die WKN des zugrundeliegenden Papiers
     *
     * @return die WKN des zugrundeliegenden Papiers
     */
    @JsonProperty("UnderlyingWKN")
    public Object getUnderlyingWKN() {
	return this.underlyingWKN;
    }

    /**
     * Liefert die URL
     *
     * @return die URL
     */
    @JsonProperty("UrlName")
    public String getUrlName() {
	return this.urlName;
    }

    /**
     * Liefert das Valor
     *
     * @return das Valor
     */
    @JsonProperty("Valor")
    public Integer getValor() {
	return this.valor;
    }

    /**
     * Liefert die WKN
     *
     * @return die WKN
     */
    @JsonProperty("WKN")
    public String getWKN() {
	return this.wKN;
    }
}
