/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.boerseffm.json;

import java.util.ArrayList;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;

/**
 * JSON Antwort eine Such-Anfrage.
 *
 * @author Frank Mnster
 *
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "Indizes", "Aktien", "Fonds", "Rohstoffe", "Anleihen" })
public class SearchResponse {

    /**
     * Liste der gefundenen Indizes
     */
    @JsonProperty("Indizes")
    private List<InstrumentType> indizes;
    /**
     * Liste der gefundenen Aktien
     */
    @JsonProperty("Aktien")
    private List<InstrumentType> aktien;
    /**
     * Liste der gefundenen Fonds
     */
    @JsonProperty("Fonds")
    private List<InstrumentType> fonds;
    /**
     * Liste der gefundenen Rohstoffe
     */
    @JsonProperty("Rohstoffe")
    private List<InstrumentType> rohstoffe;
    /**
     * Liste der gefundenen Anleihen
     */
    @JsonProperty("Anleihen")
    private List<InstrumentType> anleihen;

    /**
     * Standardkonstruktor
     */
    public SearchResponse() {
	// absichtlich leer
    }

    /**
     * Liefert die gefundenen Aktien
     *
     * @return die gefundenen Aktien
     */
    @JsonProperty("Aktien")
    public List<InstrumentType> getAktien() {
	return this.aktien;
    }

    /**
     * Liefert die gefundenen Anleihen
     *
     * @return die gefundenen Anleihen
     */
    @JsonProperty("Anleihen")
    public List<InstrumentType> getAnleihen() {
	return this.anleihen;
    }

    /**
     * Liefert die gefundenen Fonds
     *
     * @return die gefundenen Fonds
     */
    @JsonProperty("Fonds")
    public List<InstrumentType> getFonds() {
	return this.fonds;
    }

    /**
     * Liefert die gefundenen Indizes
     *
     * @return die gefundenen Indizes
     */
    @JsonProperty("Indizes")
    public List<InstrumentType> getIndizes() {
	return this.indizes;
    }

    /**
     * Liefert alle gefundenen Instruments
     *
     * @return eine Liste mit allen gefundenen Instruments
     */
    public List<InstrumentType> getInstruments() {

	final List<InstrumentType> allInstruments = new ArrayList<>();

	if (this.indizes != null) {
	    allInstruments.addAll(this.indizes);
	}
	if (this.aktien != null) {
	    allInstruments.addAll(this.aktien);
	}
	if (this.fonds != null) {
	    allInstruments.addAll(this.fonds);
	}
	if (this.rohstoffe != null) {
	    allInstruments.addAll(this.rohstoffe);
	}
	if (this.anleihen != null) {
	    allInstruments.addAll(this.anleihen);
	}
	return allInstruments;
    }

    /**
     * Liefert die gefundenen Rohstoffe
     *
     * @return die gefundenen Rhostoffe
     */
    @JsonProperty("Rohstoffe")
    public List<InstrumentType> getRohstoffe() {
	return this.rohstoffe;
    }

}
