/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.currate;

/**
 * CurRate service error
 *
 * @author Frank Mnster
 */
public enum CurRateServiceErrors {

    /**
     * Operation not supported
     */
    OPERATION_NOT_SUPPORTED(100),

    /**
     * Date parameter missing or wrong
     */
    PARAM_DATE(101),

    /**
     * Currency parameter missing or wrong
     */
    PARAM_CURRENCY(102),

    /**
     * Currency is not supported
     */
    CURRENCY_NOT_SUPPORTED(103),

    /**
     * No rate found for the given date.
     */
    NO_RATE_FOR_DATE(104),

    /**
     * The date is before 4th of January 1999
     */
    DATE_TOO_OLD(105),

    /**
     * The SQL query failed
     */
    SQL_QUERY_FAILED(201),

    /**
     * The service cannot connect to the database
     */
    DB_NOT_CONNECTED(200);

    /**
     * The error code
     */
    private int errorCode;

    /**
     * Returns the CurRateServiceErrors for the given code, or null of code is supported.
     *
     * @param code
     * @return the CurRateServiceErrors
     */
    public static CurRateServiceErrors getError(final int code) {

        CurRateServiceErrors serviceError = null;

        for (final CurRateServiceErrors error : CurRateServiceErrors.values()) {
            if (error.getErrorCode() == code) {
                serviceError = error;
                break;
            }
        }

        return serviceError;

    }

    CurRateServiceErrors(final int errorCode) {

        this.errorCode = errorCode;

    }

    /**
     * Returns the error code
     *
     * @return the error code
     */
    public int getErrorCode() {

        return this.errorCode;
    }

}
