/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.currate.json;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;

/**
 * Represent an error response of the CurRate service.
 *
 * @author Frank Mnster
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ CurRateError.JSON_ERROR_CODE, CurRateError.JSON_MESSAGE, CurRateError.JSON_METRICS })
public class CurRateError {

    /**
     * JSON metrics property
     */
    protected static final String JSON_METRICS = "metrics"; //$NON-NLS-1$

    /**
     * JSON message property
     */
    protected static final String JSON_MESSAGE = "message"; //$NON-NLS-1$

    /**
     * JSON error_code property
     */
    protected static final String JSON_ERROR_CODE = "error_code"; //$NON-NLS-1$

    /**
     * The error code.
     */
    @JsonProperty(CurRateError.JSON_ERROR_CODE)
    private int errorCode;

    /**
     * Readable error message
     */
    @JsonProperty(CurRateError.JSON_MESSAGE)
    private String message;

    /**
     * Server metrics
     */
    @JsonProperty(CurRateError.JSON_METRICS)
    private Metrics metrics;

    /**
     * Standardkonstruktor
     */
    public CurRateError() {
	// Absichtlich leer
    }

    /**
     * Returns the error code
     *
     * @return the error code
     */
    @JsonProperty(CurRateError.JSON_ERROR_CODE)
    public int getErrorCode() {

	return this.errorCode;
    }

    /**
     * Returns the readable error message
     *
     * @return the readable error message
     */
    @JsonProperty(CurRateError.JSON_MESSAGE)
    public String getMessage() {

	return this.message;
    }

    /**
     * Returns the server metrics
     *
     * @return the server metrics
     */
    @JsonProperty(CurRateError.JSON_METRICS)
    public Metrics getMetrics() {

	return this.metrics;
    }

    /**
     * Sets the error code
     *
     * @param errorCode
     */
    @JsonProperty(CurRateError.JSON_ERROR_CODE)
    public void setErrorCode(final int errorCode) {

	this.errorCode = errorCode;
    }

    /**
     * Sets the readable error message
     *
     * @param message
     */
    @JsonProperty(CurRateError.JSON_MESSAGE)
    public void setMessage(final String message) {

	this.message = message;
    }

    /**
     * Sets the server metrics
     *
     * @param metrics
     */
    @JsonProperty(CurRateError.JSON_METRICS)
    public void setMetrics(final Metrics metrics) {

	this.metrics = metrics;
    }

}
