/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.currate.json;

import java.util.List;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;

/**
 * Represent successful CurRate service response.
 *
 * @author Frank Mnster
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ CurRates.JSON_BASE_CUR, CurRates.JSON_RATES, CurRates.JSON_METRICS })
public class CurRates {

    /**
     * JSON metrics property
     */
    protected static final String JSON_METRICS = "metrics"; //$NON-NLS-1$

    /**
     * JSON rates property
     */
    protected static final String JSON_RATES = "rates"; //$NON-NLS-1$

    /**
     * JSON base_currency property
     */
    protected static final String JSON_BASE_CUR = "base_currency"; //$NON-NLS-1$

    /**
     * The base currency of all rates in this answer
     */
    @JsonProperty(CurRates.JSON_BASE_CUR)
    private String baseCurrency;

    /**
     * The list of rates
     */
    @JsonProperty(CurRates.JSON_RATES)
    private List<Rate> rates;

    /**
     * The server metrics
     */
    @JsonProperty(CurRates.JSON_METRICS)
    private Metrics metrics;

    /**
     * Standardkonstruktor
     */
    public CurRates() {
	// Absichtlich leer
    }

    /**
     * Returns the base currency
     *
     * @return the base currency
     */
    @JsonProperty(CurRates.JSON_BASE_CUR)
    public String getBaseCurrency() {

	return this.baseCurrency;
    }

    /**
     * Returns the server metrics
     *
     * @return the server metrics
     */
    @JsonProperty(CurRates.JSON_METRICS)
    public Metrics getMetrics() {

	return this.metrics;
    }

    /**
     * Returns a list of rates
     *
     * @return a list of rates
     */
    @JsonProperty(CurRates.JSON_RATES)
    public List<Rate> getRates() {

	return this.rates;
    }

    /**
     * Sets the base currency
     *
     * @param baseCurrency
     */
    @JsonProperty(CurRates.JSON_BASE_CUR)
    public void setBaseCurrency(final String baseCurrency) {

	this.baseCurrency = baseCurrency;
    }

    /**
     * Sets the server metrics
     *
     * @param metrics
     */
    @JsonProperty(CurRates.JSON_METRICS)
    public void setMetrics(final Metrics metrics) {

	this.metrics = metrics;
    }

    /**
     * The the list with rates
     *
     * @param rates
     *            List of rate
     */
    @JsonProperty(CurRates.JSON_RATES)
    public void setRates(final List<Rate> rates) {

	this.rates = rates;
    }

}
