/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.currate.json;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;

/**
 * Represent the metrics in CurRate service response.
 *
 * @author Frank Mnster
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ Metrics.JSON_VERSION, Metrics.JSON_SERVER_TIME })
public class Metrics {

    /**
     * JSON server_time property
     */
    protected static final String JSON_SERVER_TIME = "server_time"; //$NON-NLS-1$

    /**
     * JSON version property
     */
    protected static final String JSON_VERSION = "version"; //$NON-NLS-1$

    /**
     * Server version
     */
    @JsonProperty(Metrics.JSON_VERSION)
    private String version;

    /**
     * Server elapsed time for request
     */
    @JsonProperty(Metrics.JSON_SERVER_TIME)
    private String serverTime;

    /**
     * Standardkonstruktor
     */
    public Metrics() {
	// Absichtlich leer
    }

    /**
     * Return the server elapsed time for request in seconds
     *
     * @return the server elapsed time for request
     */
    @JsonProperty(Metrics.JSON_SERVER_TIME)
    public String getServerTime() {

	return this.serverTime;
    }

    /**
     * Return the server version
     *
     * @return the server version
     */
    @JsonProperty(Metrics.JSON_VERSION)
    public String getVersion() {

	return this.version;
    }

    /**
     * Sets the elapsed time
     *
     * @param serverTime
     */
    @JsonProperty(Metrics.JSON_SERVER_TIME)
    public void setServerTime(final String serverTime) {

	this.serverTime = serverTime;
    }

    /**
     * Sets the server version
     *
     * @param version
     */
    @JsonProperty(Metrics.JSON_VERSION)
    public void setVersion(final String version) {

	this.version = version;
    }

}
