/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.currate.json;

import java.math.BigDecimal;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;

/**
 * Represent a rateValue in CurRate service response.
 *
 * @author Frank Mnster
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ Rate.JSON_DATE, Rate.JSON_CURRENCY, Rate.JSON_RATE, Rate.JSON_DATE_ADAPTED })
@SuppressWarnings("PMD.ShortClassName")
public class Rate {

    /**
     * JSON date_adapted property
     */
    protected static final String JSON_DATE_ADAPTED = "date_adapted"; //$NON-NLS-1$

    /**
     * JSON rateValue property
     */
    protected static final String JSON_RATE = "rate"; //$NON-NLS-1$

    /**
     * JSON currency property
     */
    protected static final String JSON_CURRENCY = "currency"; //$NON-NLS-1$

    /**
     * JSON date property
     */
    protected static final String JSON_DATE = "date"; //$NON-NLS-1$

    /**
     * Date of the rateValue
     */
    @JsonProperty(Rate.JSON_DATE)
    private String date;

    /**
     * Currency of the rateValue
     */
    @JsonProperty(Rate.JSON_CURRENCY)
    private String currency;

    /**
     * The rateValue vale of the rateValue
     */
    @JsonProperty(Rate.JSON_RATE)
    private BigDecimal rateValue;

    /**
     * Is the date of teh request or teh next older one
     */
    @JsonProperty(Rate.JSON_DATE_ADAPTED)
    private Boolean dateAdapted;

    /**
     * Standardkonstruktor
     */
    public Rate() {
	// Absichtlich leer
    }

    /**
     * Returns the currency
     *
     * @return the currency
     */
    @JsonProperty(Rate.JSON_CURRENCY)
    public String getCurrency() {

	return this.currency;
    }

    /**
     * Returns the date
     *
     * @return the date
     */

    @JsonProperty(Rate.JSON_DATE)
    public String getDate() {

	return this.date;
    }

    /**
     * Returns if that date has been adapted to the next older one
     *
     * @return <b>true</b> if that date has been adapted to the next older one,
     *         <b>false</b> else
     */
    @JsonProperty(Rate.JSON_DATE_ADAPTED)
    public Boolean getDateAdapted() {

	return this.dateAdapted;
    }

    /**
     * Returns the rateValue value
     *
     * @return the rateValue value
     */

    @JsonProperty(Rate.JSON_RATE)
    public BigDecimal getRateValue() {

	return this.rateValue;
    }

    /**
     * Sets the currency
     *
     * @param currency
     */
    @JsonProperty(Rate.JSON_CURRENCY)
    public void setCurrency(final String currency) {

	this.currency = currency;
    }

    /**
     * Sets the date
     *
     * @param date
     */
    @JsonProperty(Rate.JSON_DATE)
    public void setDate(final String date) {

	this.date = date;
    }

    /**
     * Sets if that date has been adapted to the next older one
     *
     * @param dateAdapted
     */
    @JsonProperty(Rate.JSON_DATE_ADAPTED)
    public void setDateAdapted(final Boolean dateAdapted) {

	this.dateAdapted = dateAdapted;
    }

    /**
     * Sets the rateValue vale
     *
     * @param rateValue
     */
    @JsonProperty(Rate.JSON_RATE)
    public void setRateValue(final BigDecimal rateValue) {

	this.rateValue = rateValue;
    }

}
