/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.currencyexchangerate;

import java.math.BigDecimal;
import java.util.Date;

/**
 * Bean fr Wechselkurse
 *
 * @author Frank Mnster
 */
public class ExchangeRate {

    /**
     * Das Wechselkurs Datum
     */
    private Date date;

    /**
     * Der Wechselkurs
     */
    private BigDecimal rate;

    /**
     * Exaktes Datum
     */
    private boolean excatDate;

    /**
     * Standardkonstruktor
     *
     * @param rate
     * @param date
     */
    public ExchangeRate(final Date date, final BigDecimal rate) {
	this.date = date;
	this.rate = rate;
    }

    /**
     * Liefert das Datum des Wechselkurses
     *
     * @return das Datum des Wechselkurses
     */
    public Date getDate() {

	return this.date;
    }

    /**
     * Liefert den Wechselkurs
     *
     * @return der Wechselkurs
     */
    public BigDecimal getRate() {

	return this.rate;
    }

    /**
     * Liefert Angaben darber, ob das Wechselkursdatum dem angefragtem Datum
     * entspricht.
     *
     * @return <i>true</i>, falls das Wechselkursdatum dem angefragtem Datum
     *         entspricht, <i>false</i> sonst.
     */
    public boolean isExcatDate() {

	return this.excatDate;
    }

    /**
     * Setzt das Datum des Wechselkurses
     *
     * @param date
     *            das zu setzende Datum des Wechselkurses
     */
    public void setDate(final Date date) {

	this.date = date;
    }

    /**
     * Setzt die Angaben darber, ob das Wechselkursdatum dem angefragtem Datum
     * entspricht.
     *
     * @param excatDate
     *            <i>true</i>, falls das Wechselkursdatum dem angefragtem Datum
     *            entspricht, <i>false</i> sonst.
     */
    public void setExcatDate(final boolean excatDate) {

	this.excatDate = excatDate;
    }

    /**
     * Setzt den Wechselkurs
     *
     * @param rate
     *            der zu setzende Wechselkurs
     */
    public void setRate(final BigDecimal rate) {

	this.rate = rate;
    }

}
