/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.quandl;

import org.apache.commons.lang.StringUtils;

import com.jimmoores.quandl.RetryPolicy;
import com.jimmoores.quandl.SessionOptions;
import com.jimmoores.quandl.classic.ClassicQuandlSession;

/**
 * @author Frank Mnster
 *
 */
public final class MyQuandlSession {

    /**
     * Unsere Instanz
     */
    private static MyQuandlSession instance;

    /**
     * Die zugrundeliegende Quandl4J session
     */
    private ClassicQuandlSession session;

    /**
     * Der API-KEY
     */
    private String authToken;

    /**
     * Liefert die QuandlSearch Instanz.
     *
     * @param authToken
     *            Der API-KEY, kann NULL sein.
     *
     * @return die QuandlSearch Instanz.
     */
    public static ClassicQuandlSession getSession(final String authToken) {
	synchronized (MyQuandlSession.class) {
	    if (MyQuandlSession.instance == null || !authToken.equals(MyQuandlSession.instance.authToken)) {
		MyQuandlSession.instance = new MyQuandlSession(authToken);
	    }
	    MyQuandlSession.instance.authToken = authToken;
	}
	return MyQuandlSession.instance.session;
    }

    /**
     * privater Standardkonstruktor
     *
     * @param authToken
     *            Der API-KEY, kann NULL sein.
     */
    private MyQuandlSession(final String authToken) {

	SessionOptions options;
	if (StringUtils.isEmpty(authToken)) {
	    options = SessionOptions.Builder.withoutAuthToken().withRetryPolicy(RetryPolicy.createNoRetryPolicy())
		    .build();
	    this.session = ClassicQuandlSession.create(options);
	} else {
	    options = SessionOptions.Builder.withAuthToken(authToken).withRetryPolicy(RetryPolicy.createNoRetryPolicy())
		    .build();
	    this.session = ClassicQuandlSession.create(options);
	}
    }
}
