/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.quandl;

import java.time.LocalDate;
import java.time.OffsetDateTime;

import de.frankmuenster.jameica.finanzen.StockInfoBean;

/**
 * Quandl Wertpapier Bean
 *
 * @author Frank Mnster
 *
 */
public class QuandlStockInfoBean extends StockInfoBean {

    /**
     * Quandl Code
     */
    private String code;

    /**
     * ltester Wert
     */
    private LocalDate oldestAvailable;

    /**
     * Aktualisiert
     */
    private OffsetDateTime refeshedAt;

    /**
     * Jngster Wert
     */
    private LocalDate newestAvailable;

    /**
     * Standartkonstruktor
     */
    public QuandlStockInfoBean() {
	this(null, null, null, null);
    }

    /**
     * Konstruktor mit Angabe aller Felder.
     *
     * @param code
     *            der Quandl Code
     * @param name
     *            der Wertpapiername
     * @param exchange
     *            Der B&ouml;rsenplatz
     * @param type
     *            Der Wertpapiertyp
     */
    public QuandlStockInfoBean(final String code, final String name, final String exchange, final String type) {
	super(name, exchange, type, null);
	this.code = code;
    }

    /**
     * Liefert den Quandl Code
     *
     * @return den Quandl Code
     */
    public String getCode() {
	return this.code;
    }

    /**
     * @return the newestAvailable
     */
    public LocalDate getNewestAvailable() {
	return this.newestAvailable;
    }

    /**
     * @return the oldestAvailable
     */
    public LocalDate getOldestAvailable() {
	return this.oldestAvailable;
    }

    /**
     * @return the refeshedAt
     */
    public OffsetDateTime getRefeshedAt() {
	return this.refeshedAt;
    }

    /**
     * Setzt den Quandl Code
     *
     * @param code
     *            den Quandl Code
     */
    public void setCode(final String code) {
	this.code = code;
    }

    /**
     * @param newestAvailable
     *            the newestAvailable to set
     */
    public void setNewestAvailable(final LocalDate newestAvailable) {
	this.newestAvailable = newestAvailable;
    }

    /**
     * @param oldestAvailable
     *            the oldestAvailable to set
     */
    public void setOldestAvailable(final LocalDate oldestAvailable) {
	this.oldestAvailable = oldestAvailable;
    }

    /**
     * @param offsetDateTime
     *            the refeshedAt to set
     */
    public void setRefeshedAt(final OffsetDateTime offsetDateTime) {
	this.refeshedAt = offsetDateTime;
    }

    /**
     * @see Object#toString()
     */
    @Override
    public String toString() {
	final StringBuilder builder = new StringBuilder(super.toString());
	if (this.code != null) {
	    builder.append('\t');
	    builder.append(this.code);
	}
	return builder.toString();
    }

}
