/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.finanzen.quandl;

import java.util.List;

import com.jimmoores.quandl.DataSetRequest;
import com.jimmoores.quandl.TabularResult;
import com.jimmoores.quandl.classic.ClassicQuandlSession;
import com.jimmoores.quandl.util.QuandlRuntimeException;

import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.HistoryValueBean;
import de.frankmuenster.jameica.finanzen.WertpapierHistory;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 */
public class QuandlWertpapierHistory implements WertpapierHistory {

    /**
     * Das Auth Token
     */
    private final String authToken;

    /**
     * Der Quandl code
     */
    private final String code;

    /**
     * Konstruktor mit Angabe des Names
     *
     * @param code
     * @param authToken
     */
    public QuandlWertpapierHistory(final String code, final String authToken) {
	this.code = code;
	this.authToken = authToken;
    }

    /**
     * @see de.frankmuenster.jameica.finanzen.WertpapierHistory#getSourceName()
     */
    @Override
    public String getSourceName() {
	return "Quandle: " + this.code; //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.jameica.finanzen.WertpapierHistory#getValues()
     */
    @Override
    public List<HistoryValueBean> getValues() throws FinanceException {
	final ClassicQuandlSession session = MyQuandlSession.getSession(this.authToken);
	TabularResult result = null;
	try {
	    result = session.getDataSet(DataSetRequest.Builder.of(this.code).build());
	} catch (final QuandlRuntimeException e) {
	    Logger.error(e.getLocalizedMessage());
	}
	return HistoryReader.resultReader(this.code, result);
    }
}
