/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.action;

import java.rmi.RemoteException;

import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Enternt ein Wertpapier aus Datenbank
 *
 * @author munstfr
 * @version 1.0
 */
public class WertpapierDeleteAction implements Action {

    /**
     * Standart Konstruktor
     */
    public WertpapierDeleteAction() {
	// absichtlich leer
    }

    /**
     * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
     */
    @Override
    public void handleAction(final Object context) throws ApplicationException {

	// check if the context is a project
	if (!(context instanceof Wertpapier)) {
	    throw new ApplicationException(I18N.tr("WertpapierDeleteAction.chooseStock")); //$NON-NLS-1$
	}

	final Wertpapier wertpapier = (Wertpapier) context;

	try {
	    // Sicherheitsabfrage
	    final String question = I18N.tr("WertpapierDeleteAction.reallyDelete"); //$NON-NLS-1$
	    if (Application.getCallback().askUser(question)) {

		this.deleteOrders(wertpapier);
		wertpapier.delete();

		// Send Status update message
		Application.getMessagingFactory()
			.sendMessage(new StatusBarMessage(I18N.tr("WertpapierDeleteAction.success"), //$NON-NLS-1$
				StatusBarMessage.TYPE_SUCCESS));
	    }

	} catch (final Exception e) { // NOPMD
	    Logger.error(I18N.tr("WertpapierDeleteAction.error"), e); //$NON-NLS-1$
	    throw new ApplicationException(I18N.tr("WertpapierDeleteAction.error"), e); //$NON-NLS-1$
	}
    }

    /**
     * Deletes the orders of a stock.
     *
     * @param wertpapier
     * @throws RemoteException
     * @throws ApplicationException
     */
    private void deleteOrders(final Wertpapier wertpapier) throws RemoteException, ApplicationException {

	final DBIterator<OrderHistory> orders = wertpapier.getOrderHistory();

	while (orders.hasNext()) {
	    final OrderHistory orderHistory = orders.next();
	    orderHistory.delete();
	}

    }

}
