/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.controller;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.finanzen.WertpapierHistoryBean;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.FfmStockInfoBean;
import de.frankmuenster.jameica.depot.gui.input.PeriodChooserInput;
import de.frankmuenster.jameica.depot.gui.input.ZeitauswahlSchieberInput;
import de.frankmuenster.jameica.depot.gui.input.ZeitauswahlSchieberInputListener;
import de.frankmuenster.jameica.depot.gui.parts.AnalyseButtonPart;
import de.frankmuenster.jameica.depot.gui.parts.DiagrammAnzeigeAuswahlPart;
import de.frankmuenster.jameica.depot.gui.parts.KursHistoryGrafikPart;
import de.frankmuenster.jameica.depot.internal.objects.DateRange;
import de.frankmuenster.jameica.depot.kursanalyse.Analysen;
import de.frankmuenster.jameica.depot.kursanalyse.IKursAnalyse;
import de.frankmuenster.jameica.depot.kursanalyse.MethodeAddRemoveListener;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.gui.input.RadioInput;
import de.willuhn.logging.Logger;

/**
 * Kontrolleur f&uuml;r die Analysen-Ansicht.
 *
 * @author Frank Mnster
 */
public class AnalysenKontroller extends AbstractControl implements MethodeAddRemoveListener {

    /**
     * Das Wertpapier zur Analyse
     */
    private transient Wertpapier wertpapier;

    /**
     * Die Kurshistorien Grafik
     */
    private transient KursHistoryGrafikPart grafik;

    /**
     * Der Zeitauswahlschieber
     */
    private transient ZeitauswahlSchieberInput zeitAuswahl;

    /**
     * Diagramm-Anzeige Werkzeug
     */
    private transient DiagrammAnzeigeAuswahlPart grafikWerkzeuge;

    /**
     * Zeitauswahl-Kn6ouml;pfe
     */
    private transient PeriodChooserInput zeitAuswahlBtns;

    /**
     * Checkbox DAX anzeigen
     */
    private transient CheckboxInput showDax;

    /**
     * Checkbox Low-High anzeigen
     */
    private transient CheckboxInput showLowHigh;

    /**
     * Wertpapier History Werte
     */
    private transient WertpapierHistoryBean wpHistoryBean;

    /**
     * Dax Histroy Werte
     */
    private transient WertpapierHistoryBean daxHistory;

    /**
     * Analysemethoden Kn&ouml;pfe
     */
    private transient AnalyseButtonPart analyseButtons;

    /**
     * Creates a new controller for to the given view.
     *
     * @param view
     * @throws RemoteException
     */
    public AnalysenKontroller(final AbstractView view) throws RemoteException {
	super(view);
	this.loadData();
	this.loadDaxData();
    }

    /**
     * Return the AnalyseButtonPart to displaying the available AnalysButtons of
     * this controller.
     *
     * @return AnalyseButtonPart
     */
    public AnalyseButtonPart getAnalyseButtons() {

	if (this.analyseButtons == null) {
	    this.analyseButtons = new AnalyseButtonPart(this);
	}
	return this.analyseButtons;
    }

    /**
     * Returns a KursHistoryGrafikPart of this controller.
     *
     * @return das KursHistoryGrafikPart
     * @throws RemoteException
     */
    public KursHistoryGrafikPart getGrafik() throws RemoteException {

	if (this.grafik == null) {
	    this.grafik = new KursHistoryGrafikPart(this.getAktie().getName(), this);
	    this.grafik.setKursDaxDaten(this.wpHistoryBean, this.daxHistory);
	    this.grafik.setShowDax(false);
	    this.grafik.setShowAsCandleStick(false);
	    if (this.zeitAuswahl == null) {
		this.grafik.setRange(this.wpHistoryBean.getPeriod());
	    } else {
		this.grafik.setRange(new DateRange(this.zeitAuswahl.getLower(), this.zeitAuswahl.getUpper()));
	    }
	}
	return this.grafik;
    }

    /**
     * Returns the DiagrammAnzeigeAuswahlPart of this controller
     *
     * @return das DiagrammAnzeigeAuswahlPart
     */
    public DiagrammAnzeigeAuswahlPart getGrafikWerkzeuge() {

	if (this.grafikWerkzeuge == null) {
	    this.grafikWerkzeuge = new DiagrammAnzeigeAuswahlPart();
	    this.grafikWerkzeuge.setDax(this.getShowDax());
	    this.grafikWerkzeuge.setPeriod((PeriodChooserInput) this.getZeitauswahlButtons());
	    this.grafikWerkzeuge.setHighLow(this.getShowLowHigh());
	}
	return this.grafikWerkzeuge;
    }

    /**
     * Returns the ZeitauswahlSchieberInput of this controller.
     *
     * @return das ZeitauswahlSchieberInput
     */
    public ZeitauswahlSchieberInput getZeitauswahl() {

	if (this.zeitAuswahl == null) {
	    this.zeitAuswahl = new ZeitauswahlSchieberInput();
	    // Zeitachse setzten und das letzte Jahr auswhlen
	    this.zeitAuswahl.setScaling(this.wpHistoryBean.getPeriod().getStartDate(),
		    this.wpHistoryBean.getPeriod().getEndDate(), 1, 10);
	    this.zeitAuswahl.selectLastYear();
	    // Platz fr den Kommentar schaffen
	    this.zeitAuswahl.setComment(""); //$NON-NLS-1$
	    this.zeitAuswahl.addListener(new ZeitAuswahlListener());
	}
	return this.zeitAuswahl;
    }

    /**
     * @see MethodeAddRemoveListener#notifyAddMethode(IKursAnalyse)
     */
    @Override
    @SuppressWarnings("unchecked")
    public void notifyAddMethode(final IKursAnalyse<?, ?> methode) {

	Logger.debug(I18N.tr("AnalysenKontroller.mothodAdded", methode.getLegendTitel())); //$NON-NLS-1$
	methode.setWertpapierHistory(this.wpHistoryBean);
	Analysen.getInstance().add((IKursAnalyse<Object, Object>) methode);
	this.analyseButtons.refresh();
	this.grafik.refresh();
	this.view.getParent().layout();
    }

    /**
     * @see MethodeAddRemoveListener#notifyRemoveAll()
     */
    @Override
    public void notifyRemoveAll() {

	Analysen.getInstance().clear();
	this.analyseButtons.refresh();
	this.grafik.refresh();
	this.view.getParent().layout();
    }

    /**
     * @see MethodeAddRemoveListener#notifyRemoveMethode(IKursAnalyse)
     */
    @Override
    public void notifyRemoveMethode(final IKursAnalyse<?, ?> methode) {

	Analysen.getInstance().remove(methode);
	this.analyseButtons.refresh();
	this.grafik.refresh();
	this.view.getParent().layout();
    }

    /**
     * Gibt das aktuelle Wertpapier zur&uuml;ck.
     *
     * @return das Wertpapier.
     */
    private Wertpapier getAktie() {

	if (this.wertpapier == null) {
	    this.wertpapier = (Wertpapier) this.getCurrentObject();
	}
	return this.wertpapier;
    }

    private CheckboxInput getShowDax() {

	if (this.showDax == null) {
	    this.showDax = new CheckboxInput(false);
	    this.showDax.setName("DAX"); //$NON-NLS-1$
	    this.showDax.setEnabled(true);
	    this.showDax.addListener(new Listener() {

		/**
		 * @see org.eclipse.swt.widgets.Listener#handleEvent
		 */
		@Override
		public void handleEvent(final Event event) {

		    Logger.debug("DAX:" + AnalysenKontroller.this.showDax.getValue()); //$NON-NLS-1$
		    if (event.type == SWT.Selection) {
			if ((Boolean) AnalysenKontroller.this.showDax.getValue()) {
			    AnalysenKontroller.this.showLowHigh.setValue(false);
			}
			AnalysenKontroller.this.grafik.setShowDax((Boolean) AnalysenKontroller.this.showDax.getValue());
			AnalysenKontroller.this.grafik
				.setShowAsCandleStick((Boolean) AnalysenKontroller.this.showLowHigh.getValue());
			AnalysenKontroller.this.grafik.refresh();
		    }
		}
	    });
	}
	// if no DAX data disbale checkbox
	if (this.daxHistory == null) {
	    this.showDax.setEnabled(false);
	}
	return this.showDax;
    }

    private CheckboxInput getShowLowHigh() {

	if (this.showLowHigh == null) {
	    this.showLowHigh = new CheckboxInput(false);
	    this.showLowHigh.setName("LowHigh"); //$NON-NLS-1$
	    this.showLowHigh.addListener(new Listener() {

		/**
		 * @see org.eclipse.swt.widgets.Listener#handleEvent
		 */
		@Override
		public void handleEvent(final Event event) {

		    Logger.debug("LowHigh:" + AnalysenKontroller.this.showLowHigh.getValue()); //$NON-NLS-1$
		    if (event.type == SWT.Selection) {
			if ((Boolean) AnalysenKontroller.this.showLowHigh.getValue()) {
			    AnalysenKontroller.this.showDax.setValue(false);
			}
			AnalysenKontroller.this.grafik.setShowDax((Boolean) AnalysenKontroller.this.showDax.getValue());
			AnalysenKontroller.this.grafik
				.setShowAsCandleStick((Boolean) AnalysenKontroller.this.showLowHigh.getValue());
			AnalysenKontroller.this.grafik.refresh();
		    }
		}
	    });
	}
	// If data source is ARD-Boerse, disable Checkbox
	if ("Brese ARD".equals(this.wpHistoryBean.getDataSource())) { //$NON-NLS-1$
	    this.showLowHigh.setEnabled(false);
	}
	return this.showLowHigh;
    }

    private Part getZeitauswahlButtons() {

	if (this.zeitAuswahlBtns == null) {
	    this.zeitAuswahlBtns = new PeriodChooserInput();
	    this.zeitAuswahlBtns.addListener(new ZeitAuswahlButtonListener());
	}
	return this.zeitAuswahlBtns;
    }

    private void loadData() throws RemoteException {

	if (StringUtils.isEmpty(this.getAktie().getName())) {
	    throw new RemoteException(I18N.tr("AnalysenKontroller.symbolIsNull")); //$NON-NLS-1$
	}
	try {
	    final FfmStockInfoBean ffmInfoBean = new FfmStockInfoBean(this.getAktie().getFfmInstrument(),
		    this.getAktie().getFfmName(), null, null);
	    this.wpHistoryBean = new WertpapierHistoryBean(this.getAktie().getArdId(), this.getAktie().getQuandlCode(),
		    ffmInfoBean);
	    // If data source is ARD-Boerse, disable Checkbox
	    if ("Brese ARD".equals(this.wpHistoryBean.getDataSource())) { //$NON-NLS-1$
		this.getShowLowHigh().setEnabled(false);
	    }
	} catch (final FinanceException e) {
	    this.getShowLowHigh().setEnabled(false);
	    throw (RemoteException) new RemoteException(I18N.tr("AnalysenKontroller.stockHistoryLoadError")) //$NON-NLS-1$
		    .initCause(e);
	}
    }

    private void loadDaxData() {

	try {
	    final FfmStockInfoBean ffmInfoBean = FfmStockInfoBean.getDaxBean();
	    this.daxHistory = new WertpapierHistoryBean(WertpapierHistoryBean.DAX_ID, null, ffmInfoBean);
	} catch (final FinanceException e) {
	    GUI.getStatusBar().setErrorText(I18N.tr("AnalysenKontroller.daxHistoryLoadError")); //$NON-NLS-1$
	    this.getShowDax().disable();
	}
    }

    /**
     * Private Listener Klasse, die auf ZeitAuswahlButtonEvents h&ouml;rt.
     *
     * @author munstfr
     * @version 1.0
     */
    private class ZeitAuswahlButtonListener implements Listener {

	/**
	 * Standartkonstruktor
	 */
	public ZeitAuswahlButtonListener() {
	    // absichtlich leer
	}

	/**
	 * @see ZeitAuswahlButtonListener#handleEvent(Event)
	 */
	@Override
	public void handleEvent(final Event event) {

	    if (event.type != SWT.FocusIn && event.type != SWT.FocusOut
		    && event.widget instanceof org.eclipse.swt.widgets.Button) {
		final int time = (Integer) RadioInput.getValue(PeriodChooserInput.GROUP_ID);
		Logger.warn(I18N.tr("AnalysenKontroller.timeRangeChoosen", Integer.toString(time))); //$NON-NLS-1$
		final int toDate = AnalysenKontroller.this.zeitAuswahl.getMaximun();
		AnalysenKontroller.this.zeitAuswahl.setUpper(toDate);
		AnalysenKontroller.this.zeitAuswahl.setLower(toDate - time);
		AnalysenKontroller.this.grafik.setRange(new DateRange(AnalysenKontroller.this.zeitAuswahl.getLower(),
			AnalysenKontroller.this.zeitAuswahl.getUpper()));
		AnalysenKontroller.this.grafik.refresh();
	    }
	}
    }

    /**
     * Private Listener Klasse, die auf ZeitAuswahlEvents h&ouml;rt.
     *
     * @author Frank Mnster
     */
    private class ZeitAuswahlListener implements ZeitauswahlSchieberInputListener {

	/**
	 * Standartkonstruktor
	 */
	public ZeitAuswahlListener() {
	    // absichtlich leer
	}

	/**
	 * @see ZeitAuswahlListener#rangeChanged()
	 */
	@Override
	public void rangeChanged() {

	    Logger.warn(I18N.tr("AnalysenKontroller.timeRangeChanged")); //$NON-NLS-1$
	    AnalysenKontroller.this.grafik.setRange(new DateRange(AnalysenKontroller.this.zeitAuswahl.getLower(),
		    AnalysenKontroller.this.zeitAuswahl.getUpper()));
	    AnalysenKontroller.this.grafik.refresh();
	}
    }
}
