/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.controller;

import java.rmi.RemoteException;
import java.util.AbstractMap.SimpleImmutableEntry;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.FfmStockInfoBean;
import de.frankmuenster.jameica.depot.finanzen.quandl.QuandlStockInfoBean;
import de.frankmuenster.jameica.depot.gui.dialog.SearchDialog;
import de.frankmuenster.jameica.depot.gui.input.CurrencyInput;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.AbstractStockSearchPart.SearchResultSelectListener;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.BoerseArdSelectExchangePart;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.BoerseArdSelectExchangePart.ExchangeSelectListener;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.BoerseArdStockSearchPart;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.FfmSearchPart;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.QuandlSearchPart;
import de.frankmuenster.jameica.depot.gui.views.DepotView;
import de.frankmuenster.jameica.depot.gui.views.WertpapierDetailsView;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.boerseard.ArdStockInfoBean;
import de.frankmuenster.jameica.finanzen.boerseard.BoerseArdApi;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.dialogs.AbstractDialog;
import de.willuhn.jameica.gui.input.TextAreaInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Controller fr die Wertpapier Bearbeiten Ansicht.
 *
 * @author Frank Mnster
 */
@SuppressWarnings({ "PMD.GodClass", "PMD.ExcessiveImports" })
public class EditWertpapierController extends AbstractControl
        implements SearchResultSelectListener<ArdStockInfoBean>, ExchangeSelectListener {

    /**
     *
     */
    private static final String FFM_INSTRUMENT = "ffmInstrument"; //$NON-NLS-1$

    /**
     * Das Ard-Suchfeld
     */
    private transient BoerseArdStockSearchPart ardSearchPart;

    /**
     * Die ARD Wertpapier-Infos
     */
    private transient ArdStockInfoBean bean;

    /**
     * Der Abbrechen-Button
     */
    private transient Button cancle;

    /**
     * Kommentar
     */
    private transient TextAreaInput comment;

    /**
     * Das ARD-Bresen-Suchfeld
     */
    private transient BoerseArdSelectExchangePart exchangesPart;

    /**
     * Der FFM Instrument Name des Wertpapieres
     */
    private transient TextInput ffmInstrument;

    /**
     * Der FFM Such-Button
     */
    private transient Button ffmSearchBtn;

    /**
     * Ist es ein neues Wertpapier?
     */
    private transient boolean isNeuesWertpapier;

    /**
     * Der Wertpapiername
     */
    private transient TextInput name;

    /**
     * Der Quandl Code des Wertpapieres
     */
    private transient TextInput quandlCode;

    /**
     * Der Quandl-Code Such-Button
     */
    private transient Button quandlSearchBtn;

    /**
     * Die W&auml;hrung des Kauf-/Verkaufpreis.
     */
    private transient CurrencyInput stockCur;

    /**
     * Der Speicher-Button
     */
    private transient Button store;

    /**
     * Die ARD Brese ID des Wertpapieres
     */
    private transient TextInput ardId;

    /**
     * Das zuge&ouml;rigige Wertpapier-Objekt
     */
    private transient Wertpapier wertpapier;

    /**
     * Die W&auml;hrung des Wertpapiers
     */
    private transient String wPapierWaehrung;

    /**
     * The ISIN of the security
     */
    private TextInput isin;

    /**
     * Standardkonstruktor
     *
     * @param view
     */
    public EditWertpapierController(final AbstractView view) {
        super(view);
        final Object currentObject = this.getCurrentObject();
        if (currentObject instanceof Wertpapier) {
            Logger.debug("Bearbeite Wertpapier..."); //$NON-NLS-1$
            this.wertpapier = (Wertpapier) currentObject;
            try {
                this.wPapierWaehrung = ((Wertpapier) currentObject).getCurrency();
            } catch (final RemoteException e) {
                Logger.warn("Whrung konnte nicht gelesen werden..." + //$NON-NLS-1$
                        e.getMessage());
            }
            this.isNeuesWertpapier = false;
        } else {
            this.isNeuesWertpapier = true;
        }
    }

    /**
     * @see ExchangeSelectListener#exchangeSelected(SimpleImmutableEntry)
     */
    @Override
    public void exchangeSelected(final SimpleImmutableEntry<String, String> exchange) {

        Logger.debug("Brsenplatz ausgewhlt: " + exchange.getKey() + ", ID: " + exchange.getValue()); //$NON-NLS-1$ //$NON-NLS-2$
        this.bean.setExchange(exchange.getKey());
        this.bean.setId(exchange.getValue());
        if (this.ardId != null && this.name != null) {
            // set ARD ID
            this.ardId.setValue(this.bean.getId());
            // set name
            this.name.setValue(this.bean.getName());
            // set isin
            this.isin.setValue(this.bean.getIsin());
            // set currency - check for index (empty currency)
            if (StringUtils.isEmpty(this.bean.getCurrency())) {
                this.bean.setCurrency("IDX"); //$NON-NLS-1$
            } else {
                this.stockCur.setValue(this.bean.getCurrency());
            }
            this.stockCur.setValue(this.bean.getCurrency());
        }
    }

    /**
     * Liefert das SearchPart
     *
     * @return das SearchPart
     */
    public BoerseArdStockSearchPart getBoerseArdSearchPart() {

        if (this.ardSearchPart == null) {
            this.ardSearchPart = new BoerseArdStockSearchPart(this, 5);
        }
        return this.ardSearchPart;
    }

    /**
     * Liefert eine Liste mit allen zum WP gehrenden Brsenpltzen
     *
     * @return das exchnageListPart
     */
    public BoerseArdSelectExchangePart getBoerseArdSelectExchangePart() {

        if (this.exchangesPart == null) {
            this.exchangesPart = new BoerseArdSelectExchangePart(this);
        }
        return this.exchangesPart;
    }

    /**
     * Liefert den Abbrechen-Button
     *
     * @return den Abbrechen-Button
     */
    public Button getCancel() {

        if (this.cancle == null) {
            this.cancle = new Button(I18N.tr("_cancelBtn"), //$NON-NLS-1$
                    new Action() {

                        /**
                         * @see Action#handleAction(Object context)
                         */
                        @Override
                        public void handleAction(final Object context) throws ApplicationException {

                            if (EditWertpapierController.this.isNeuesWertpapier) {
                                GUI.startView(DepotView.class, null);
                            } else {
                                GUI.startView(WertpapierDetailsView.class, EditWertpapierController.this.wertpapier);
                            }
                        }
                    }, null, false, "cancle-icon.png"); //$NON-NLS-1$
        }
        return this.cancle;
    }

    /**
     * @return comment
     * @throws RemoteException
     */
    public TextAreaInput getComment() throws RemoteException {

        if (this.comment == null) {
            this.comment = new TextAreaInput(
                    this.wertpapier == null ? StringUtils.EMPTY : this.wertpapier.getComment());
        }
        return this.comment;
    }

    /**
     * @return ffmInstrument
     * @throws RemoteException
     */
    public TextInput getFfmInstrument() throws RemoteException {

        if (this.ffmInstrument == null) {
            this.ffmInstrument = new TextInput(StringUtils.EMPTY);
            if (this.wertpapier == null) {
                this.ffmInstrument.setData(EditWertpapierController.FFM_INSTRUMENT, StringUtils.EMPTY);
            } else {
                this.ffmInstrument.setValue(this.wertpapier.getFfmName());
                this.ffmInstrument.setData(EditWertpapierController.FFM_INSTRUMENT, this.wertpapier.getFfmInstrument());
            }
        }
        return this.ffmInstrument;
    }

    /**
     * Liefert den FFM Suchen-Button.
     *
     * @return den FFM Suchen-Button
     */
    public Button getFfmSearchBtn() {

        if (this.ffmSearchBtn == null) {
            this.ffmSearchBtn = new Button("...", new Action() { //$NON-NLS-1$

                /**
                 * @see Action#handleAction(Object)
                 */
                @Override
                public void handleAction(final Object context) throws ApplicationException {

                    // Open SearchDialog
                    String ffmName = StringUtils.EMPTY;
                    try {
                        ffmName = EditWertpapierController.this.wertpapier == null ? StringUtils.EMPTY
                                : EditWertpapierController.this.wertpapier.getName();
                    } catch (final RemoteException e) {
                        // Nur Loggen
                        Logger.debug("Fehler beim Lesen des FFM Wertpapiers. " + e.getLocalizedMessage()); //$NON-NLS-1$
                    }
                    final SearchDialog<FfmStockInfoBean, FfmSearchPart> searchDialog = new SearchDialog<>(
                            AbstractDialog.POSITION_CENTER, ffmName, FfmSearchPart.class);
                    try {
                        final Object result = searchDialog.open();
                        if (result instanceof FfmStockInfoBean) {
                            EditWertpapierController.this.getFfmInstrument()
                                    .setValue(((FfmStockInfoBean) result).getName());
                            EditWertpapierController.this.getFfmInstrument().setData(
                                    EditWertpapierController.FFM_INSTRUMENT,
                                    ((FfmStockInfoBean) result).getInstrumentId());
                        }
                    } catch (final Exception e) { // NOPMD
                        Logger.error(I18N.tr("EditWertpapierController.dialogError"), //$NON-NLS-1$
                                e);
                        throw new ApplicationException(I18N.tr("EditWertpapierController.dialogError"), //$NON-NLS-1$
                                e);
                    }
                }
            });
        }
        return this.ffmSearchBtn;
    }

    /**
     * @return {@link TextInput} field for teh ISIN
     * @throws RemoteException
     */
    public TextInput getIsin() throws RemoteException {

        if (this.isin == null) {
            this.isin = new TextInput(this.wertpapier == null ? StringUtils.EMPTY : this.wertpapier.getIsin());
        }
        return this.isin;
    }

    /**
     * @return name
     * @throws RemoteException
     */
    public TextInput getName() throws RemoteException {

        if (this.name == null) {
            this.name = new TextInput(this.wertpapier == null ? StringUtils.EMPTY : this.wertpapier.getName());
        }
        return this.name;
    }

    /**
     * @return quandlCode
     * @throws RemoteException
     */
    public TextInput getQuandlCode() throws RemoteException {

        if (this.quandlCode == null) {
            final String code = this.wertpapier == null ? StringUtils.EMPTY : this.wertpapier.getQuandlCode();
            this.quandlCode = new TextInput(code);
        }
        return this.quandlCode;
    }

    /**
     * Liefert den Quandl-Code Suchen-Button.
     *
     * @return den Quandl-Code Suchen-Button
     */
    public Button getQuandlSearchBtn() {

        if (this.quandlSearchBtn == null && StringUtils.isNotEmpty(Settings.getQuandlApiKey())) {
            this.quandlSearchBtn = new Button("...", new Action() { //$NON-NLS-1$

                /**
                 * @see Action#handleAction(Object)
                 */
                @Override
                public void handleAction(final Object context) throws ApplicationException {

                    // Open SearchDialog
                    String code = StringUtils.EMPTY;
                    try {
                        code = EditWertpapierController.this.wertpapier == null ? StringUtils.EMPTY
                                : EditWertpapierController.this.wertpapier.getQuandlCode();
                    } catch (final RemoteException e) {
                        // Nur Loggen
                        Logger.debug(
                                I18N.tr("EditWertpapierController.errorReadingQuandlCode") + e.getLocalizedMessage()); //$NON-NLS-1$
                    }
                    final SearchDialog<QuandlStockInfoBean, QuandlSearchPart> searchDialog = new SearchDialog<>(
                            AbstractDialog.POSITION_CENTER, code, QuandlSearchPart.class);
                    try {
                        final Object result = searchDialog.open();
                        if (result instanceof QuandlStockInfoBean) {
                            EditWertpapierController.this.getQuandlCode()
                                    .setValue(((QuandlStockInfoBean) result).getCode());
                        }
                    } catch (final Exception e) { // NOPMD
                        Logger.error(e.getLocalizedMessage(), e);
                        throw new ApplicationException(I18N.tr("EditWertpapierController.errorSearchingQuandlCode"), //$NON-NLS-1$
                                e);
                    }
                }
            });
        }
        return this.quandlSearchBtn;
    }

    /**
     * Whrung
     *
     * @return Whrungsauswahlfeld
     */
    public CurrencyInput getStockCurrency() {

        if (this.stockCur == null) {
            this.stockCur = new CurrencyInput(this.wPapierWaehrung);
            this.stockCur.addListener(new Listener() {

                /**
                 * @see org.eclipse.swt.widgets.Listener#handleEvent(Event
                 *      event)
                 */
                @Override
                public void handleEvent(final Event event) {

                    Logger.warn(event.toString());
                    if (event.type == SWT.Selection) {
                        EditWertpapierController.this.wPapierWaehrung = (String) EditWertpapierController.this.stockCur
                                .getValue();
                    }
                }
            });
        }
        return this.stockCur;
    }

    /**
     * Liefert den Speichern-Button
     *
     * @return den Speichern-Button
     */
    public Button getStore() {

        if (this.store == null) {
            this.store = new Button(I18N.tr("_saveBtn"), new Action() { //$NON-NLS-1$

                /**
                 * @see Action#handleAction(Object context)
                 */
                @Override
                public void handleAction(final Object context) throws ApplicationException {

                    try {
                        // Wertpapier daten speichern
                        EditWertpapierController.this.handleStore();
                        // Wertpapier anzeigen
                        GUI.startView(WertpapierDetailsView.class.getName(), EditWertpapierController.this.wertpapier);
                    } catch (final RemoteException e) {
                        Logger.error(I18N.tr("EditWertpapierController.saveStockError") + e.getLocalizedMessage()); //$NON-NLS-1$
                        throw new ApplicationException(I18N.tr("EditWertpapierController.saveStockError"), //$NON-NLS-1$
                                e);
                    }
                }
            }, null, true, "save-icon.png"); //$NON-NLS-1$
            this.store.setEnabled(true);
        }
        return this.store;
    }

    /**
     * @return ardId
     * @throws RemoteException
     */
    public TextInput getSymbol() throws RemoteException {

        if (this.ardId == null) {
            this.ardId = new TextInput(this.wertpapier == null ? StringUtils.EMPTY : this.wertpapier.getArdId());
        }
        return this.ardId;
    }

    /**
     * @see SearchResultSelectListener#searchResultSelected(de.frankmuenster.jameica.finanzen.StockInfoBean)
     */
    @Override
    public void searchResultSelected(final ArdStockInfoBean stockInfoBean) {

        this.handleArdSelected(stockInfoBean);
    }

    /**
     * @param stockInfo
     */
    private void handleArdSelected(final ArdStockInfoBean stockInfo) {

        this.bean = stockInfo;
        // Brsenpltze aktualisiern
        this.updateExchangeList();
        if (this.ardId != null) {
            this.ardId.setValue(StringUtils.EMPTY);
        }
        if (this.name != null) {
            this.name.setValue(StringUtils.EMPTY);
        }
        if (this.isin != null) {
            this.isin.setValue(StringUtils.EMPTY);
        }
    }

    private void handleStore() throws RemoteException, ApplicationException {

        final DBService service = Settings.getDBService();
        if (this.wertpapier == null) {
            // Es muss ein neues WP angelegt werden.
            this.wertpapier = (Wertpapier) service.createObject(Wertpapier.class, null);
        }
        this.wertpapier.setArdId((String) this.getSymbol().getValue());
        this.wertpapier.setName((String) this.getName().getValue());
        this.wertpapier.setIsin((String) this.getIsin().getValue());
        this.wertpapier.setFfmName((String) this.getFfmInstrument().getValue());
        this.wertpapier
                .setFfmInstrument((String) this.getFfmInstrument().getData(EditWertpapierController.FFM_INSTRUMENT));
        this.wertpapier.setQuandlCode((String) this.getQuandlCode().getValue());
        this.wertpapier.setCurrency((String) this.getStockCurrency().getValue());
        this.wertpapier.setComment((String) this.getComment().getValue());
        // Aktuellen Kurs ermitteln
        try {
            ArdStockInfoBean infoBean = new ArdStockInfoBean();
            infoBean.setId(this.wertpapier.getArdId());
            infoBean = BoerseArdApi.getInstance().getActualValue(infoBean);
            if (infoBean != null) {
                this.wertpapier.setLastTradePriceOnly(infoBean.getActValue());
                this.wertpapier.setLastTradeDate(infoBean.getDate());
            }
        } catch (final FinanceException ye) {
            // OK, dann halt nicht.
            Logger.info(I18N.tr("EditWertpapierController.noFinancialInfoFound")); //$NON-NLS-1$
        }
        this.wertpapier.setInArd();
        this.wertpapier.store();
    }

    /**
     * Zeigt die Brsenliste des aktuellen Wertpapieres an
     */
    private void updateExchangeList() {

        if (this.bean == null) {
            if (this.exchangesPart != null) {
                this.exchangesPart.fillList(null);
            }
        } else {
            try {
                BoerseArdApi.getInstance().getActualValue(this.bean);
                if (this.exchangesPart != null) {
                    this.exchangesPart.fillList(this.bean.getExchanges());
                }
            } catch (final FinanceException e) {
                Logger.error(I18N.tr("EditWertpapierController.errorGettingStockExchanges") + e.getLocalizedMessage(), //$NON-NLS-1$
                        e);
            }
        }
    }
}
