/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.controller;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.finanzen.quandl.QuandlStockInfoBean;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.AbstractStockSearchPart.SearchResultSelectListener;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.QuandlSearchPart;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.FormTextPart;

/**
 * Controller for the Quandl search dialog window
 * 
 * @author Frank Mnster
 */
public class QuandlDialogController implements SearchResultSelectListener<QuandlStockInfoBean> {

    /**
     * Der Bescheibungstext
     */
    private FormTextPart description;

    /**
     * Das QuandlCode Input
     */
    private TextInput quandlCodeInput;

    /**
     * Der Quandl Code
     */
    private String quandlCode;

    /**
     * Quandl-Suchfeld
     */
    private QuandlSearchPart quandlSearchPart;

    /**
     * Returns a new QuandlDialogController initialised with the given quandl
     * code
     *
     * @param quandlCode
     */
    public QuandlDialogController(final String quandlCode) {
        this.quandlCode = quandlCode;
    }

    /**
     * Liefert den Bereich, in der die Beschreibung angezeigt wird.
     *
     * @return FormTextPart fr die Beschreibung
     */
    public FormTextPart getDescription() {

        if (this.description == null) {
            this.description = new FormTextPart();
        }
        this.description = new FormTextPart("<form><p></p><p></p><p></p></form>"); //$NON-NLS-1$
        return this.description;
    }

    /**
     * Liefert eine TextInput fr den Quandl-Code
     *
     * @return quandlCodeInput TextInput fr den Quandl-Code
     * @throws RemoteException
     */
    public TextInput getQuandlCode() throws RemoteException {

        if (this.quandlCodeInput == null) {
            this.quandlCodeInput = new TextInput(this.quandlCode == null ? StringUtils.EMPTY : this.quandlCode);
        }
        return this.quandlCodeInput;
    }

    /**
     * Liefert eine QuandlSearchPart
     *
     * @return eine QuandlSearchPart
     */
    public QuandlSearchPart getQuandlSearchPart() {

        if (this.quandlSearchPart == null) {
            this.quandlSearchPart = new QuandlSearchPart(this, 15);
        }
        return this.quandlSearchPart;
    }

    /**
     * @see SearchResultSelectListener#searchResultSelected(de.frankmuenster.jameica.finanzen.StockInfoBean)
     */
    @Override
    public void searchResultSelected(final QuandlStockInfoBean bean) {

        this.quandlCode = bean.getCode();
        this.quandlCodeInput.setValue(this.quandlCode);
        final StringBuilder text = new StringBuilder("<form><p>"); //$NON-NLS-1$
        text.append(bean.getDescription()).append("</p></form>"); //$NON-NLS-1$

        try {
            this.description.setText(text.toString());
        } catch (final IllegalArgumentException e) {
            this.description.setText(bean.getDescription());
        }
        this.description.refresh();

    }

    /**
     * Liefert den aktuell ausgewhlten oder gesetzten QuandlCode.
     *
     * @return den aktuellen QuandlCode
     */
    public String getSelectedCode() {

        return this.quandlCode;
    }
}
