/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.controller;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.gui.views.SettingsView;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.gui.input.TextInput;

/**
 * Kontroller fr die Settings View
 *
 * @author Frank Mnster
 */
public class SettingsController extends AbstractControl {

    /**
     * Zeige die Hibiscus Konto Info's
     */
    private transient CheckboxInput showHibiscusInfo;

    /**
     * Quandl API
     */
    private transient TextInput quandlApiKey;

    /**
     * Erstellt eine neue Instanz des Kontrollers
     *
     * @param view
     */
    public SettingsController(final SettingsView view) {
	super(view);
    }

    /**
     * Liefert ein TextInput zur Eingabe des Quandl-API keys.
     *
     * @return ein TextInput zur Eingabe des Quandl-API keys
     */
    public TextInput getQuandlApiKey() {

	if (this.quandlApiKey == null) {
	    this.quandlApiKey = new TextInput(Settings.getQuandlApiKey());
	    this.quandlApiKey.setComment(I18N.tr("SettingsController.seeQuandlWebsite")); //$NON-NLS-1$
	}

	return this.quandlApiKey;
    }

    /**
     * @return Ein CheckBoxInput zum setzten der Anzeige der
     *         Hibiscuszusammenfassung.
     */
    public CheckboxInput getShowHibiscusSummery() {

	if (this.showHibiscusInfo == null) {
	    this.showHibiscusInfo = new CheckboxInput(Settings.isShowHibiscusSummery());
	}
	return this.showHibiscusInfo;
    }

    /**
     * Speichert die aktuellen Einstellungen im Settings Objekt.
     */
    public void speichern() {

	if (Settings.isHibiscusInstalled()) {
	    Settings.setShowHibiscusSummery((Boolean) this.showHibiscusInfo.getValue());
	} else {
	    Settings.setShowHibiscusSummery(false);

	}
	Settings.setQuandlApiKey((String) this.quandlApiKey.getValue());

	GUI.getStatusBar().setSuccessText(I18N.tr("SettingsController.saveSuccess")); //$NON-NLS-1$

    }
}
