/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/
package de.frankmuenster.jameica.depot.gui.dialog;

import java.rmi.RemoteException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.depot.gui.controller.SearchDialogController;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.AbstractStockSearchPart;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.frankmuenster.jameica.finanzen.StockInfoBean;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.dialogs.AbstractDialog;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.parts.FormTextPart;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.util.ApplicationException;

/**
 * Suchfenster fr Wertpapiere
 *
 * @author Frank Mnster
 * @param <I>
 *            the StockInfoBean
 * @param <S>
 *            the Part with the search details for the stock
 */
public class SearchDialog<I extends StockInfoBean, S extends AbstractStockSearchPart<I>>
	extends AbstractDialog<Object> {

    /**
     * Der ausgewhlte Code
     */
    private transient I bean;

    /**
     * Unser Kontrolleur
     */
    private final transient SearchDialogController<I, S> controller;

    /**
     * Liefert einen modalen Dialog, der mittel .open() angezeigt werden kann
     *
     * @param position
     *            die Position, an der der Dialog angezeigt werden soll.
     *            Mgliche Werte sind POSITION_MOUSE und POSITION_CENTER.
     * @param name
     *            der Infobean aus einem ggf. vorbelegtem Feld
     * @param searchPartClazz
     *            the class of the searchPart
     */
    public SearchDialog(final int position, final String name, final Class<S> searchPartClazz) {
	super(position, false);
	this.setPanelText(I18N.tr("SearchDialog.panelText")); //$NON-NLS-1$
	this.controller = new SearchDialogController<>(name, searchPartClazz);
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
     */
    @Override
    protected Object getData() {

	return this.bean;
    }

    /**
     * @throws ApplicationException
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    protected void paint(final Composite parent) throws RemoteException, ApplicationException {

	final ColumnLayout columns = new ColumnLayout(parent, 2);
	final Container links = new SimpleContainer(columns.getComposite());
	// rechte Seite
	final Container rechts = new SimpleContainer(columns.getComposite(), true, 1);
	final GridData gridData = (GridData) rechts.getComposite().getLayoutData();
	gridData.minimumWidth = 200;
	gridData.widthHint = 200;
	final AbstractStockSearchPart<?> searchPart = this.controller.getSearchPart();
	searchPart.paint(links.getComposite());
	this.setTitle(searchPart.getTitle());
	// Felder
	links.addLabelPair(I18N.tr("SearchDialog.codeLbl"), this.controller.getResultInput()); //$NON-NLS-1$
	// Beschreibung
	rechts.addHeadline(I18N.tr("_description")); //$NON-NLS-1$
	final FormTextPart descriptionPart = this.controller.getDescription();
	descriptionPart.paint(rechts.getComposite());
	final ButtonArea buttons = new ButtonArea();
	buttons.addButton(I18N.tr("_takeover"), new Action() { //$NON-NLS-1$

	    /**
	     * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
	     */
	    @Override
	    public void handleAction(final Object context) throws ApplicationException {

		SearchDialog.this.bean = SearchDialog.this.controller.getSelectedBean();
		SearchDialog.this.close();
	    }
	}, null, true, "ok.png"); //$NON-NLS-1$
	buttons.addButton(I18N.tr("_close"), new Action() { //$NON-NLS-1$

	    /**
	     * @see de.willuhn.jameica.gui.Action#handleAction(Object context)
	     */
	    @Override
	    public void handleAction(final Object context) throws ApplicationException {

		SearchDialog.this.close();
	    }
	}, null, true, "window-close.png"); //$NON-NLS-1$
	final SimpleContainer footer = new SimpleContainer(parent);
	footer.addButtonArea(buttons);
	this.setSize(650, SWT.DEFAULT);
    }
}
