/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.formatter;

import java.text.DecimalFormat;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.willuhn.jameica.gui.formatter.Formatter;

/**
 * Formatierer fr Dezimalzahlen.
 *
 * @author Frank Mnster
 */
public class DecimalFormatter implements Formatter {

    /**
     * Das Decimalformat
     */
    private transient DecimalFormat formatter = FormatterUtils.DECIMAL_FORMAT_6.get();

    /**
     * Erzeugt einen neuen Formatierer mit dem angegeben Waehrungsstring.
     *
     * @param formatter
     *            kann optional angegeben werden, um den Betrag zu formatieren.
     *            Wird der Parameter weggelassen, werden die Werte auf 2 Stellen
     *            hinter dem Komma formatiert.
     */
    public DecimalFormatter(final DecimalFormat formatter) {

	if (formatter == null) {
	    this.formatter.applyPattern("#0.00"); //$NON-NLS-1$
	} else {
	    this.formatter = formatter;
	}
    }

    /**
     * Formatiert das bergebene Objekt. Es kann von folgenden Typen sein:
     * <ul>
     * <li>String</li>
     * <li>Number (oder davon abgeleitete Typen)</li>
     * </ul>
     *
     * @see de.willuhn.jameica.gui.formatter.Formatter#format(java.lang.Object)
     */
    @Override
    public String format(final Object objectToFormat) {

	String formatedText;
	if (objectToFormat == null) {
	    formatedText = StringUtils.EMPTY;
	} else if (objectToFormat instanceof Number) {
	    formatedText = this.formatter.format(((Number) objectToFormat).doubleValue());
	} else {
	    formatedText = objectToFormat.toString();
	}
	return formatedText;
    }

}
