/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.input;

import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import de.frankmuenster.jameica.depot.utils.I18N;
import de.frankmuenster.jameica.finanzen.currency.Currency;
import de.willuhn.jameica.gui.input.SelectInput;

/**
 * Auswahlfeld f&uuml;r die W&auml;hrung.
 *
 * @author Frank Mnster
 */
public class CurrencyInput extends SelectInput {

    /**
     * Liefert eine Liste der moeglichen W&auml;hrungen.
     *
     * @return Liste der moeglichen W&auml;hrungen.
     */
    private static List<String> getCodes() {

        return Currency.codes();
    }

    /**
     * Konstruktor mit Angabe der W&auml;hrung
     *
     * @param value
     *            die vorausgew&auml;hlte W&auml;hrung.
     */
    public CurrencyInput(final String value) {
        super(CurrencyInput.getCodes(), value);
        this.setName(I18N.tr("_currency")); //$NON-NLS-1$
        this.setEditable(false); // Man kann selbst keine Werte eingeben
        this.setComment(""); //$NON-NLS-1$

        final Listener listener = new Listener() {

            /**
             * @see org.eclipse.swt.widgets.Listener#handleEvent(Event event)
             */
            @Override
            public void handleEvent(final Event event) {

                final String value = (String) CurrencyInput.this.getValue();
                final Currency currency = Currency.find(value);
                CurrencyInput.this.setComment(currency == null ? StringUtils.EMPTY : currency.getName());
            }
        };
        this.addListener(listener);

        // Einmal initial ausloesen
        listener.handleEvent(null);
    }

    /**
     * @see de.willuhn.jameica.gui.input.SelectInput#setValue(java.lang.Object)
     */
    @Override
    public void setValue(final Object currency) {

        super.setValue(currency);

        // Kommentar aktualisieren
        final String value = (String) this.getValue();
        final Currency currencyCode = Currency.find(value);
        this.setComment(currencyCode == null ? StringUtils.EMPTY : currencyCode.getName());
    }
}
