/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.input;

import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.mihalis.opal.rangeSlider.RangeSlider;

/**
 * Listener der &Auml;nderungen an dem Zeitauswahlschieber verarbeitet.
 *
 * @author Frank Mnster
 */
public class ZeitauswahlSchieberInputChangedListener implements KeyListener, MouseMoveListener, SelectionListener {

    /**
     * Der Zeitauswahlschieber f&uer die Callback-Aufrufe
     */
    private final transient ZeitauswahlSchieberInput source;

    /**
     * Konstruktor mit Angabe des Source Zeitauswahlschiebers
     * 
     * @param source
     */
    public ZeitauswahlSchieberInputChangedListener(final ZeitauswahlSchieberInput source) {
	this.source = source;
    }

    /**
     * @see org.eclipse.swt.events.KeyListener#keyPressed(KeyEvent)
     */
    @Override
    public void keyPressed(final KeyEvent arg0) {
	// Hier nix machen

    }

    /**
     * @see org.eclipse.swt.events.KeyListener#keyReleased(KeyEvent)
     */
    @Override
    public void keyReleased(final KeyEvent arg0) {
	handleAction();
    }

    /**
     * @see org.eclipse.swt.events.MouseMoveListener#mouseMove(MouseEvent)
     */
    @Override
    public void mouseMove(final MouseEvent arg0) {
	handleAction();
    }

    /**
     * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(SelectionEvent)
     */
    @Override
    public void widgetDefaultSelected(final SelectionEvent arg0) {
	// Hier nix machen

    }

    /**
     * @see org.eclipse.swt.events.SelectionListener#widgetSelected(SelectionEvent)
     */
    @Override
    public void widgetSelected(final SelectionEvent arg0) {
	handleAction();

    }

    /**
     * Private Funktion, die sich um die &Auml;nderungen k&uuml;mmert.
     */
    private void handleAction() {

	final RangeSlider slider = (RangeSlider) this.source.getControl();

	final int lNew = slider.getLowerValue();
	final int uNew = slider.getUpperValue();

	final int lOld = ((int[]) this.source.getValue())[0];
	final int uOld = ((int[]) this.source.getValue())[1];

	// etwas verndert
	if (lNew != lOld || uNew != uOld) {
	    this.source.setLower(lNew);
	    this.source.setUpper(uNew);
	    this.source.notifyListeners();
	}
    }

}
