/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.parts;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.rmi.RemoteException;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import de.frankmuenster.jameica.depot.kursanalyse.Analysen;
import de.frankmuenster.jameica.depot.kursanalyse.IHandleAddAction;
import de.frankmuenster.jameica.depot.kursanalyse.MethodeAddRemoveListener;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.Part;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Part das dynamisch Buttons fr noch verfgbare Analysen anzeigt und einen
 * 'Alle Entfernen' Button enthlt.
 *
 * @author Frank Mnster
 *
 */
public class AnalyseButtonPart implements Part {

    /**
     * Unser Parent Composite
     */
    private transient Composite parent;

    /**
     * Der Listener
     */
    private final transient MethodeAddRemoveListener listener;

    /**
     * Part das dynamisch Buttons fr noch verfgbare Analysen anzeigt und einen
     * 'Alle Entfernen' Button enthlt.
     * 
     * @param listener
     *            MethodeAddRemoveListener der an die Button angehangen wird.
     */
    public AnalyseButtonPart(final MethodeAddRemoveListener listener) {
	this.listener = listener;
    }

    /**
     * @see Part#paint(Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {

	this.parent = parent;
	refresh();

    }

    /**
     * Aktualisiert die angezeigten Buttons.
     */
    public void refresh() {

	// clear everything
	for (final Control child : this.parent.getChildren()) {
	    child.dispose();
	}

	// Add Buttons
	final Composite buttonArea = new Composite(this.parent, SWT.NONE);
	final GridLayout gridLayout = new GridLayout();
	gridLayout.numColumns = 3;
	gridLayout.makeColumnsEqualWidth = true;
	buttonArea.setLayout(gridLayout);

	final List<Class<?>> methodesClassList = Analysen.getMethods();

	for (final Class<?> methodesClass : methodesClassList) {

	    try {
		final Method getName = methodesClass.getMethod("getMethodName", //$NON-NLS-1$
			(Class<?>[]) null);
		final Method getAction = methodesClass.getMethod("getHandleAction", (Class<?>[]) null); //$NON-NLS-1$

		final String name = (String) getName.invoke(null);
		final IHandleAddAction addAction = (IHandleAddAction) getAction.invoke(null);
		if (addAction != null) {

		    // Analyse schon vorhanden
		    final Analysen<?, ?> analysen = Analysen.getInstance();

		    if (!analysen.conatinsAnalyseWithLegendTitel(name)) {
			addAction.addListener(this.listener);
			final AnalyseButton analyseButton = new AnalyseButton( // NOPMD
				name, addAction);
			analyseButton.paint(buttonArea);

		    }
		}

	    } catch (NoSuchMethodException | SecurityException | IllegalAccessException | IllegalArgumentException
		    | InvocationTargetException e) {
		Logger.error(I18N.tr("AnalyseButtonPart.actionNotFound", e.getMessage())); //$NON-NLS-1$
	    } catch (final RemoteException e) {
		Logger.error(I18N.tr("AnalyseButtonPart.buttonNotFound", e.getMessage())); //$NON-NLS-1$
	    }
	}

	// Remove all button
	addRemoveAllButton(buttonArea);

	// Layout berechnen
	buttonArea.layout();
	this.parent.layout();

    }

    /**
     * F&uuml;gt den RemoveAll Button hinzu
     * 
     * @param buttonArea
     *            das Composite zu dem den Button hinzuge&uuml;gt werden soll.
     */
    private void addRemoveAllButton(final Composite buttonArea) {
	final AnalyseButton removeAllBtn = new AnalyseButton(I18N.tr("AnalyseButtonPart.removeAllBtn"), //$NON-NLS-1$
		new Action() {
		    /**
		     * @see de.willuhn.jameica.gui.Action#handleAction(Object
		     *      context)
		     */
		    @Override
		    public void handleAction(final Object context) throws ApplicationException {
			AnalyseButtonPart.this.listener.notifyRemoveAll();
		    }
		});

	try {
	    removeAllBtn.paint(buttonArea);
	} catch (final RemoteException e) {
	    Logger.error(I18N.tr("AnalyseButtonPart.addButtonError", e.getMessage())); //$NON-NLS-1$
	}
    }

}
