/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.parts;

import java.rmi.RemoteException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.depot.gui.input.PeriodChooserInput;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.input.CheckboxInput;

/**
 * Part das verschiedene Werkzeuge zur Diagrammanzeige bietet.
 *
 * @author Frank Mnster
 *
 */
public class DiagrammAnzeigeAuswahlPart implements Part {

    /**
     * Dax anzeuigen
     */
    private CheckboxInput dax;

    /**
     * Diagramm als Kerzen zeigen
     */
    private CheckboxInput highLow;

    /**
     * Der Zeitauswahlschieber
     */
    private PeriodChooserInput period;

    /**
     * Unser Composite in dem wir malen
     */
    private transient Composite werkzeugArea;

    /**
     * Standardkonstruktor
     */
    public DiagrammAnzeigeAuswahlPart() {
	super();
    }

    /**
     * @return the werkzeugArea
     */
    public Composite getComposite() {
	return this.werkzeugArea;
    }

    /**
     * @return the dax
     */
    public CheckboxInput getDax() {
	return this.dax;
    }

    /**
     * @return the highLow
     */
    public CheckboxInput getHighLow() {
	return this.highLow;
    }

    /**
     * @return the period
     */
    public PeriodChooserInput getPeriod() {
	return this.period;
    }

    /**
     * @see Part#paint(Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {

	this.werkzeugArea = new Composite(parent, SWT.NONE);

	final GridLayout gridLayout = new GridLayout();
	gridLayout.numColumns = 3;
	gridLayout.makeColumnsEqualWidth = false;

	this.werkzeugArea.setLayout(gridLayout);

	final GC graphic = new GC(this.werkzeugArea);
	if (this.dax != null) {
	    final Point daxSize = graphic.textExtent(this.dax.getName());
	    this.dax.paint(this.werkzeugArea, daxSize.x + 4 + 24);
	}

	final Point highLowSize = graphic.textExtent(this.highLow.getName());
	graphic.dispose();

	this.highLow.paint(this.werkzeugArea, highLowSize.x + 15 + 24);
	this.period.paint(this.werkzeugArea, 300);

    }

    /**
     * @param dax
     *            the dax to set
     */
    public void setDax(final CheckboxInput dax) {
	this.dax = dax;
    }

    /**
     * @param highLow
     *            the highLow to set
     */
    public void setHighLow(final CheckboxInput highLow) {
	this.highLow = highLow;
    }

    /**
     * @param period
     *            the period to set
     */
    public void setPeriod(final PeriodChooserInput period) {
	this.period = period;
    }

}
