/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.parts.column;

import java.math.BigDecimal;
import java.rmi.RemoteException;

import org.apache.commons.lang3.StringUtils;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.formatter.Formatter;
import de.willuhn.jameica.gui.parts.Column;
import de.willuhn.logging.Logger;

/**
 * Tabellenspalte f&uuml;f die Order-Tabelle mit der Wert der Ordner in
 * Fremdw&auml;hrung.
 *
 * @author Frank Mnster
 */
public class OrderTotal extends Column {

    /**
     * serialVersionUID
     */
    private static final long serialVersionUID = -5056571653872956430L;

    /**
     * Standardkonstruktor
     */
    public OrderTotal() {
        this(null);
    }

    /**
     * Konstruktor mit Angabe eines Formatters
     *
     * @param formatter
     */
    public OrderTotal(final Formatter formatter) {
        super("id", I18N.tr("OrderTotal.total"), formatter, false, Column.ALIGN_RIGHT); //$NON-NLS-1$ //$NON-NLS-2$
    }

    /**
     * @see de.willuhn.jameica.gui.parts.Column#getFormattedValue(java.lang.Object,
     *      java.lang.Object)
     */
    @Override
    public String getFormattedValue(final Object objectId, final Object context) {

        final OrderHistory orderHistory = (OrderHistory) context;
        BigDecimal betrag;
        String display = StringUtils.EMPTY;
        try {
            betrag = orderHistory.getTotalOrderCosts();

            // Formatter definiert?
            if (this.getFormatter() == null) {
                display = betrag.toString() + StringUtils.SPACE + Settings.getDepotWaehrung();
            } else {
                display = this.getFormatter().format(betrag);
            }
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("OrderTotal.formatError", //$NON-NLS-1$
                    (String) objectId, (String) context), e);
        }

        return display;
    }

}
