/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.parts.column;

import java.math.BigDecimal;
import java.rmi.RemoteException;

import org.apache.commons.lang3.StringUtils;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.formatter.Formatter;
import de.willuhn.jameica.gui.parts.Column;
import de.willuhn.logging.Logger;

/**
 * Tabellenspalte f&uuml;r die Order-Tabelle mit den Betr&auml;gen in der
 * Standardw&auml;hrung.
 *
 * @author Frank Mnster
 */
public class OrderTotalInStandartWaehrung extends Column {

    /**
     * Serial UID
     */
    private static final long serialVersionUID = -3910333581193391135L;

    /**
     * Standardkonstruktor
     */
    public OrderTotalInStandartWaehrung() {
        this(null);
    }

    /**
     * Konstruktor mit Angabe eines Formatters
     *
     * @param formatter
     */
    public OrderTotalInStandartWaehrung(final Formatter formatter) {
        super("id", StringUtils.EMPTY, formatter, false, Column.ALIGN_RIGHT); //$NON-NLS-1$
        this.setName(I18N.tr("OrderTotalInStandartWaehrung.totalIn") + Settings.getDepotWaehrung()); //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.jameica.gui.parts.Column#getFormattedValue(java.lang.Object,
     *      java.lang.Object)
     */
    @Override
    public String getFormattedValue(final Object objectId, final Object context) {

        BigDecimal betrag = null;
        String display = "Format Error"; //$NON-NLS-1$

        if (context instanceof OrderHistory) {
            final OrderHistory orderHistory = (OrderHistory) context;
            try {
                betrag = orderHistory.getTotalOrderCostsInMainCurrency();

            } catch (final RemoteException e) {
                Logger.error(I18N.tr("OrderTotalInStandartWaehrung.formatError", //$NON-NLS-1$
                        (String) objectId, (String) context), e);
            }
        } else if (context instanceof Wertpapier) {
            final Wertpapier wertpapier = (Wertpapier) context;
            try {
                betrag = wertpapier.getKauf();
            } catch (final RemoteException e) {
                Logger.error(I18N.tr("OrderTotalInStandartWaehrung.formatError", //$NON-NLS-1$
                        (String) objectId, (String) context), e);
            }
        }

        if (betrag != null) {
            // Formatter definiert?
            if (this.getFormatter() == null) {
                display = betrag.toString() + StringUtils.SPACE + Settings.getDepotWaehrung();
            } else {
                display = this.getFormatter().format(betrag);
            }
        }

        return display;
    }

}
