/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/
package de.frankmuenster.jameica.depot.gui.parts.column;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.formatter.Formatter;
import de.willuhn.logging.Logger;

/**
 * Tabellenspalte f&uuml;r die Wertpapiertabelle in Standardw&auml;hrung.
 *
 * @author Frank Mnster
 */
public class WertpapierInStandardWaehrung extends AbstractColumnInStandardWaehrung {

    /**
     * Serial UID
     */
    private static final long serialVersionUID = -1784848538204822166L;

    /**
     * Standardkonstruktor
     */
    public WertpapierInStandardWaehrung() {
	this(null);
    }

    /**
     * Konstruktor mit Angabe eines Formatters
     *
     * @param formatter
     */
    public WertpapierInStandardWaehrung(final Formatter formatter) {
	super(formatter);
    }

    /**
     * @see de.willuhn.jameica.gui.parts.Column#getFormattedValue(java.lang.Object,
     *      java.lang.Object)
     */
    @Override
    public String getFormattedValue(final Object objectId, final Object context) {

	String display = StringUtils.EMPTY;

	if (context instanceof Wertpapier) {
	    final Wertpapier wertpapier = (Wertpapier) context;
	    try {
		display = getFormattedValue(wertpapier.getGesamtWert(), wertpapier.getCurrency(), null);
	    } catch (final RemoteException e) {
		Logger.error(I18N.tr("AbstractColumnInStandardWaehrung.formatError", //$NON-NLS-1$
			(String) objectId, (String) context), e);
		GUI.getStatusBar().setErrorText(I18N.tr("AbstractColumnInStandardWaehrung.userMsg")); //$NON-NLS-1$
	    }
	}
	return display;
    }

}
