/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.parts.filedropper;

import java.util.Locale;

import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;

import de.frankmuenster.jameica.depot.gui.utils.GUIUtil;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.logging.Logger;

/**
 * The DropTargetAdapter for the drop here label
 *
 * @author Frank Mnster
 *
 */

final class DropHereDropTargetAdapter extends DropTargetAdapter {

    /**
     * The FileLoadDropPastePart
     */
    private final FileLoadDropPastePart fileLoadDropPastePart;

    /**
     * Constructs a new DropTragetAdapter that can be attached to a DropTarget
     *
     * @param fileLoadDropPastePart
     *            the main part which needs to be informed
     */
    protected DropHereDropTargetAdapter(final FileLoadDropPastePart fileLoadDropPastePart) {
        super();
        this.fileLoadDropPastePart = fileLoadDropPastePart;
    }

    /**
     * @see org.eclipse.swt.dnd.DropTargetAdapter#dragEnter
     */
    @Override
    public void dragEnter(final DropTargetEvent event) {
        event.detail = DND.DROP_COPY;
    }

    /**
     * @see org.eclipse.swt.dnd.DropTargetAdapter#drop
     */
    @Override
    public void drop(final DropTargetEvent event) {
        if (event.data instanceof String[]) {
            final String[] filenames = (String[]) event.data;
            if (filenames.length > 0) {
                // bring Jameica to front
                GUIUtil.bringToFront(this.fileLoadDropPastePart.getParent().getShell());
                final String file = filenames[0];
                Logger.debug("Filename: " + file); //$NON-NLS-1$
                final String fileType = file.substring(file.lastIndexOf('.') + 1).toLowerCase(Locale.getDefault());
                if (FileLoadDropPastePart.SUPPORTED_TYPES.contains(fileType)) {
                    this.fileLoadDropPastePart.informListers(file);
                } else {
                    this.fileLoadDropPastePart
                            .setStatusText(I18N.tr("DropHereDropTargetAdapter.fileTypeNotSupportted")); //$NON-NLS-1$
                    Logger.info("Dateityp wird nicht untersttzt."); //$NON-NLS-1$
                }
            }
        }
    }
}