/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.parts.filedropper; // NOPMD

import java.io.File;
import java.rmi.RemoteException;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;

import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.Part;
import de.willuhn.logging.Logger;

/**
 * A nice small {@link Part} which contains a Drop Label, an open button and a status label .<br/>
 *
 * @author Frank Mnster
 */
public class FileLoadDropPastePart implements Part {

    /**
     * Maximal Lnge eines Dateinamens in der Statusanzeige
     */
    private static final int MAX_FILE_NAME_LENGTH = 23;

    /**
     * Untersttzte Dateiformate
     */
    protected static final List<String> SUPPORTED_TYPES = Arrays.asList("csv", "txt"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * Composite For the whole drop label
     */
    private Composite parent;

    /**
     * The status label
     */
    private Label lblStatus;

    /**
     * The registered listeners
     */
    private final Set<FileAddedListener> listeners = new CopyOnWriteArraySet<>();

    /**
     * The loaded file
     */
    private File file;

    /**
     * Constructs a new drop label
     */
    public FileLoadDropPastePart() {

        // Intentionally left blank
    }

    /**
     * Adds a listener. Needed?
     *
     * @param listener
     *            FileAddedListener
     */
    public final void addListener(final FileAddedListener listener) {

        this.listeners.add(listener);
    }

    /**
     * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {

        if (parent == null) {
            Logger.error("Parent is null"); //$NON-NLS-1$
        } else {
            final GridData gdata = (GridData) parent.getLayoutData();
            gdata.grabExcessHorizontalSpace = true;
            gdata.grabExcessVerticalSpace = false;
            parent.setLayoutData(gdata);
            // neues Composite erstellen, dass Text und Buttons auf nimmt
            this.parent = new Composite(parent, SWT.NONE);
            final GridLayout gridLayout = new GridLayout();
            gridLayout.numColumns = 1;
            this.parent.setLayout(gridLayout);
            // neues Composite erstellen, welches Platz fuer den Kommentar
            // laesst.
            final Composite lableRow = new Composite(this.parent, SWT.NONE);
            final RowLayout rowLayout = new RowLayout();
            rowLayout.wrap = false;
            rowLayout.pack = true;
            rowLayout.justify = true;
            rowLayout.type = SWT.HORIZONTAL;
            rowLayout.marginLeft = 5;
            rowLayout.marginTop = 5;
            rowLayout.marginRight = 5;
            rowLayout.marginBottom = 5;
            rowLayout.spacing = 0;
            rowLayout.center = true;
            lableRow.setLayout(rowLayout);
            // DropLabel erstellen
            final Button buttonDrop = new Button(lableRow, SWT.FLAT);
            buttonDrop.setLayoutData(new RowData(180, 39));
            buttonDrop.setText(I18N.tr("FileLoadDropPastePart.dropHere")); //$NON-NLS-1$
            // drop target
            final FileTransfer fTransfer = FileTransfer.getInstance();
            final Transfer[] transfers = new Transfer[] { fTransfer };
            final int ops = DND.DROP_COPY | DND.DROP_LINK | DND.DROP_DEFAULT;
            final DropTarget target = new DropTarget(buttonDrop, ops);
            target.setTransfer(transfers);
            target.addDropListener(new DropHereDropTargetAdapter(this));
            // FileDialog ffnen Button
            final Button button = new Button(lableRow, SWT.PUSH);
            button.setText("..."); //$NON-NLS-1$
            button.setLayoutData(new RowData(40, 39));
            button.addSelectionListener(new OpenFileDialogSelectionAdapter(this));
            // Seperator
            final Label seperatorLabel = new Label(lableRow, SWT.SEPARATOR | SWT.VERTICAL);
            final RowData seperatoLayout = new RowData();
            seperatoLayout.height = 30;
            seperatoLayout.width = 20;
            seperatorLabel.setLayoutData(seperatoLayout);
            // Paste Button
            final Button pasteButton = new Button(lableRow, SWT.PUSH);
            pasteButton.setText("Einfgen"); //$NON-NLS-1$
            pasteButton.setLayoutData(new RowData(60, 39));
            pasteButton.addSelectionListener(new PasteButtonSelectionAdapter(this));
            // Status Feld
            this.lblStatus = new Label(this.parent, SWT.NONE);
        }
    }

    /**
     * Entfernd einen Listener aus der Liste
     *
     * @param listener
     *            QrCodeFoundListener der entfernt werden soll
     */
    public final void removeListener(final FileAddedListener listener) {

        this.listeners.remove(listener);
    }

    /**
     * Setzt den Statustext innerhalb des DropLabelParts
     *
     * @param status
     *            der Statustext der angezeigt werden soll
     */
    public void setStatusText(final String status) {

        if (this.lblStatus != null) {

            Display.getDefault().asyncExec(() -> {
                // ... do any work that updates the screen ...
                if (!this.lblStatus.isDisposed()) {
                    this.lblStatus.setText(status);
                    this.lblStatus.redraw();
                    this.lblStatus.update();
                    this.lblStatus.getParent().layout();
                }
            });

        }
    }

    /**
     * Liefert das Composite, in dem das Control gemalt werden soll.
     *
     * @return das Composite, in dem das Control platziert wird.
     */
    protected Composite getParent() {

        return this.parent;
    }

    /**
     * Should be called from the Adapters to inform the part about a new file.
     *
     * @param fileName
     *            the file name that has been dropped/pasted or opened.
     */
    protected void informListers(final String fileName) {

        if (StringUtils.isNoneEmpty(fileName)) {

            final StringBuilder statusTxt = new StringBuilder(50);
            statusTxt.append("File choosen "); //$NON-NLS-1$
            if (fileName.length() > FileLoadDropPastePart.MAX_FILE_NAME_LENGTH) {
                statusTxt.append("...").append(fileName //$NON-NLS-1$
                        .substring(fileName.length() - FileLoadDropPastePart.MAX_FILE_NAME_LENGTH - 3));
            } else {
                statusTxt.append(fileName);
            }
            this.setStatusText(statusTxt.toString());

            this.file = new File(fileName);
            this.notifyListeners();
        }
    }

    /**
     * Benachrichtigen der registrierten Listeners mit bergabe des gefundenen Strings
     */
    private void notifyListeners() {

        for (final FileAddedListener listener : this.listeners) {
            listener.newFileAvailable(this.file);
        }
    }

}
