/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.parts.filedropper;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.FileDialog;

import de.frankmuenster.jameica.depot.utils.Utils;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.logging.Logger;

/**
 * A SelectionAdatper that open a FileDialog and returns the chosen file.
 *
 * @author Frank Mnster
 *
 */
final class OpenFileDialogSelectionAdapter extends SelectionAdapter {
    /**
     *
     */
    private final FileLoadDropPastePart fileLoadDropPastePart;

    /**
     * Constructs a SelectionAdatper that open a FileDialog and returns the
     * chosen file.
     *
     * @param fileLoadDropPastePart
     *            the main part which needs to be informed
     */
    protected OpenFileDialogSelectionAdapter(final FileLoadDropPastePart fileLoadDropPastePart) {
        super();
        this.fileLoadDropPastePart = fileLoadDropPastePart;
    }

    /**
     * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected
     */
    @Override
    public void widgetSelected(final SelectionEvent event) {
        final FileDialog dialog = new FileDialog(GUI.getShell(), SWT.OPEN);
        final String[] extensions = FileLoadDropPastePart.SUPPORTED_TYPES.toArray(new String[0]);
        final String[] filertExt = new String[1];
        filertExt[0] = Utils.getAllCassesForExtensionInString(extensions);
        dialog.setFilterExtensions(filertExt);
        dialog.setFilterNames(new String[] { "CSV, Textdateien " }); //$NON-NLS-1$
        final String file = dialog.open();
        Logger.debug("Selected file: " + file); //$NON-NLS-1$
        if (file != null) {
            this.fileLoadDropPastePart.informListers(file);
        }
    }
}