/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.parts.filedropper;

import java.util.Locale;

import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;

import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.GUI;

final class PasteButtonSelectionAdapter extends SelectionAdapter {

    /**
     * The FileLoadDropPastePart
     */
    private final FileLoadDropPastePart fileLoadDropPastePart;

    /**
     * Constructs a new SelectionAdapter that can be attached to a Button
     *
     * @param fileLoadDropPastePart
     *            the main part which needs to be informed
     */
    protected PasteButtonSelectionAdapter(final FileLoadDropPastePart fileLoadDropPastePart) {

        super();
        this.fileLoadDropPastePart = fileLoadDropPastePart;
    }

    /**
     * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected
     */
    @Override
    public void widgetSelected(final SelectionEvent event) {

        final Clipboard clipboard = new Clipboard(GUI.getDisplay());
        final String[] fileData = (String[]) clipboard.getContents(FileTransfer.getInstance());
        if (fileData == null) {
            this.fileLoadDropPastePart.setStatusText(I18N.tr("PasteButtonSelectionAdapter.pastedContentNotAFile")); //$NON-NLS-1$
        } else {
            boolean isSupported = false;
            // Es wurden Dateien kopiert
            for (final String file : fileData) {
                final String fileType = file.substring(file.lastIndexOf('.') + 1).toLowerCase(Locale.getDefault());
                if (FileLoadDropPastePart.SUPPORTED_TYPES.contains(fileType)) {
                    isSupported = true;
                    this.fileLoadDropPastePart.informListers(file);
                    break;
                }
            }
            if (!isSupported) {
                this.fileLoadDropPastePart.setStatusText(I18N.tr("PasteButtonSelectionAdapter.fileTypeNotSupportted")); //$NON-NLS-1$
            }
        }
    }
}
