/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/
package de.frankmuenster.jameica.depot.gui.parts.grafiken;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

import org.jfree.data.time.Day;
import org.jfree.data.time.TimeSeries;

import de.frankmuenster.jameica.depot.gui.utils.DateUtils;

/**
 * Extended ArrayList mit DateValue Paaren, die eine Kurshistorie abbilden. Zus&auml;tzliche Funktionen liefert Listen
 * nur bestehende aus dem datum, bzw. den Werten.
 *
 * @author Frank Mnster
 */
public class KursDaten extends ArrayList<DateValuePair> {

    /**
     * Serial UID
     */
    private static final long serialVersionUID = 5029613231402146456L;

    /**
     * Standardkonstrukto
     */
    public KursDaten() {
        super();
    }

    /**
     * Liefert eine Liste mit allen Daten
     * 
     * @return Liste mit allen Daten
     */
    public List<LocalDate> getDateList() {
        final List<LocalDate> dateList = new ArrayList<>();
        for (final DateValuePair resultValue : this) {
            dateList.add(resultValue.getDate());
        }
        return dateList;
    }

    /**
     * Liefert eine Liste mit allen Werten
     * 
     * @return Liste mit allen Werten
     */
    public List<Double> getValueList() {
        final List<Double> valueList = new ArrayList<>();
        for (final DateValuePair resultValue : this) {
            valueList.add(resultValue.getValue());
        }
        return valueList;
    }

    /**
     * Erstellt ein TimeSeries Objekt aus eine Kursdaten Objekt
     * 
     * @param titel
     * @return TimeSeries Objekt mit den Kursdaten
     */
    public TimeSeries getTimeSeriesFromKursDaten(final String titel) {
        final TimeSeries serieAnalyse = new TimeSeries(titel);
        for (final DateValuePair value : this) {
            final Day day = new Day(DateUtils.asDate(value.getDate())); // NOPMD
            serieAnalyse.add(day, value.getValue());
        }
        return serieAnalyse;
    }
}
