/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/
package de.frankmuenster.jameica.depot.gui.parts.grafiken;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.concurrent.ConcurrentSkipListMap;

/**
 * Ein TreeMap Objekt f&uuml;r Kusdaten, bestehend aus einem Datum und einem
 * Wert.
 *
 * @author Frank Mnster
 */
public class KursDatenAsMap {

    /**
     * Das interen TreeMap Objekt
     */
    private final Map<Date, Double> valueMap = new ConcurrentSkipListMap<>(); // NOPMD

    /**
     * Standardkonstruktor
     */
    public KursDatenAsMap() {
	// Absichlich leer
    }

    /**
     * F&uuml;gt ein Wertepaar hinzu.
     *
     * @param date
     * @param value
     */
    public void add(final Date date, final Double value) {
	this.valueMap.put(date, value);
    }

    /**
     * L&ouml;scht alle Werte aus der Map.
     */
    public void clear() {
	this.valueMap.clear();
    }

    /**
     * Liefert eine Liste aller Daten aus der Map.
     *
     * @return Liste aller Daten
     */
    public List<Date> getDateList() {

	final List<Date> dateList = new ArrayList<>();

	final Set<Entry<Date, Double>> set2 = this.valueMap.entrySet();
	final Iterator<Entry<Date, Double>> iterator2 = set2.iterator();
	while (iterator2.hasNext()) {
	    final Map.Entry<Date, Double> entry = iterator2.next();
	    dateList.add(entry.getKey());
	}
	return dateList;
    }

    /**
     * Liefert eine Liste aller Werte aus der Map.
     *
     * @return Liste aller Werte
     */
    public List<Double> getValueList() {

	final List<Double> valueList = new ArrayList<>();

	final Set<Entry<Date, Double>> set2 = this.valueMap.entrySet();
	final Iterator<Entry<Date, Double>> iterator2 = set2.iterator();
	while (iterator2.hasNext()) {
	    final Map.Entry<Date, Double> entry = iterator2.next();
	    valueList.add(entry.getValue());
	}
	return valueList;
    }

}
