/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.parts.grafiken.analysegrafik;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
import java.net.URL;
import java.rmi.RemoteException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.jfree.chart.ChartMouseEvent;
import org.jfree.chart.ChartMouseListener;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.annotations.XYTextAnnotation;
import org.jfree.chart.annotations.XYTitleAnnotation;
import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.block.BlockBorder;
import org.jfree.chart.entity.ChartEntity;
import org.jfree.chart.entity.EntityCollection;
import org.jfree.chart.entity.LegendItemEntity;
import org.jfree.chart.plot.CombinedDomainXYPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.PlotRenderingInfo;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.CandlestickRenderer;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.chart.renderer.xy.XYAreaRenderer;
import org.jfree.chart.title.LegendTitle;
import org.jfree.data.Range;
import org.jfree.data.general.DatasetUtilities;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.experimental.chart.swt.ChartComposite;
import org.jfree.ui.Align;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.RectangleInsets;
import org.jfree.ui.TextAnchor;

import de.frankmuenster.jameica.depot.JaDepot;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.AbstractGrafik;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.KursDaten;
import de.frankmuenster.jameica.depot.gui.utils.ColorUtils;
import de.frankmuenster.jameica.depot.kursanalyse.AnalyseNotInListeException;
import de.frankmuenster.jameica.depot.kursanalyse.Analysen;
import de.frankmuenster.jameica.depot.kursanalyse.IKursAnalyse;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseLineStyle;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseTyp;
import de.frankmuenster.jameica.depot.kursanalyse.MethodeAddRemoveListener;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.Part;
import de.willuhn.logging.Logger;

/**
 * Klasse die eine Analysegrafik im JFreeChart format bereit stellt. Dabei
 * werden die Analysen dargestellt, die in der Klasse {@link Analysen} enthalten
 * sind.
 *
 * @author Frank Mnster
 *
 */
@SuppressWarnings({ "PMD.ExcessiveImports", "PMD.GodClass" })
public class AnalyseGrafik extends AbstractGrafik implements Part, ChartMouseListener {

    /**
     * I18N translation key for the date string
     */
    private static final String I18N_DATE = "_date"; //$NON-NLS-1$

    /**
     * Das Schlieen Image
     */
    private static final String CLOSE_IMAGE_NAME = "/img/black-cross.png"; //$NON-NLS-1$

    /**
     * Das Parent Composite
     */
    private transient Composite parent;

    /**
     * Das ChartComposite
     */
    private transient ChartComposite frame;

    /**
     * Anzahl AnalyseCharts
     */
    private transient int anzAnalyseCharts;

    /**
     * Die Listener f&uuml; Methoden Hinzug&uuml;gen/Entfernen Aktionen
     */
    private final transient MethodeAddRemoveListener listener;

    /**
     * Der Schlieen Button
     */
    private transient Image img;

    /**
     * Die Kurswert-Annotion
     */
    private transient XYTextAnnotation kursWertAnno;

    /**
     * @param listener
     */
    public AnalyseGrafik(final MethodeAddRemoveListener listener) {
	super();
	this.listener = listener;
    }

    /**
     * handle mouse click on Chart
     */
    @Override
    public void chartMouseClicked(final ChartMouseEvent event) {

	if (event.getTrigger().getButton() != MouseEvent.BUTTON1) {
	    this.reload();
	    return;
	}

	// Wo wurde geklickt
	final int mouseX = event.getTrigger().getX();
	final int mouseY = event.getTrigger().getY();

	// Das Chart
	final JFreeChart chart = event.getChart();

	// Finde den Plot
	final CombinedDomainXYPlot mainPlot = (CombinedDomainXYPlot) chart.getPlot();

	final int anzSubPlots = mainPlot.getSubplots().size();

	// Welcher Plot wurde angeklickt?
	int clickedPlot = -1;
	Rectangle2D clickedPlotArea = new Rectangle2D.Double();
	for (int i = 0; i < anzSubPlots; i++) {

	    final Rectangle2D plotArea = this.frame.getChartRenderingInfo().getPlotInfo().getSubplotInfo(i)
		    .getDataArea();
	    Logger.warn(plotArea.toString());
	    if (plotArea.contains(mouseX, mouseY)) {
		Logger.debug("Plot " + i + " angeklickt"); //$NON-NLS-1$ //$NON-NLS-2$
		clickedPlot = i;
		clickedPlotArea = plotArea;
		break;
	    }
	}

	if (clickedPlot == 0) {
	    this.removeAnalysisFromMainPlot(mouseX, mouseY);
	} else if (clickedPlot > 0) {
	    this.removeAdditionalAnalysisPlot(mainPlot, clickedPlot, clickedPlotArea, mouseX, mouseY);
	} else {
	    Logger.debug("Kein Plot geklicked"); //$NON-NLS-1$
	}

	XYPlot plot1 = (XYPlot) ((CombinedDomainXYPlot) this.chart.getPlot()).getSubplots().get(0);
	final DateAxis dateAxis = (DateAxis) plot1.getDomainAxis();
	final PlotRenderingInfo plotInfo = this.frame.getChartRenderingInfo().getPlotInfo();
	final double val = dateAxis.java2DToValue(mouseX, plotInfo.getDataArea(), plot1.getDomainAxisEdge());
	final Date xDate = new Date((long) val);
	Logger.debug("val: " + val + "xdate: " + xDate.toString()); //$NON-NLS-1$ //$NON-NLS-2$
	DateUtils.setHours(xDate, 0);
	DateUtils.setMinutes(xDate, 0);
	DateUtils.setSeconds(xDate, 0);
	DateUtils.setMilliseconds(xDate, 0);
	this.reload();

	// Crosshair wieder malen
	plot1 = (XYPlot) ((CombinedDomainXYPlot) this.chart.getPlot()).getSubplots().get(0);
	plot1.setDomainCrosshairVisible(true);
	plot1.setDomainCrosshairValue(xDate.getTime(), true);
	// Y-Wert and CrossHier X Position ermitteln.
	final double kursWert = DatasetUtilities.findYValue(plot1.getDataset(), 0, xDate.getTime() - 60000.0d);
	this.createKursPlotAnnotion(plot1, xDate.getTime(), kursWert);

    }

    /**
     * Aktualisiert das Crosshair &uuml;ber alle Grafiken (senkrechte Linie)
     */
    @Override
    public void chartMouseMoved(final ChartMouseEvent event) {

	final double crossHairX = ((XYPlot) ((CombinedDomainXYPlot) this.chart.getPlot()).getSubplots().get(0))
		.getDomainCrosshairValue();

	// Wo wurde geklickt
	final int mouseX = event.getTrigger().getX();

	Logger.debug("X:" + mouseX + ", xx: " + (long) crossHairX); //$NON-NLS-1$ //$NON-NLS-2$
	Logger.debug("Time: " + new Date((long) crossHairX).toString()); //$NON-NLS-1$

	final XYPlot kursPlot = (XYPlot) ((CombinedDomainXYPlot) this.chart.getPlot()).getSubplots().get(0);
	final DateAxis dateAxis = (DateAxis) kursPlot.getDomainAxis();

	final Date minDate = dateAxis.getMinimumDate();
	final Date maxDate = dateAxis.getMaximumDate();

	final PlotRenderingInfo plotInfo = this.frame.getChartRenderingInfo().getPlotInfo();

	final int minX = (int) dateAxis.dateToJava2D(minDate, plotInfo.getDataArea(), kursPlot.getDomainAxisEdge());
	final int maxX = (int) dateAxis.dateToJava2D(maxDate, plotInfo.getDataArea(), kursPlot.getDomainAxisEdge());

	Logger.debug("xMinDate: " + minX + ", xMaxDate: " + maxX); //$NON-NLS-1$ //$NON-NLS-2$

	final double val = dateAxis.java2DToValue(mouseX, plotInfo.getDataArea(), kursPlot.getDomainAxisEdge());
	final Date xDate = new Date((long) val);

	DateUtils.setHours(xDate, 0);
	DateUtils.setMinutes(xDate, 0);
	DateUtils.setSeconds(xDate, 0);
	DateUtils.setMilliseconds(xDate, 0);

	// Setzte Crosshair in kursPlot
	kursPlot.setDomainCrosshairValue(xDate.getTime(), true);

	// Y-Wert and CrossHier X Position ermitteln.
	final double kursWert = DatasetUtilities.findYValue(kursPlot.getDataset(), 0, xDate.getTime() - 60000.0d);
	Logger.debug("val: " + val + "val1: " + kursWert + "xdate: " //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		+ xDate.toString());

	this.createKursPlotAnnotion(kursPlot, xDate.getTime(), kursWert);

	// Und nun in allen anderen Plots
	@SuppressWarnings("unchecked")
	final List<XYPlot> subPlots = ((CombinedDomainXYPlot) this.chart.getPlot()).getSubplots();
	for (final XYPlot p : subPlots) {
	    p.setDomainCrosshairValue(xDate.getTime(), true);
	}

    }

    /**
     * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {

	this.parent = parent;

	final CombinedDomainXYPlot plot = new CombinedDomainXYPlot(new DateAxis(I18N.tr(AnalyseGrafik.I18N_DATE)));
	plot.setGap(10.0);

	// return a new chart containing the overlaid plot...
	this.chart = new JFreeChart(this.title, JFreeChart.DEFAULT_TITLE_FONT, plot, true);
	this.chart.removeLegend();

	this.frame = new ChartComposite(this.parent, SWT.NONE, this.chart, false);
	this.frame.addChartMouseListener(this);

	final GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
	gridData.heightHint = 300 + this.anzAnalyseCharts * 200;
	this.frame.setLayout(new FillLayout());
	this.frame.setLayoutData(gridData);

	this.frame.pack();
	this.frame.setVisible(true);

	this.frame.setDomainZoomable(false);
	this.frame.setRangeZoomable(false);

    }

    /**
     * Zeichnet die Grafik neu
     */
    @Override
    public void reload() {

	if (this.frame != null) {

	    this.frame.setVisible(false);
	    CombinedDomainXYPlot mainPlot = (CombinedDomainXYPlot) this.chart.getPlot();

	    // Entferne all Plots
	    while (!mainPlot.getSubplots().isEmpty()) {
		final XYPlot plot = (XYPlot) mainPlot.getSubplots().get(0);
		mainPlot.remove(plot);

	    }

	    this.frame.dispose();

	    mainPlot = new CombinedDomainXYPlot(new DateAxis(I18N.tr(AnalyseGrafik.I18N_DATE)));
	    mainPlot.setGap(10.0);

	    // return a new chart containing the overlaid plot...
	    this.chart = new JFreeChart(this.title, JFreeChart.DEFAULT_TITLE_FONT, mainPlot, false);

	    this.frame = new ChartComposite(this.parent, SWT.NONE, this.chart, false);
	    this.frame.setVisible(false);
	    this.frame.addChartMouseListener(this);

	    this.frame.setDomainZoomable(false);
	    this.frame.setRangeZoomable(false);

	    // Erzeuge die Plots
	    this.createCombinedChart(mainPlot);

	    final GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
	    if (this.anzAnalyseCharts == 0) {
		gridData.heightHint = 350;
	    } else {
		gridData.heightHint = 200 + this.anzAnalyseCharts * 150;
	    }

	    // Datumanzeige anpassen
	    final DateAxis dateAxis = (DateAxis) mainPlot.getDomainAxis();
	    final Date min = dateAxis.getMinimumDate();
	    final Date max = dateAxis.getMaximumDate();
	    final long diff = max.getTime() - min.getTime();
	    final long oneYear = 1000l * 60 * 60 * 24 * 365;

	    SimpleDateFormat dateFormat;
	    if (diff > oneYear) {
		dateFormat = new SimpleDateFormat("MM-yy", Locale.GERMAN); //$NON-NLS-1$
		dateAxis.setLabel(I18N.tr("AnalyseGrafik.lbldateAxisMonthYear")); //$NON-NLS-1$
	    } else {
		dateFormat = new SimpleDateFormat("dd.MM", Locale.GERMAN); //$NON-NLS-1$
		final int yearMin = de.frankmuenster.jameica.depot.gui.utils.DateUtils.getYear(min);
		final int yearMax = de.frankmuenster.jameica.depot.gui.utils.DateUtils.getYear(max);
		if (yearMin == yearMax) {
		    dateAxis.setLabel(I18N.tr("AnalyseGrafik.lbldateAxisDayInMonthOfYear", //$NON-NLS-1$
			    Integer.toString(yearMin)));
		} else {
		    dateAxis.setLabel(I18N.tr("AnalyseGrafik.lbldateAxisDayInMonthOfYears", //$NON-NLS-1$
			    Integer.toString(yearMin), Integer.toString(yearMax)));
		}
	    }
	    dateAxis.setDateFormatOverride(dateFormat);

	    this.frame.setLayout(new FillLayout());
	    this.frame.setLayoutData(gridData);

	    this.frame.update();
	    this.frame.pack();

	    this.frame.getParent().pack();
	    this.frame.getParent().update();
	    this.frame.getParent().getParent().pack();
	    this.frame.getParent().getParent().update();

	    this.frame.setVisible(true);
	}

    }

    /**
     * F&uuml;gt eine Analyse in den Kurs-Chart-Plot ein.
     *
     * @param iKursAnalyse
     *            die Kursanalyse
     * @param subplotKurs
     *            der Kurs-Chart-Plot
     * @param anzKursPlotLines
     *            die Nummer der Linie an
     */
    @SuppressWarnings("PMD.AvoidInstantiatingObjectsInLoops")
    private void addAnalyseToPLot(final IKursAnalyse<?, ?> iKursAnalyse, final XYPlot subplotKurs,
	    final int anzKursPlotLines) {

	final BasicStroke dotted = new BasicStroke(2.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 1.0f,
		new float[] { 1.0f, 2.0f }, 0.0f);

	Logger.debug("Adding OVERLAY_LINE " + iKursAnalyse.getLegendTitel()); //$NON-NLS-1$

	final KursDaten[] kursDaten = (KursDaten[]) iKursAnalyse.getResult(this.displayPeriod);
	for (int i = 0; i < kursDaten.length; i++) {
	    // Datenwerte
	    final KursDaten analyseDaten = kursDaten[i];
	    final TimeSeries timeSeries = analyseDaten.getTimeSeriesFromKursDaten(iKursAnalyse.getLegendTitel());
	    final TimeSeriesCollection series = new TimeSeriesCollection();
	    series.addSeries(timeSeries);

	    // Renderer
	    final StandardXYItemRenderer rendererLine = new StandardXYItemRenderer();
	    rendererLine.setSeriesPaint(0, ColorUtils.toAwtColor(iKursAnalyse.getColor()[i]));
	    if (i > 0) {
		rendererLine.setSeriesVisibleInLegend(0, false);
	    }
	    // LineStyle
	    if (iKursAnalyse.getLineStyle() != null) {
		final KursAnalyseLineStyle[] lineStyles = iKursAnalyse.getLineStyle();
		if (lineStyles[i].equals(KursAnalyseLineStyle.DOTTED)) {
		    rendererLine.setSeriesStroke(0, dotted);
		}
	    }

	    // Linie hinzufgen
	    subplotKurs.setDataset(anzKursPlotLines + i, series);
	    subplotKurs.setRenderer(anzKursPlotLines + i, rendererLine);

	}
    }

    private CombinedDomainXYPlot createCombinedChart(final CombinedDomainXYPlot mainPlot) {

	this.anzAnalyseCharts = 0;
	int anzKursPlotLines = 0;
	final List<XYPlot> analysePlots = new ArrayList<>();

	final DateAxis dateAxisKurs = new DateAxis(I18N.tr(AnalyseGrafik.I18N_DATE));
	final NumberAxis rangeAxisKurs = new NumberAxis(I18N.tr("AnalyseGrafik.axisValue")); //$NON-NLS-1$
	dateAxisKurs.setLabelFont(AbstractGrafik.FONT_AXIS);
	rangeAxisKurs.setLabelFont(AbstractGrafik.FONT_AXIS);
	rangeAxisKurs.setAutoRangeIncludesZero(false);

	final XYPlot subplotKurs = new XYPlot(null, dateAxisKurs, rangeAxisKurs, null);

	subplotKurs.setRangeAxisLocation(AxisLocation.BOTTOM_OR_LEFT);

	subplotKurs.setDomainCrosshairVisible(true);
	subplotKurs.setDomainCrosshairLockedOnData(false);

	for (final IKursAnalyse<?, ?> iKursAnalyse : Analysen.getInstance()) {

	    if (iKursAnalyse.getType().equals(KursAnalyseTyp.OVERLAY_LINE) && !this.isShowDax()) {
		this.addAnalyseToPLot(iKursAnalyse, subplotKurs, anzKursPlotLines);
		anzKursPlotLines += iKursAnalyse.getLineCount();
	    } else if (iKursAnalyse.getType().equals(KursAnalyseTyp.OVERLAY_BAR)) {
		Logger.warn(I18N.tr("AnalyseGrafik.overlayNotImplement")); //$NON-NLS-1$
	    } else if (iKursAnalyse.getType().equals(KursAnalyseTyp.SEPERATE_GRAPH)) {
		final XYPlot analysePlot = this.createSeperateAnaylsePlot(iKursAnalyse);
		analysePlots.add(analysePlot);
	    }
	}

	// Kurschart Dataset und Renderer setzten abhngig vom Charttyp
	this.setKursChartDatasetAndRenderer(subplotKurs, anzKursPlotLines);

	// Legende hinzufgen
	final LegendTitle legendTitle = new LegendTitle(subplotKurs);
	legendTitle.setItemFont(new Font("Dialog", Font.PLAIN, 9)); //$NON-NLS-1$
	legendTitle.setBackgroundPaint(new Color(200, 200, 255, 100));
	legendTitle.setFrame(new BlockBorder(Color.white));
	legendTitle.setPosition(RectangleEdge.BOTTOM);
	final RectangleInsets padding = new RectangleInsets(4, 2, 0, 10);
	legendTitle.setItemLabelPadding(padding);
	final XYTitleAnnotation titleAnnotation = new XYTitleAnnotation(1.0, 0.00, legendTitle,
		RectangleAnchor.BOTTOM_RIGHT);

	titleAnnotation.setMaxWidth(0.48);
	subplotKurs.addAnnotation(titleAnnotation);

	// Datenquelle hinzufgen
	if (StringUtils.isNotEmpty(this.dataSource)) {
	    final Range domainRange = subplotKurs.getDomainAxis().getRange();
	    final Range rangeRange = subplotKurs.getRangeAxis().getRange();
	    final Double xPos = domainRange.getLowerBound() + domainRange.getLength() * 0.01;
	    final Double yPos = rangeRange.getLowerBound() + rangeRange.getLength() * 0.03;
	    final XYTextAnnotation dataSrcTxt = new XYTextAnnotation(this.dataSource, xPos, yPos);
	    dataSrcTxt.setFont(new Font("Serif", Font.PLAIN, 9)); //$NON-NLS-1$
	    dataSrcTxt.setTextAnchor(TextAnchor.BASELINE_LEFT);
	    dataSrcTxt.setText(I18N.tr("AnalyseGrafik.dataSource", this.dataSource)); //$NON-NLS-1$
	    subplotKurs.addAnnotation(dataSrcTxt);
	}

	// Der Kurs-Chart-Plot in dem Hauptplot setzten
	mainPlot.add(subplotKurs, 4);

	// Und nun die Analyseplots drunter
	for (final XYPlot xyPlot : analysePlots) {
	    xyPlot.setDomainCrosshairVisible(true);
	    xyPlot.setDomainCrosshairLockedOnData(false);
	    mainPlot.add(xyPlot, 3);
	}
	mainPlot.setOrientation(PlotOrientation.VERTICAL);

	return mainPlot;

    }

    /**
     * Den Kurswert als Annotation ausgegeben.
     *
     * @param kursPlot
     *            Plot
     * @param mouseX
     *            X-Position
     * @param kursWert
     *            der Kurswert
     */
    private void createKursPlotAnnotion(final XYPlot kursPlot, final double mouseX, final double kursWert) {

	if (this.kursWertAnno != null) {
	    kursPlot.removeAnnotation(this.kursWertAnno);
	}

	if (!this.showCandle) {
	    final StringBuilder annotationValue = new StringBuilder(AbstractGrafik.DECIMAL_FOMAT.format(kursWert));
	    if (this.showDax) {
		annotationValue.append(" %"); //$NON-NLS-1$
	    } else {
		annotationValue.append(" EUR"); //$NON-NLS-1$
	    }
	    this.kursWertAnno = new XYTextAnnotation(annotationValue.toString(), mouseX + 5, kursWert);
	    this.kursWertAnno.setFont(new Font("SansSerif", Font.BOLD, 12)); //$NON-NLS-1$
	    this.kursWertAnno.setPaint(Color.blue);
	    this.kursWertAnno.setTextAnchor(TextAnchor.BOTTOM_LEFT);
	    kursPlot.addAnnotation(this.kursWertAnno);
	}

    }

    private XYPlot createSeperateAnaylsePlot(final IKursAnalyse<?, ?> iKursAnalyse) {

	Logger.debug("Adding SEPERATE_GRAPH " + iKursAnalyse.getLegendTitel()); //$NON-NLS-1$

	this.anzAnalyseCharts++;
	final XYPlot analysePlot = (XYPlot) iKursAnalyse.getResult(this.displayPeriod);

	analysePlot.getRangeAxis().setLabelFont(AbstractGrafik.FONT_AXIS);
	if (analysePlot.getDomainAxis() != null) {
	    analysePlot.getDomainAxis().setLabelFont(AbstractGrafik.FONT_AXIS);
	}

	if (this.img == null) {
	    // schlieen Knopf
	    final URL imgURL = JaDepot.class.getResource(AnalyseGrafik.CLOSE_IMAGE_NAME);
	    final Toolkit toolkit = Toolkit.getDefaultToolkit();
	    this.img = toolkit.getImage(imgURL);

	    // Mit Mediatracker warten, bis das Bild geladen ist.
	    final Component component = new Component() {
		private static final long serialVersionUID = 1L;
	    };
	    final MediaTracker tracker = new MediaTracker(component);
	    tracker.addImage(this.img, 0);
	    try {
		tracker.waitForAll();
	    } catch (final InterruptedException ie) {
		Logger.warn("Interrupted!"); //$NON-NLS-1$
		// Restore interrupted state...
		Thread.currentThread().interrupt();
	    }

	}
	// ...und das Bild nun malen
	analysePlot.setBackgroundImage(this.img);
	analysePlot.setBackgroundImageAlignment(Align.TOP_RIGHT);

	return analysePlot;
    }

    /**
     * Entfernt eine kompletten AnalysePlot
     *
     * @param mainPlot
     *            der Haupt-Plot mit allen Grafiken
     * @param clickedPlot
     *            die Sub-Plot Nummer
     * @param clickedPlotArea
     *            das Rectangle2D des Sub-Plots
     * @param mouseX
     *            Maus X
     * @param mouseY
     *            Maus Y
     */
    private void removeAdditionalAnalysisPlot(final CombinedDomainXYPlot mainPlot, final int clickedPlot,
	    final Rectangle2D clickedPlotArea, final int mouseX, final int mouseY) {

	// hole Plot und teste ob es ein XYPlot ist
	final Object plot = mainPlot.getSubplots().get(clickedPlot);
	if (plot instanceof XYPlot) {
	    // Rectangle der CloseBox berechnen
	    final double closeBoxX = clickedPlotArea.getBounds2D().getX() + clickedPlotArea.getBounds2D().getWidth()
		    - 20.0;
	    final double closeBoxY = clickedPlotArea.getBounds2D().getY();
	    final Rectangle2D closeBox = new Rectangle2D.Double(closeBoxX, closeBoxY, 20, 20);

	    // Test ob close-Button geklickt wurde
	    if (closeBox.contains(mouseX, mouseY)) {
		// hole PlotKey
		final String plotKey = ((XYPlot) plot).getDataset(0).getSeriesKey(0).toString();

		Logger.info(I18N.tr("AnalyseGrafik.removeAnalysis") + plotKey); //$NON-NLS-1$
		// Remove Analyse, falls vorhanden
		if (Analysen.getInstance().conatinsAnalyseWithLegendTitel(plotKey)) {
		    try {
			final IKursAnalyse<?, ?> ana = Analysen.getInstance().getAnalyseWithLegendTitel(plotKey);
			this.listener.notifyRemoveMethode(ana);
		    } catch (final AnalyseNotInListeException e) {
			Logger.debug("Analysis not in list"); //$NON-NLS-1$
		    }
		}
	    }
	}

    }

    /**
     * Entfernt ein Analyse aus dem Main-Plot
     *
     * @param mouseX
     * @param mouseY
     */
    private void removeAnalysisFromMainPlot(final int mouseX, final int mouseY) {

	// leider muessen wir ueber alle Entities iterieren
	final EntityCollection entities = this.frame.getChartRenderingInfo().getEntityCollection();

	@SuppressWarnings("unchecked")
	final Iterator<ChartEntity> chartEntities = entities.iterator();

	while (chartEntities.hasNext()) {

	    final ChartEntity chartEntity = chartEntities.next();

	    // Wir suchen nur die LegenItemEntitys
	    if (chartEntity instanceof LegendItemEntity) {

		final LegendItemEntity lie = (LegendItemEntity) chartEntity;
		final Rectangle2D legendItemArea = lie.getArea().getBounds2D();

		if (legendItemArea.contains(mouseX, mouseY)) {
		    final String analyseTitel = lie.getSeriesKey().toString();
		    Logger.info(I18N.tr("AnalyseGrafik.removeAnalysis") + analyseTitel); //$NON-NLS-1$
		    if (Analysen.getInstance().conatinsAnalyseWithLegendTitel(analyseTitel)) {
			try {
			    final IKursAnalyse<?,
				    ?> ana = Analysen.getInstance().getAnalyseWithLegendTitel(analyseTitel);
			    this.listener.notifyRemoveMethode(ana);
			} catch (final AnalyseNotInListeException e) {
			    Logger.debug("Analysis not in list"); //$NON-NLS-1$
			}
		    }

		}
	    }
	}

    }

    /**
     * Kurschart Dataset und Renderer setzten abhngig vom Charttyp.
     *
     * @param subplotKurs
     *            der Kurs-Chart-Plot
     * @param anzKursPlotLines
     *            Anzahl vorhandener Plotlinien
     */
    private void setKursChartDatasetAndRenderer(final XYPlot subplotKurs, final int anzKursPlotLines) {

	if (this.showCandle) {

	    final CandlestickRenderer rendererCandle = new CandlestickRenderer();
	    rendererCandle.setDrawVolume(true);
	    rendererCandle.setSeriesPaint(0, Color.lightGray);
	    rendererCandle.setSeriesVisibleInLegend(0, false);
	    rendererCandle.setBaseToolTipGenerator(null);

	    subplotKurs.setDataset(anzKursPlotLines, this.getDatasetCandleStick());
	    subplotKurs.setRenderer(anzKursPlotLines, rendererCandle);

	} else if (this.showDax) {
	    final StandardXYItemRenderer rendererKurs = new StandardXYItemRenderer();
	    final StandardXYItemRenderer rendererDax = new StandardXYItemRenderer();
	    rendererKurs.setSeriesPaint(0, Color.GREEN);
	    rendererDax.setSeriesPaint(1, Color.BLUE);

	    final TimeSeriesCollection seriesKurs = new TimeSeriesCollection();
	    final TimeSeriesCollection seriesDax = new TimeSeriesCollection();
	    seriesKurs.addSeries(this.getDatasetKursDaxVergleich().getSeries(0));
	    seriesDax.addSeries(this.getDatasetKursDaxVergleich().getSeries(1));

	    subplotKurs.setDataset(anzKursPlotLines, seriesKurs);
	    subplotKurs.setRenderer(anzKursPlotLines, rendererKurs);
	    subplotKurs.setDataset(anzKursPlotLines + 1, seriesDax);
	    subplotKurs.setRenderer(anzKursPlotLines + 1, rendererDax);

	    subplotKurs.getRangeAxis().setLabel(I18N.tr("AnalyseGrafik.procentageAxisLabel")); //$NON-NLS-1$

	} else {
	    final XYAreaRenderer rendererKurs = new XYAreaRenderer();
	    rendererKurs.setSeriesPaint(0, Color.lightGray);
	    rendererKurs.setSeriesFillPaint(0, Color.lightGray);
	    rendererKurs.setSeriesOutlinePaint(0, Color.darkGray);
	    rendererKurs.setOutline(true);
	    rendererKurs.setSeriesVisibleInLegend(0, false);

	    subplotKurs.setDataset(anzKursPlotLines, this.getDataSetKurs());
	    subplotKurs.setRenderer(anzKursPlotLines, rendererKurs);
	}
    }

}
