/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.parts.grafiken.kursgrafik;

import java.awt.Color;
import java.awt.Font;
import java.awt.Point;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.rmi.RemoteException;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartMouseEvent;
import org.jfree.chart.ChartMouseListener;
import org.jfree.chart.ChartRenderingInfo;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.annotations.XYTextAnnotation;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.entity.ChartEntity;
import org.jfree.chart.entity.PlotEntity;
import org.jfree.chart.entity.XYItemEntity;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.chart.plot.PlotRenderingInfo;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.CandlestickRenderer;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.data.Range;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.OHLCDataset;
import org.jfree.data.xy.XYDataset;
import org.jfree.experimental.chart.swt.ChartComposite;
import org.jfree.ui.TextAnchor;

import de.frankmuenster.jameica.depot.gui.parts.grafiken.AbstractGrafik;
import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.frankmuenster.jameica.depot.gui.utils.ItemUtils;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.Part;

/**
 * Die Chartgrafik f&uuml;r die WertpapierDeatil View.
 *
 * @author Frank Mnster
 */
@SuppressWarnings("PMD.ExcessiveImports")
public class KursGrafik extends AbstractGrafik implements ChartMouseListener, Part {

    /**
     * I18n translation key for the chart time axis label
     */
    private static final String I18N_TIME_AXIS_LABEL = "KursGrafik.chartTimeAxisLabel"; //$NON-NLS-1$

    /**
     * I18n translation key for the chart title
     */
    private static final String I18N_TITLE = "KursGrafik.chartTitle"; //$NON-NLS-1$

    /**
     * Unser Composite in dem wir malen
     */
    private transient Composite body;

    /**
     * Das ChartComposite
     */
    private transient ChartComposite frame;

    /**
     * Standardkonstruktor
     */
    public KursGrafik() {
	super();
    }

    /**
     * @see org.jfree.chart.ChartMouseListener#chartMouseClicked(org.jfree.chart.ChartMouseEvent)
     */
    @Override
    public void chartMouseClicked(final ChartMouseEvent arg0) {
	final double xValue = ((XYPlot) this.chart.getPlot()).getDomainCrosshairValue();
	final double yValue = ((XYPlot) this.chart.getPlot()).getRangeCrosshairValue();
	this.reload();
	((XYPlot) this.chart.getPlot()).setDomainCrosshairValue(xValue);
	((XYPlot) this.chart.getPlot()).setRangeCrosshairValue(yValue);

    }

    /**
     * @see org.jfree.chart.ChartMouseListener#chartMouseMoved(org.jfree.chart.ChartMouseEvent)
     */
    @Override
    public void chartMouseMoved(final ChartMouseEvent event) {

	final XYPlot cdPlot = (XYPlot) this.chart.getPlot();
	final ChartEntity entity = event.getEntity();

	if (entity instanceof PlotEntity || entity instanceof XYItemEntity) {

	    if (!cdPlot.isDomainCrosshairVisible()) {
		cdPlot.setDomainCrosshairVisible(true);
		cdPlot.setRangeCrosshairVisible(true);
	    }
	    final int mouseX = event.getTrigger().getX();
	    final int mouseY = event.getTrigger().getY();
	    final Point mousePoint = new Point(mouseX, mouseY);

	    // convert the Java2D coordinate to axis coordinates...
	    final ChartRenderingInfo chartInfo = this.frame.getChartRenderingInfo();
	    final Point2D java2DPoint = this.frame.translateScreenToJava2D(mousePoint);
	    final PlotRenderingInfo plotInfo = chartInfo.getPlotInfo();

	    // all subplots have the domain crosshair
	    // the x coordinate is the same for all subplots
	    final Rectangle2D dataArea = plotInfo.getDataArea();
	    final double x2DValue = cdPlot.getDomainAxis().java2DToValue(java2DPoint.getX(), dataArea,
		    cdPlot.getDomainAxisEdge());

	    final org.eclipse.swt.graphics.Rectangle rect = this.frame.getScreenDataArea(mouseX, mouseY);
	    final Rectangle2D panelArea = new Rectangle2D.Double();
	    panelArea.setRect(rect.x, rect.y, rect.width, rect.height);

	    final double y2DValue = cdPlot.getRangeAxis().java2DToValue(mousePoint.getY(), panelArea,
		    cdPlot.getRangeAxisEdge());

	    cdPlot.setDomainCrosshairValue(x2DValue, true);
	    cdPlot.setRangeCrosshairValue(y2DValue, true);
	} else {
	    cdPlot.setDomainCrosshairVisible(false);
	    cdPlot.setRangeCrosshairVisible(false);
	}

    }

    /**
     * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {

	this.body = new Composite(parent, SWT.NONE);
	this.body.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
	this.body.setLayout(new FillLayout());

	this.reload();
    }

    /**
     *
     * @see de.frankmuenster.jameica.depot.gui.parts.grafiken.AbstractGrafik#reload()
     */
    @Override
    public void reload() {

	JFreeChart chart;

	for (final Control element : this.body.getChildren()) {
	    element.dispose();
	}

	if (this.showCandle) {
	    chart = this.getCandleStickChart(this.getDatasetCandleStick());
	} else if (this.showDax) {
	    chart = this.getDaxChart(this.getDatasetKursDaxVergleich());
	} else {
	    chart = this.getKursChart(this.getDataSetKurs());
	}

	final XYPlot mainPlot = (XYPlot) chart.getPlot();

	mainPlot.setDomainCrosshairVisible(true);
	mainPlot.setDomainCrosshairLockedOnData(true);
	mainPlot.setRangeCrosshairVisible(true);

	// Datumanzeige anpassen
	final DateAxis dateAxis = (DateAxis) mainPlot.getDomainAxis();
	final Date min = dateAxis.getMinimumDate();
	final Date max = dateAxis.getMaximumDate();
	final long diff = max.getTime() - min.getTime();
	final long oneYear = 1000l * 60 * 60 * 24 * 365;

	SimpleDateFormat dateFormat;
	if (diff > oneYear) {
	    dateFormat = new SimpleDateFormat("MMM-yy", Locale.GERMAN); //$NON-NLS-1$
	} else {
	    dateFormat = new SimpleDateFormat("dd.MMM", Locale.GERMAN); //$NON-NLS-1$
	    final Calendar calendar = new GregorianCalendar();
	    calendar.setTime(min);
	    final int yearMin = calendar.get(Calendar.YEAR);
	    calendar.setTime(max);
	    final int yearMax = calendar.get(Calendar.YEAR);
	    if (yearMin == yearMax) {
		dateAxis.setLabel(I18N.tr("KursGrafik.dayOfMonthInYear" //$NON-NLS-1$
			, Integer.toString(yearMin)));
	    } else {
		dateAxis.setLabel(I18N.tr("KursGrafik.dayOfMonthInYears", //$NON-NLS-1$
			Integer.toString(yearMin), Integer.toString(yearMax)));
	    }
	}
	dateAxis.setDateFormatOverride(dateFormat);

	chart.setTitle(StringUtils.EMPTY);

	// Hauptlegende entfernen
	chart.removeLegend();

	// Datenquelle hinzufgen
	if (StringUtils.isNotEmpty(this.dataSource)) {
	    final Range domainRange = mainPlot.getDomainAxis().getRange();
	    final Range rangeRange = mainPlot.getRangeAxis().getRange();
	    final Double xPos = domainRange.getLowerBound() + domainRange.getLength() * 0.01;
	    final Double yPos = rangeRange.getLowerBound() + rangeRange.getLength() * 0.03;
	    final XYTextAnnotation dataSrcTxt = new XYTextAnnotation(this.dataSource, xPos, yPos);
	    dataSrcTxt.setFont(new Font("Serif", Font.PLAIN, 9)); //$NON-NLS-1$
	    dataSrcTxt.setTextAnchor(TextAnchor.BASELINE_LEFT);
	    dataSrcTxt.setText(I18N.tr("KursGrafik.dataSource", this.dataSource)); //$NON-NLS-1$
	    mainPlot.addAnnotation(dataSrcTxt);
	}

	this.chart = chart;

	this.frame = new ChartComposite(this.body, SWT.NONE, chart, true);
	this.frame.addChartMouseListener(this);

	this.body.layout(true, true);

    }

    /**
     * Liefert das Chart als Kerzengrafik.
     *
     * @param dataset
     * @return das Chart als Kerzengrafik
     */
    private JFreeChart getCandleStickChart(final OHLCDataset dataset) {

	final JFreeChart chart = ChartFactory.createCandlestickChart(I18N.tr(KursGrafik.I18N_TITLE),
		I18N.tr(KursGrafik.I18N_TIME_AXIS_LABEL), I18N.tr("KursGrafik.chartValueAxisLabel"), //$NON-NLS-1$
		dataset, true);

	// Common basic chart settings
	setChartBasics(chart);

	// Now volume drawn
	final CandlestickRenderer renderer = (CandlestickRenderer) ((XYPlot) chart.getPlot()).getRenderer();
	renderer.setDrawVolume(true);

	return chart;
    }

    /**
     * Liefert das DAX Chart
     *
     * @param dataset
     * @return das DAX Chart
     */
    private JFreeChart getDaxChart(final TimeSeriesCollection dataset) {

	final JFreeChart chart = ChartFactory.createTimeSeriesChart(I18N.tr(KursGrafik.I18N_TITLE),
		I18N.tr(KursGrafik.I18N_TIME_AXIS_LABEL), I18N.tr("KursGrafik.chartProcentageAxisLabel"), //$NON-NLS-1$
		dataset, true, true, false);

	// Common basic chart settings
	setChartBasics(chart);

	// Common time series chart settings
	setChartTimeSeriesBasics(chart);

	// Legende hinzufgen
	final XYPlot plot = (XYPlot) chart.getPlot();
	ItemUtils.addLegendTitle(plot);

	return chart;
    }

    /**
     * Liefert das KursChart
     *
     * @param dataset
     * @return das KursChart
     */
    private JFreeChart getKursChart(final XYDataset dataset) {

	final JFreeChart chart = ChartFactory.createTimeSeriesChart(I18N.tr(KursGrafik.I18N_TITLE),
		I18N.tr(KursGrafik.I18N_TIME_AXIS_LABEL), I18N.tr("KursGrafik.chartValueAxisLabel"), //$NON-NLS-1$
		dataset, true, true, false);

	// Common basic chart settings
	setChartBasics(chart);

	// Common time series chart settings
	setChartTimeSeriesBasics(chart);

	return chart;
    }

    private void setChartBasics(final JFreeChart chart) {

	// Set chart background
	chart.setBackgroundPaint(Color.white);

	// Set a few custom plot features
	final XYPlot plot = (XYPlot) chart.getPlot();
	plot.setBackgroundPaint(Color.WHITE);
	plot.setDomainGridlinesVisible(true);
	plot.setDomainGridlinePaint(Color.lightGray);
	plot.setRangeGridlinePaint(Color.lightGray);
	((NumberAxis) plot.getRangeAxis()).setAutoRangeIncludesZero(false);
	((NumberAxis) plot.getRangeAxis()).setLabelFont(AbstractGrafik.FONT_AXIS);
	((DateAxis) plot.getDomainAxis()).setLabelFont(AbstractGrafik.FONT_AXIS);

    }

    private void setChartTimeSeriesBasics(final JFreeChart chart) {

	final XYPlot plot = (XYPlot) chart.getPlot();

	((DateAxis) plot.getDomainAxis()).setDateFormatOverride(new SimpleDateFormat("dd.MM", Locale.GERMAN)); //$NON-NLS-1$
	((DateAxis) plot.getDomainAxis()).setLabelFont(AbstractGrafik.FONT_AXIS);

	// Nice tool tip format
	final XYItemRenderer renderer = chart.getXYPlot().getRenderer();
	final StandardXYToolTipGenerator toolTipGenerator = new StandardXYToolTipGenerator(
		StandardXYToolTipGenerator.DEFAULT_TOOL_TIP_FORMAT, FormatterUtils.GERMAN_DATE_FORMAT.get(),
		new DecimalFormat("0.00 EUR")); //$NON-NLS-1$
	renderer.setBaseToolTipGenerator(toolTipGenerator);

    }

}
