/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/


package de.frankmuenster.jameica.depot.gui.parts.wpsearch;

import java.util.List;

import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.boerseard.ArdStockInfoBean;
import de.frankmuenster.jameica.finanzen.boerseard.BoerseArdApi;
import de.willuhn.jameica.gui.Part;

/**
 * @author Frank Mnster
 */
public class BoerseArdStockSearchPart extends AbstractStockSearchPart<ArdStockInfoBean> implements Part {

    /**
     * Der Dialogtitel
     */
    private static final String DIALOG_TITLE = "ARD Brseninfo Suche"; //$NON-NLS-1$

    /**
     * Standardkonstruktor mit Angabe eines Listeners
     *
     * @param listener
     *            der zu benachrichtigende Listener
     */
    public BoerseArdStockSearchPart(final SearchResultSelectListener<ArdStockInfoBean> listener) {
	this(listener, 7);
    }

    /**
     * Standardkonstruktor mit Angabe eines Listeners
     *
     * @param listener
     *            der zu benachrichtigende Listener
     * @param resultRows
     */
    public BoerseArdStockSearchPart(final SearchResultSelectListener<ArdStockInfoBean> listener, final int resultRows) {
	super(listener, resultRows);
    }

    /**
     * @see AbstractStockSearchPart#getTitle()
     */
    @Override
    public String getTitle() {
	return BoerseArdStockSearchPart.DIALOG_TITLE;
    }

    /**
     * @see AbstractStockSearchPart#formatListEntry(de.frankmuenster.jameica.finanzen.StockInfoBean)
     */
    @Override
    protected String formatListEntry(final ArdStockInfoBean bean) {
	final StringBuilder result = new StringBuilder(37);
	StringBuilder help = new StringBuilder(bean.getName()).append(AbstractStockSearchPart.SPACE_20);
	result.append(help.substring(0, 20)).append(AbstractStockSearchPart.SPACE_3);
	help = new StringBuilder(bean.getType()).append(AbstractStockSearchPart.SPACE_10);
	result.append(help.substring(0, 10)).append(AbstractStockSearchPart.SPACE_3);
	return result.toString();
    }

    /**
     * @see AbstractStockSearchPart#perfromSearch(java.lang.String)
     */
    @Override
    protected List<ArdStockInfoBean> perfromSearch(final String search) throws FinanceException {
	return BoerseArdApi.getInstance().getSearchResults(search);
    }
}
