/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.parts.wpsearch;

import java.util.List;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.finanzen.boerseffm.FfmSearchWertpapier;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.FfmStockInfoBean;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.willuhn.jameica.gui.Part;

/**
 * Part das es erlaubt nach einem Wertpapier bei der Brese FFM zu suchen.
 *
 * @author Frank Mnster
 */
public class FfmSearchPart extends AbstractStockSearchPart<FfmStockInfoBean> implements Part {

    /**
     * Der Dialogtitel
     */
    private static final String DIALOG_TITLE = "Brese FFM Suche"; //$NON-NLS-1$

    /**
     * Standardkonstruktor mit Angabe eines Listeners
     *
     * @param listener
     *            der zu benachrichtigende Listener
     */
    public FfmSearchPart(final SearchResultSelectListener<FfmStockInfoBean> listener) {
	this(listener, 7);
    }

    /**
     * Standardkonstruktor mit Angabe eines Listeners
     *
     * @param listener
     *            der zu benachrichtigende Listener
     * @param resultRows
     */
    public FfmSearchPart(final SearchResultSelectListener<FfmStockInfoBean> listener, final int resultRows) {
	super(listener, resultRows);
    }

    /**
     * @see AbstractStockSearchPart#getTitle()
     */
    @Override
    public String getTitle() {
	return FfmSearchPart.DIALOG_TITLE;
    }

    /**
     * @see AbstractStockSearchPart#formatListEntry(de.frankmuenster.jameica.finanzen.StockInfoBean)
     */
    @Override
    protected String formatListEntry(final FfmStockInfoBean bean) {
	final StringBuilder result = new StringBuilder(50);
	StringBuilder help = new StringBuilder(bean.getName()).append(AbstractStockSearchPart.SPACE_20);
	result.append(help.substring(0, 20)).append(AbstractStockSearchPart.SPACE_3);
	help = new StringBuilder(bean.getType()).append(AbstractStockSearchPart.SPACE_10);
	result.append(help.substring(0, 10)).append(AbstractStockSearchPart.SPACE_3);
	final String wkn = bean.getWkn() == null ? StringUtils.EMPTY : bean.getWkn();
	help = new StringBuilder(wkn).append(AbstractStockSearchPart.SPACE_10);
	result.append(help.substring(0, 10)).append(AbstractStockSearchPart.SPACE_3);
	return result.toString();
    }

    /**
     * @see AbstractStockSearchPart#perfromSearch(java.lang.String)
     */
    @Override
    protected List<FfmStockInfoBean> perfromSearch(final String search) throws FinanceException {
	return FfmSearchWertpapier.getIntance().search(search);
    }
}
