/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.parts.wpsearch;

import java.util.List;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.finanzen.quandl.QuandlSearch;
import de.frankmuenster.jameica.depot.finanzen.quandl.QuandlStockInfoBean;
import de.frankmuenster.jameica.finanzen.FinanceException;
import de.willuhn.jameica.gui.Part;

/**
 * Part das es erlaubt nach einem Wertpapier in Quandl zu suchen.
 *
 * @author Frank Mnster
 */
public class QuandlSearchPart extends AbstractStockSearchPart<QuandlStockInfoBean> implements Part {

    /**
     * Der Dialogtitel
     */
    private static final String DIALOG_TITLE = "Quandl Suche"; //$NON-NLS-1$

    /**
     * Der Quandl API Key
     */
    private final transient String authToken;

    /**
     * Standardkonstruktor mit Angabe eines Listeners
     *
     * @param listener
     *            der zu benachrichtigende Listener
     */
    public QuandlSearchPart(final SearchResultSelectListener<QuandlStockInfoBean> listener) {
	this(listener, 7);
    }

    /**
     * Standardkonstruktor mit Angabe eines Listeners
     *
     * @param listener
     *            der zu benachrichtigende Listener
     * @param resultRows
     */
    public QuandlSearchPart(final SearchResultSelectListener<QuandlStockInfoBean> listener, final int resultRows) {
	super(listener, resultRows);
	this.authToken = Settings.getQuandlApiKey();
    }

    /**
     * @see AbstractStockSearchPart#getTitle()
     */
    @Override
    public String getTitle() {
	return QuandlSearchPart.DIALOG_TITLE;
    }

    /**
     * @see AbstractStockSearchPart#formatListEntry(de.frankmuenster.jameica.finanzen.StockInfoBean)
     */
    @Override
    protected String formatListEntry(final QuandlStockInfoBean bean) {
	final StringBuilder result = new StringBuilder(56);
	StringBuilder help = new StringBuilder(bean.getCode()).append(AbstractStockSearchPart.SPACE_10);
	result.append(help.substring(0, 10)).append(AbstractStockSearchPart.SPACE_3);
	help = new StringBuilder(bean.getExchange()).append(AbstractStockSearchPart.SPACE_6);
	result.append(help.substring(0, 6)).append(AbstractStockSearchPart.SPACE_3);
	help = new StringBuilder(bean.getName()).append(AbstractStockSearchPart.SPACE_30);
	result.append(help.substring(0, 30)).append(AbstractStockSearchPart.SPACE_3);
	return result.toString();
    }

    /**
     * @see AbstractStockSearchPart#perfromSearch(java.lang.String)
     */
    @Override
    protected List<QuandlStockInfoBean> perfromSearch(final String search) throws FinanceException {
	return QuandlSearch.getIntance(this.authToken).search(search);
    }
}
