/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.utils;

import java.util.Random;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;

import de.willuhn.jameica.gui.GUI;

/**
 * Utility Klasse fr Farben
 *
 * @author Frank Mnster
 */
public final class ColorUtils {

    /**
     * The random number generator
     */
    private static Random rand = new Random();

    /**
     * Creates a new Jameica {@link de.willuhn.jameica.gui.util.Color}
     * 
     * @param name
     *            the name of the color
     * @return the color
     */
    public static de.willuhn.jameica.gui.util.Color getJameicaColor(final String name) {

        final Color col = Display.getCurrent().getSystemColor(SWT.COLOR_BLUE);
        final de.willuhn.jameica.gui.util.Color color = new de.willuhn.jameica.gui.util.Color(name, col.getRGB());
        col.dispose();
        return color;

    }

    /**
     * Erzeugt eine zuf&auml;llige Farbe
     *
     * @return eine zuf&auml;llig erzeugt Farbe
     */
    public static Color getRandomColor() {

        int red = ColorUtils.rand.nextInt(512);
        int green = ColorUtils.rand.nextInt(512);
        int blue = ColorUtils.rand.nextInt(512);

        final Color mix = GUI.getDisplay().getSystemColor(SWT.COLOR_RED);

        // mix the color
        if (mix != null) {
            red = (red + mix.getRed()) / 3;
            green = (green + mix.getGreen()) / 3;
            blue = (blue + mix.getBlue()) / 3;
        }

        return new Color(GUI.getDisplay(), red, green, blue);
    }

    /**
     * Erstellt eine AWT Farbe basierend auf den RGB-Werten einer SWT Farbe.
     *
     * @param color
     *            Die SWT Farbe.
     * @return ein AWT Farbobjekt.
     */
    public static java.awt.Color toAwtColor(final Color color) {

        return new java.awt.Color(color.getRed(), color.getGreen(), color.getBlue());
    }

    /**
     * Standardkonstruktor
     */
    private ColorUtils() {

        // absichlich leer
    }

}
