/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.utils;

import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.time.format.DateTimeFormatter;
import java.util.Locale;

import de.willuhn.jameica.gui.formatter.CurrencyFormatter;

/**
 * Utility-Klasse f&uml;r verschiedene Formatter. Diese werden Thread-Safe
 * geliefert und m&uuml;ssen deshalb mit <i>.get()</i> benutzt werden.
 *
 * @author Frank Mnster
 */
@SuppressWarnings("PMD.LongVariable")
public final class FormatterUtils {

    /**
     * Dezimal-Format mit genau 2 Nachkommastellen
     */
    public static final ThreadLocal<DecimalFormat> DECIMAL_FORMAT = ThreadLocal
            .withInitial(() -> new DecimalFormat("###,###,##0.00")); //$NON-NLS-1$

    /**
     * Dezimal-Format mit min. 2 bis zu 3 Nachkommastellen
     */
    public static final ThreadLocal<DecimalFormat> DECIMAL_FORMAT_3 = ThreadLocal
            .withInitial(() -> new DecimalFormat("###,###,##0.00#")); //$NON-NLS-1$

    /**
     * Dezimal-Format mit min. 2 bis zu 6 Nachkommastellen
     */
    public static final ThreadLocal<DecimalFormat> DECIMAL_FORMAT_6 = ThreadLocal
            .withInitial(() -> new DecimalFormat("###,###,##0.00####")); //$NON-NLS-1$

    /**
     * W&auml;hrungs-Format mit genau 2 Nachkommastellen
     */
    public static final ThreadLocal<CurrencyFormatter> CURRENCY_FORMAT = ThreadLocal
            .withInitial(() -> new CurrencyFormatter(null, FormatterUtils.DECIMAL_FORMAT.get()));

    /**
     * W&auml;hrungs-Format mit bis zu 3 Nachkommastellen
     */
    public static final ThreadLocal<CurrencyFormatter> CURRENCY_FORMAT_3 = ThreadLocal
            .withInitial(() -> new CurrencyFormatter(null, FormatterUtils.DECIMAL_FORMAT_3.get()));

    /**
     * W&auml;hrungs-Format mit bis zu 6 Nachkommastellen
     */
    public static final ThreadLocal<CurrencyFormatter> CURRENCY_FORMAT_6 = ThreadLocal
            .withInitial(() -> new CurrencyFormatter(null, FormatterUtils.DECIMAL_FORMAT_6.get()));

    /**
     * Deutsches Datumsformat "dd.MM.yyyy"
     */
    public static final ThreadLocal<DateFormat> GERMAN_DATE_FORMAT = ThreadLocal
            .withInitial(() -> new SimpleDateFormat("dd.MM.yyyy", Locale.GERMAN)); //$NON-NLS-1$

    /**
     * Deutsches Datumsformat "dd.MM.yyyy" als DateTimeFormatter
     */
    public static final ThreadLocal<DateTimeFormatter> GERMAN_DATE_TIME_FORMATER = ThreadLocal
            .withInitial(() -> DateTimeFormatter.ofPattern("dd.MM.yyyy", Locale.GERMAN)); //$NON-NLS-1$

    /**
     * Einfaches Zeitformat "h:mm"
     */
    public static final ThreadLocal<DateFormat> SIMPLE_TIME_FORMAT = ThreadLocal
            .withInitial(() -> new SimpleDateFormat("h:mma", Locale.GERMAN)); //$NON-NLS-1$

    /**
     * Einfaches Zeitformat "h:mm"
     */
    public static final ThreadLocal<DateTimeFormatter> SIMPLE_DATE_TIME_FORMATTER = ThreadLocal
            .withInitial(() -> DateTimeFormatter.ofPattern("h:mma", Locale.GERMAN)); //$NON-NLS-1$

    /**
     * Standardkonstruktor
     */
    private FormatterUtils() {
        // Absichtlich leer
    }
}
