/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.utils;

import java.awt.Color;
import java.awt.Font;

import org.jfree.chart.annotations.XYTitleAnnotation;
import org.jfree.chart.block.BlockBorder;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.title.LegendTitle;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.RectangleInsets;

/**
 * Utility Klasse fr GUI Items
 *
 * @author Frank Mnster
 */
public final class ItemUtils {

    /**
     * Creates and adds a standard formatted <code>XYTitleAnnotation</code> on
     * the given plot.
     *
     * @param plot
     *            to add the title annotation.
     */
    public static void addLegendTitle(final XYPlot plot) {

        final LegendTitle legendTitle = new LegendTitle(plot);
        legendTitle.setItemFont(new Font("Dialog", Font.PLAIN, 9)); //$NON-NLS-1$
        legendTitle.setBackgroundPaint(new Color(200, 200, 255, 100));
        legendTitle.setFrame(new BlockBorder(Color.white));
        legendTitle.setPosition(RectangleEdge.BOTTOM);
        final RectangleInsets padding = new RectangleInsets(4, 2, 0, 10);
        legendTitle.setItemLabelPadding(padding);
        final XYTitleAnnotation titleAnnotation = new XYTitleAnnotation(1.0, 0.00, legendTitle,
                RectangleAnchor.BOTTOM_RIGHT);
        titleAnnotation.setMaxWidth(0.48);
        plot.addAnnotation(titleAnnotation);

    }

    /**
     * Standardkonstruktor
     */
    private ItemUtils() {
        // absichlich leer
    }

}
