/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.views;

import java.rmi.RemoteException;

import de.frankmuenster.jameica.depot.gui.controller.AnalysenKontroller;
import de.frankmuenster.jameica.depot.kursanalyse.Analysen;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.Headline;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.util.ApplicationException;

/**
 * The view to present the analysis charts
 *
 * @author Frank Mnster
 *
 */
public class AnalysenView extends AbstractView {

    /**
     * Der AnalysenView-Kontroller
     */
    private transient AnalysenKontroller control;

    /**
     * Standardkonstruktor
     */
    public AnalysenView() {
	super();
    }

    /**
     * @see de.willuhn.jameica.gui.AbstractView#bind()
     */
    @SuppressWarnings("unused")
    @Override
    public void bind() throws RemoteException {

	// draw the title
	GUI.getView().setTitle(I18N.tr("AnalysenView.viewTitle")); //$NON-NLS-1$

	// Instantiate controller
	this.control = new AnalysenKontroller(this);

	final Container container = new SimpleContainer(getParent());

	// layout with 2 columns
	final ColumnLayout columns = new ColumnLayout(container.getComposite(), 2);

	final Container left = new SimpleContainer(columns.getComposite());
	final Container right = new SimpleContainer(columns.getComposite(), true);

	// left side
	left.addPart(this.control.getZeitauswahl());
	left.addPart(this.control.getGrafikWerkzeuge());

	// Rechte Seite
	right.addHeadline(I18N.tr("AnalysenView.analysis")); //$NON-NLS-1$
	right.addPart(this.control.getAnalyseButtons());

	// zum Schluss die Grafik
	new Headline(getParent(), I18N.tr("AnalysenView.Chart")); //$NON-NLS-1$
	this.control.getGrafik().paint(getParent());

    }

    /**
     * @see de.willuhn.jameica.gui.AbstractView#unbind()
     */
    @Override
    public void unbind() throws ApplicationException {
	super.unbind();
	// Analysen leeren
	Analysen.getInstance().clear();
	// Kontroller lschen
	this.control = null; // NOPMD

    }

}
