/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.views;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.gui.action.EditWertpapierAction;
import de.frankmuenster.jameica.depot.gui.action.WertpapierAktualisiernAction;
import de.frankmuenster.jameica.depot.gui.controller.DepotController;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Klasse die die View zur Depotbersicht bereit stellt.
 *
 * @author Frank Mnster
 *
 */
public class DepotView extends AbstractView {

    /**
     * Der Depot-Kontroller
     */
    private transient DepotController control;

    /**
     * Standardkonstruktor
     */
    public DepotView() {
	super();
    }

    /**
     * @see de.willuhn.jameica.gui.AbstractView#bind()
     */
    @Override
    public void bind() throws RemoteException {

	this.control = new DepotController(this);

	final SimpleContainer container = new SimpleContainer(this.getParent(), true);

	container.addHeadline(I18N.tr("DepotView.hlStockOverview")); //$NON-NLS-1$
	// WertpapierListePart
	this.control.getAktienListe().paint(container.getComposite());
	container.addText(StringUtils.EMPTY, true);

	if (this.control.getAktienListe().isShowWaehrungsSpalte()) {
	    container.addHeadline(I18N.tr("DepotView.hlExchangeRates")); //$NON-NLS-1$
	    final SimpleContainer changeRates = new SimpleContainer(container.getComposite());
	    changeRates.addPart(this.control.getWechselKursInfos());
	}

	if (Settings.isShowHibiscusSummery()) {
	    container.addHeadline(I18N.tr("DepotView.hlTotal")); //$NON-NLS-1$
	    final SimpleContainer spHibiscusSummery = new SimpleContainer(container.getComposite(), false, 6);
	    spHibiscusSummery.addLabelPair(I18N.tr("DepotView.lblStockTotal"), //$NON-NLS-1$
		    this.control.getJaDepotTotal());
	    spHibiscusSummery.addText(" ", false); //$NON-NLS-1$
	    spHibiscusSummery.addText(" ", false); //$NON-NLS-1$
	    spHibiscusSummery.addLabelPair(I18N.tr("DepotView.lblHibiscusTotal"), //$NON-NLS-1$
		    this.control.getHibiscusTotal());
	    spHibiscusSummery.addText(" ", false); //$NON-NLS-1$
	    spHibiscusSummery.addText(" ", false); //$NON-NLS-1$
	    spHibiscusSummery.addLabelPair(I18N.tr("DepotView.lblTotal"), //$NON-NLS-1$
		    this.control.getHibiscusJaDepotTotal());
	}

	final ButtonArea actionButtons = new ButtonArea();
	final WertpapierAktualisiernAction wpUpdateAction = new WertpapierAktualisiernAction();
	wpUpdateAction.addListener(this.control);
	actionButtons.addButton(I18N.tr("DepotView.btnRefresh"), //$NON-NLS-1$
		wpUpdateAction, null, false, "view-refresh.png"); //$NON-NLS-1$
	actionButtons.addButton(I18N.tr("DepotView.btnNewStock"), //$NON-NLS-1$
		new EditWertpapierAction(), null, false, "document-new.png"); //$NON-NLS-1$
	container.addButtonArea(actionButtons);

    }

    /**
     * @see de.willuhn.jameica.gui.AbstractView#unbind()
     */
    @Override
    public void unbind() throws ApplicationException {

	Logger.debug("DepotView: unbind"); //$NON-NLS-1$

	super.unbind();
	this.control = null; // NOPMD
    }

}
