/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.views;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.layout.GridData;

import de.frankmuenster.jameica.depot.gui.buttons.ViewDepotButton;
import de.frankmuenster.jameica.depot.gui.controller.OrderController;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.util.ApplicationException;

/**
 * View to show and edit a stock order
 *
 * @author Frank Mnster
 *
 */
public class OrderView extends AbstractView {

    /**
     * Der OrderController
     */
    private transient OrderController control;

    /**
     * Standardkonstruktor
     */
    public OrderView() {
        super();
    }

    /**
     *
     * @see de.willuhn.jameica.gui.AbstractView#bind()
     */
    @Override
    public void bind() throws RemoteException {

        GUI.getView().setTitle(I18N.tr("OrderView.newOrder")); //$NON-NLS-1$

        this.control = new OrderController(this);

        // layout with 2 columns
        final ColumnLayout columns = new ColumnLayout(this.getParent(), 2);

        final Container links = new SimpleContainer(columns.getComposite());

        // right side
        final Container right = new SimpleContainer(columns.getComposite(), true);
        final GridData gridData = (GridData) right.getComposite().getLayoutData();
        gridData.minimumWidth = 250;
        right.addHeadline(I18N.tr("OrderView.description")); //$NON-NLS-1$
        right.addInput(this.control.getComment());
        right.addText(StringUtils.EMPTY, true);
        right.addText(StringUtils.EMPTY, true);
        right.addText(StringUtils.EMPTY, true);
        right.addText(StringUtils.EMPTY, true);
        right.addText(StringUtils.EMPTY, true);
        right.addText(StringUtils.EMPTY, true);
        right.addText(StringUtils.EMPTY, true);
        right.addText(StringUtils.EMPTY, true);

        // left lower
        links.addHeadline(I18N.tr("OrderView.symboleName")); //$NON-NLS-1$
        links.addLabelPair(I18N.tr("OrderView.symbole"), this.control.getSymbol()); //$NON-NLS-1$
        links.addLabelPair(I18N.tr("OrderView.name"), this.control.getName()); //$NON-NLS-1$
        links.addText(StringUtils.EMPTY, true);

        links.addHeadline(I18N.tr("OrderView.DateAndNumber")); //$NON-NLS-1$
        final ColumnLayout dateLay = new ColumnLayout(links.getComposite(), 1);
        final Container date = new SimpleContainer(dateLay.getComposite(), false, 4);
        date.addLabelPair(I18N.tr("OrderView.Date"), this.control.getOrderDate()); //$NON-NLS-1$
        date.addText("                                                        ", //$NON-NLS-1$
                false);
        date.addLabelPair(I18N.tr("OrderView.number"), this.control.getVolume()); //$NON-NLS-1$
        links.addText(StringUtils.EMPTY, true);

        links.addHeadline(I18N.tr("OrderView.prices")); //$NON-NLS-1$
        final ColumnLayout priceLay = new ColumnLayout(links.getComposite(), 1);
        final Container price = new SimpleContainer(priceLay.getComposite(), false, 6);
        price.addLabelPair(I18N.tr("OrderView.price"), this.control.getPrice()); //$NON-NLS-1$
        price.addInput(this.control.getStockCurrency());
        price.addPart(this.control.getPriceInMainCurrency());
        links.addText(StringUtils.EMPTY, true);

        links.addHeadline(I18N.tr("OrderView.additionalCosts")); //$NON-NLS-1$
        final ColumnLayout addPriceLay = new ColumnLayout(links.getComposite(), 1);
        final Container addPrice = new SimpleContainer(addPriceLay.getComposite(), false, 6);
        addPrice.addLabelPair(I18N.tr("OrderView.costs"), //$NON-NLS-1$
                this.control.getAdditionalCosts());
        addPrice.addInput(this.control.getAdditionalCostsCurrency());
        addPrice.addPart(this.control.getAdditionalCostsInMainCurrency());
        // Jetzt schon anzeigen
        this.control.updateView();

        // Speicher Button einfgen
        final ButtonArea buttons = new ButtonArea();
        buttons.addButton(new ViewDepotButton());
        final Button store = this.control.getStore();
        final Button cancel = this.control.getCancel();

        buttons.addButton(store);
        buttons.addButton(cancel);

        buttons.paint(this.getParent());

    }

    /**
     * @see de.willuhn.jameica.gui.AbstractView#unbind()
     */
    @Override
    public void unbind() throws ApplicationException {
        super.unbind();
        this.control = null; // NOPMD
    }
}
