/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.views;

import java.rmi.RemoteException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.TabFolder;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.gui.controller.SettingsController;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.jameica.gui.util.TabGroup;
import de.willuhn.util.ApplicationException;

/**
 * View fr die Einstellungen
 *
 * @author Frank Mnster
 *
 */
public class SettingsView extends AbstractView {

    /**
     * Der Settings-Kontrolleur
     */
    private transient SettingsController control;

    /**
     * Standardkonstruktor
     */
    public SettingsView() {
	super();
    }

    /**
     * @see de.willuhn.jameica.gui.AbstractView#bind()
     */
    @Override
    public void bind() throws RemoteException {

	GUI.getView().setTitle(I18N.tr("SettingsView.viewTitle")); //$NON-NLS-1$

	// Kontrolleur erstellen
	this.control = new SettingsController(this);

	final GridLayout gridLayout = new GridLayout(2, false);
	gridLayout.verticalSpacing = 10;

	final TabGroup grundEinstellung = new TabGroup(getTabFolder(), I18N.tr("SettingsView.tabBasicSettings")); //$NON-NLS-1$

	final ColumnLayout columnLayout = new ColumnLayout(grundEinstellung.getComposite(), 2);
	final SimpleContainer leftContainer = new SimpleContainer(columnLayout.getComposite());
	final SimpleContainer rightContainer = new SimpleContainer(columnLayout.getComposite());

	leftContainer.addHeadline(I18N.tr("SettingsView.hlCommonSettings")); //$NON-NLS-1$
	if (Settings.isHibiscusInstalled()) {
	    leftContainer.addLabelPair(I18N.tr("SettingsView.lblShowHibiscus"), //$NON-NLS-1$
		    this.control.getShowHibiscusSummery());
	}
	leftContainer.addLabelPair(I18N.tr("SettingsView.quandlApiKey"), this.control.getQuandlApiKey()); //$NON-NLS-1$
	rightContainer.addText("       ", false); //$NON-NLS-1$

	final ButtonArea buttonArea = new ButtonArea();
	buttonArea.addButton(I18N.tr("_saveBtn"), new Action() { //$NON-NLS-1$
	    /**
	     * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
	     */
	    @Override
	    public void handleAction(final Object context) throws ApplicationException {
		SettingsView.this.control.speichern();
	    }
	}, null, true, "document-save.png"); //$NON-NLS-1$
	buttonArea.paint(getParent());

    }

    /**
     * @see de.willuhn.jameica.gui.AbstractView#unbind()
     */
    @Override
    public void unbind() throws ApplicationException {
	super.unbind();
	this.control = null; // NOPMD
    }

    /**
     * Liefert den Tab-Folder, in dem die einzelnen Module der Einstellungen
     * untergebracht sind.
     * 
     * @return der Tab-Folder.
     */
    private TabFolder getTabFolder() {

	final TabFolder folder = new TabFolder(getParent(), SWT.NONE);
	folder.setLayoutData(new GridData(GridData.FILL_BOTH));
	return folder;
    }

}
