/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.importer;

import java.io.BufferedReader;
import java.math.BigDecimal;
import java.util.Date;

import com.opencsv.bean.MappingStrategy;

import de.frankmuenster.jameica.depot.importer.exceptions.OrderValueMissing;
import de.frankmuenster.jameica.depot.internal.objects.Order;

/**
 * The interface of an Order Importer
 *
 * @author Frank Mnster
 */
public interface CsvOrderBean {

    /**
     * Gets the comment.
     *
     * @return the comment
     */
    String getComment();

    /**
     * Gets the isin.
     *
     * @return the isin
     */
    String getIsin();

    /**
     * Get the mapping strategy. can be <i>Null</i> if not needed
     *
     * @return the MappingStrategy that needs to be used with this CsvOrderBean
     */
    MappingStrategy<CsvOrderBean> getMappingStrategy();

    /**
     * Gets the name.
     *
     * @return the name
     */
    String getName();

    /**
     * Gets the nominal.
     *
     * @return the nominal
     */
    BigDecimal getNominal();

    /**
     * Gets a new order object
     *
     * @return the new order object
     * @throws OrderValueMissing
     */
    Order getOrder() throws OrderValueMissing;

    /**
     * Gets the order currency.
     *
     * @return the orderCurrency
     */
    String getOrderCurrency();

    /**
     * Gets the order date.
     *
     * @return the order date
     */
    Date getOrderDate();

    /**
     * Get the number of the oder
     *
     * @return teh number of the order
     */
    String getOrderNumber();

    /**
     * Gets the order price.
     *
     * @return the order price
     */
    BigDecimal getOrderPrice();

    /**
     * Gets the separator to use
     *
     * @return the separator to use
     */
    char getSeparator();

    /**
     * Gets the number of lines to skip
     *
     * @return the number of lines to skip
     */
    int getSkipLines();

    /**
     * Checks if a given file can be read by this CsvOrderBean
     *
     * @param csvExport
     *            the file to be checked
     * @return <i>true</i> if the file can be read by this CsvOrderBean,
     *         <i>false</i> else
     */
    boolean isReading(final BufferedReader csvExport);

    /**
     * Set the order number
     *
     * @param orderNumber
     */
    void setOrderNumber(final String orderNumber);

}
