/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.importer;

import java.math.BigDecimal;
import java.util.Date;

import com.opencsv.bean.CsvBindByName;
import com.opencsv.bean.CsvDate;
import com.opencsv.bean.MappingStrategy;

/**
 * The Class ImportOrder.
 */
@RegisterCsvOrderBean(name = "DiBa")
public class CsvOrderBeanDibaImpl extends AbstractCsvOrderBean {

    /** The ISIN of the stock. */
    @CsvBindByName(column = "ISIN", required = true)
    private String isin;

    @CsvBindByName(column = "Wertpapiername", required = true)
    /** Name of the stock. */
    private String name;

    @CsvBindByName(column = "Orderart", required = true)
    /** Type of the stock. */
    private String orderType;

    /** Order Date. */
    @CsvBindByName(column = "Ausfhrungsdatum", required = true, locale = AbstractCsvOrderBean.DE_DE)
    @CsvDate("dd.MM.yyyy")
    private Date orderDate;

    /** Amount/nominal of the order. */
    @CsvBindByName(column = "Stck/Nominale", required = true, locale = AbstractCsvOrderBean.DE_DE)
    private BigDecimal nominal;

    /** The order price. */
    @CsvBindByName(column = "Ausfhrungskurs", required = true, locale = AbstractCsvOrderBean.DE_DE)
    private BigDecimal orderPrice;

    /** Currency of the order. */
    @CsvBindByName(column = "Whrung", required = true, locale = AbstractCsvOrderBean.DE_DE)
    private String orderCurrency;

    /** Order */
    @CsvBindByName(column = "Ordernummer")
    private String orderNumber;

    /** Order */
    @CsvBindByName(column = "Limittyp")
    private String limitTyp;

    /** Status of the order */
    @CsvBindByName(column = "Status")
    private String status;

    /**
     * Standard constructor
     */
    public CsvOrderBeanDibaImpl() {

        super();
    }

    /*
     * (non-Javadoc)
     * @see de.frankmuenster.jameica.depot.importer.ImportOrder#getComment()
     */
    @Override
    public String getComment() {

        final StringBuilder builder = new StringBuilder();
        builder.append(this.orderType).append(" - ").append(this.limitTyp); //$NON-NLS-1$
        return builder.toString();
    }

    /*
     * (non-Javadoc)
     * @see de.frankmuenster.jameica.depot.importer.ImportOrder#getIsin()
     */
    @Override
    public String getIsin() {

        return this.isin;
    }

    /**
     * @return the limittyp
     */
    public String getLimittyp() {

        return this.limitTyp;
    }

    /**
     * @see CsvOrderBean#getMappingStrategy()
     */
    @Override
    public MappingStrategy<CsvOrderBean> getMappingStrategy() {

        // No mapping strategy needed
        return null;
    }

    /*
     * (non-Javadoc)
     * @see de.frankmuenster.jameica.depot.importer.ImportOrder#getName()
     */
    @Override
    public String getName() {

        return this.name;
    }

    /*
     * (non-Javadoc)
     * @see de.frankmuenster.jameica.depot.importer.ImportOrder#getNominal()
     */
    @Override
    public BigDecimal getNominal() {

        return this.nominal;
    }

    /*
     * (non-Javadoc)
     * @see de.frankmuenster.jameica.depot.importer.ImportOrder#getOrderCurrency()
     */
    @Override
    public String getOrderCurrency() {

        return this.orderCurrency;
    }

    /*
     * (non-Javadoc)
     * @see de.frankmuenster.jameica.depot.importer.ImportOrder#getOrderDate()
     */
    @Override
    public Date getOrderDate() {

        return this.orderDate;
    }

    /**
     * @return the orderNumber
     */
    @Override
    public String getOrderNumber() {

        return this.orderNumber;
    }

    @Override
    public BigDecimal getOrderPrice() {

        return this.orderPrice;
    }

    /**
     * @return the kindOfOrder
     */
    public String getOrderType() {

        return this.orderType;
    }

    /**
     * @see de.frankmuenster.jameica.depot.importer.CsvOrderBean#getSeparator()
     */
    @Override
    public char getSeparator() {

        // ING-Diba uses a semicolon as separator
        return ';';
    }

    /**
     * @see de.frankmuenster.jameica.depot.importer.CsvOrderBean#getSkipLines()
     */
    @Override
    public int getSkipLines() {

        // ING-Diba needs to skip 4 lines
        return 4;
    }

    /**
     * @return the status
     */
    public String getStatus() {

        return this.status;
    }

    /**
     * @param isin
     */
    public void setIsin(final String isin) {

        this.isin = isin;
    }

    /**
     * @param limittyp
     *            the limittyp to set
     */
    public void setLimittyp(final String limittyp) {

        this.limitTyp = limittyp;
    }

    /**
     * @param name
     */
    public void setName(final String name) {

        this.name = name;
    }

    /**
     * @param nominal
     */
    public void setNominal(final BigDecimal nominal) {

        this.nominal = nominal;
    }

    /**
     * @param orderCurrency
     */
    public void setOrderCurrency(final String orderCurrency) {

        this.orderCurrency = orderCurrency;
    }

    /**
     * @param orderDate
     */
    public void setOrderDate(final Date orderDate) {

        this.orderDate = orderDate;
    }

    /**
     * @param orderNumber
     *            the orderNumber to set
     */
    @Override
    public void setOrderNumber(final String orderNumber) {

        this.orderNumber = orderNumber;
    }

    /**
     * @param orderPrice
     */
    public void setOrderPrice(final BigDecimal orderPrice) {

        this.orderPrice = orderPrice;
    }

    /**
     * @param orderType
     */
    public void setOrderType(final String orderType) {

        this.orderType = orderType;
    }

    /**
     * @param status
     *            the status to set
     */
    public void setStatus(final String status) {

        this.status = status;
    }

    @Override
    protected BigDecimal getNominalForOrderBean() {

        BigDecimal nominalOrder;

        if ("verkauf".equalsIgnoreCase(this.getOrderType())) { //$NON-NLS-1$
            nominalOrder = this.getNominal().negate();
        } else {
            nominalOrder = this.getNominal();
        }
        return nominalOrder;
    }

    /**
     * @see AbstractCsvOrderBean#isSupported(String)
     */
    @Override
    protected boolean isSupported(final String line) {

        return line.startsWith("Orderart;Ausf"); //$NON-NLS-1$
    }

}
