/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.importer;

import java.math.BigDecimal;
import java.util.Date;

import com.opencsv.bean.CsvBindByPosition;
import com.opencsv.bean.CsvDate;
import com.opencsv.bean.MappingStrategy;

/**
 * The Class ImportOrder.
 */
@RegisterCsvOrderBean(name = "Flatex")
public class CsvOrderBeanFlatexImpl extends AbstractCsvOrderBean {
    /**
     * Text indication a write-off value (negative)
     */
    // private static final List<String> WRITE_OFF_VALUES = Arrays.asList( //
    // Options:
    // "Ausfhrung ORDER Kauf", //$NON-NLS-1$
    // "Ausbuchung fiktive Veruerung", //$NON-NLS-1$
    // "Storno Einbuchung fiktive Veruerung"); //$NON-NLS-1$

    /** The ISIN of the stock. */
    @CsvBindByPosition(position = 3, required = true)
    private String isin;

    @CsvBindByPosition(position = 4, required = true)
    /** Name of the stock. */
    private String name;

    /** Order Date. */
    @CsvBindByPosition(position = 2, required = true, locale = AbstractCsvOrderBean.DE_DE)
    @CsvDate("ddMMyyyy")
    private Date orderValuta;

    /** Amount/nominal of the order. */
    // @CsvNumber("")
    @CsvBindByPosition(position = 5, required = true, locale = AbstractCsvOrderBean.DE_DE)
    private BigDecimal nominal;

    /** The order price. */
    @CsvBindByPosition(position = 9, required = true, locale = AbstractCsvOrderBean.DE_DE)
    private BigDecimal orderPrice;

    /** Currency of the order. */
    @CsvBindByPosition(position = 10, required = true, locale = AbstractCsvOrderBean.DE_DE)
    private String orderCurrency;

    /** Order number */
    @CsvBindByPosition(position = 8)
    private String orderNumber;

    /** order book date */
    @CsvBindByPosition(position = 1, locale = AbstractCsvOrderBean.DE_DE)
    @CsvDate("ddMMyyyy")
    private Date orderDate;

    /** Information about the order */
    @CsvBindByPosition(position = 7)
    private String info;

    /**
     * Standard constructor
     */
    public CsvOrderBeanFlatexImpl() {

        super();
    }

    /*
     * (non-Javadoc)
     *
     * @see de.frankmuenster.jameica.depot.importer.ImportOrder#getComment()
     */
    @Override
    public String getComment() {

        return this.info;
    }

    /**
     * @return the oder booking info
     */
    public String getInfo() {

        return this.info;
    }

    /*
     * (non-Javadoc)
     *
     * @see de.frankmuenster.jameica.depot.importer.ImportOrder#getIsin()
     */
    @Override
    public String getIsin() {

        return this.isin;
    }

    @Override
    public MappingStrategy<CsvOrderBean> getMappingStrategy() {

        // Not (yet) needed here
        return null;
    }

    /*
     * (non-Javadoc)
     *
     * @see de.frankmuenster.jameica.depot.importer.ImportOrder#getName()
     */
    @Override
    public String getName() {

        return this.name;
    }

    /*
     * (non-Javadoc)
     *
     * @see de.frankmuenster.jameica.depot.importer.ImportOrder#getNominal()
     */
    @Override
    public BigDecimal getNominal() {

        return this.nominal;
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * de.frankmuenster.jameica.depot.importer.ImportOrder#getOrderCurrency()
     */
    @Override
    public String getOrderCurrency() {

        return this.orderCurrency;
    }

    /*
     * (non-Javadoc)
     *
     * @see de.frankmuenster.jameica.depot.importer.ImportOrder#getOrderDate()
     */
    @Override
    public Date getOrderDate() {

        return this.orderDate;
    }

    /**
     * @return the orderNumber
     */
    @Override
    public String getOrderNumber() {

        return this.orderNumber;
    }

    @Override
    public BigDecimal getOrderPrice() {

        return this.orderPrice;
    }

    /**
     * @return the orderValuta
     */
    public Date getOrderValuta() {

        return this.orderValuta;
    }

    /**
     * @see de.frankmuenster.jameica.depot.importer.CsvOrderBean#getSeparator()
     */
    @Override
    public char getSeparator() {

        // Flatex uses a semicolon as separator
        return ';';
    }

    /**
     * @see de.frankmuenster.jameica.depot.importer.CsvOrderBean#getSkipLines()
     */
    @Override
    public int getSkipLines() {

        // As we use CSV we should skip the first line
        return 0;
    }

    /**
     * @param info
     */
    public void setInfo(final String info) {

        this.info = info;
    }

    /**
     * @param isin
     */
    public void setIsin(final String isin) {

        this.isin = isin;
    }

    /**
     * @param name
     */
    public void setName(final String name) {

        this.name = name;
    }

    /**
     * @param nominal
     */
    public void setNominal(final BigDecimal nominal) {

        this.nominal = nominal;
    }

    /**
     * @param orderCurrency
     */
    public void setOrderCurrency(final String orderCurrency) {

        this.orderCurrency = orderCurrency;
    }

    /**
     * @param orderDate
     */
    public void setOrderDate(final Date orderDate) {

        this.orderDate = orderDate;
    }

    /**
     * @param orderNumber
     *            the orderNumber to set
     */
    @Override
    public void setOrderNumber(final String orderNumber) {

        this.orderNumber = orderNumber;
    }

    /**
     * @param orderPrice
     */
    public void setOrderPrice(final BigDecimal orderPrice) {

        this.orderPrice = orderPrice;
    }

    /**
     * @param orderValuta
     */
    public void setOrderValuta(final Date orderValuta) {

        this.orderValuta = orderValuta;
    }

    /**
     * @see AbstractCsvOrderBean#getOrderFromBean()
     */
    @Override
    protected BigDecimal getNominalForOrderBean() {

        // we simply can give back the nominal as it show the correct value
        return this.getNominal();

    }

    /**
     * @see AbstractCsvOrderBean#isSupported(String)
     */
    @Override
    protected boolean isSupported(final String line) {
        return "Nummer;Buchtag;Valuta;ISIN;Bezeichnung;Nominal;;Buchungsinformationen;TA-Nr.;Kurs;;Depot".equals(line); //$NON-NLS-1$
    }

}
