/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.importer;

import java.io.BufferedReader;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 */
public final class CsvOrderBeanRegistry {

    /**
     * Liste der registrierten QR-Code-Handlers
     */
    private static Map<String, CsvOrderBean> readerBeans = new HashMap<>(2);

    /**
     * Checks if a given file can be read by this any known CsvOrderBean
     *
     * @param csvExport
     *            the file to be checked
     * @return the CsvOrderBean if one could be found, <b>NULL</b> else
     */
    public static CsvOrderBean getCsvOrderBean(final BufferedReader csvExport) {

        CsvOrderBean orderBean = null;
        final Collection<CsvOrderBean> csvOrderBeans = CsvOrderBeanRegistry.readerBeans.values();
        for (final CsvOrderBean csvOrderBean : csvOrderBeans) {
            if (csvOrderBean.isReading(csvExport)) {
                orderBean = csvOrderBean;
            }
        }
        return orderBean;
    }

    /**
     * Register a new CsvOrderBean instance
     *
     * @param csvOrderBean
     * @param beanName
     *            a name identifing this bean
     */
    public static void register(final CsvOrderBean csvOrderBean, final String beanName) {

        if (!CsvOrderBeanRegistry.readerBeans.containsKey(beanName)) {
            if (Logger.isLogging(Level.INFO)) {
                Logger.info("CsvOrderBean-Name: " + beanName); //$NON-NLS-1$
            }
            CsvOrderBeanRegistry.readerBeans.put(beanName, csvOrderBean);
        }
    }

    /**
     * Private constructor for utility class
     */
    private CsvOrderBeanRegistry() {

        // Intentionally left blank
    }

}
