/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.importer.exceptions;

/**
 * Exception for any kind of order processing failure
 *
 * @author Frank Mnster
 *
 */
public class OrderProcessException extends Exception {

    /**
     * Our UID
     */
    private static final long serialVersionUID = 5723962994985463602L;

    /**
     * The exception type
     */
    private final OrderProcessExceptionType type;

    /**
     * Constructs a new exception with the specified exception type. The
     * detailed message is set to
     * {@link OrderProcessExceptionType#getDetailMsg()}. The cause is not
     * Initialised, and may subsequently be initialised by a call to
     * {@link #initCause}.
     *
     * @param type
     *            the exception type. The exception type is saved for later
     *            retrieval by the {@link #getType()} method.
     */
    public OrderProcessException(final OrderProcessExceptionType type) {
        this(type.getDetailMsg(), type);
    }

    /**
     * Constructs a new exception with the specified exception type and cause.
     * <p>
     * Note that the detail message is set to
     * {@link OrderProcessExceptionType#getDetailMsg()}.
     * </p>
     *
     * @param type
     *            the exception type. The exception type is saved for later
     *            retrieval by the {@link #getType()} method.
     * @param cause
     *            the cause (which is saved for later retrieval by the
     *            {@link #getCause()} method). (A <tt>null</tt> value is
     *            permitted, and indicates that the cause is nonexistent or
     *            unknown.)
     */
    public OrderProcessException(final OrderProcessExceptionType type, final Throwable cause) {
        this(type.getDetailMsg(), type, cause);
    }

    /**
     * Constructs a new exception with the specified detail message and
     * exception type. The cause is not initialised, and may subsequently be
     * Initialised by a call to {@link #initCause}.
     *
     * @param message
     *            the detail message (which is saved for later retrieval by the
     *            {@link #getMessage()} method).
     * @param type
     *            the exception type. The exception type is saved for later
     *            retrieval by the {@link #getType()} method.
     *
     */
    public OrderProcessException(final String message, final OrderProcessExceptionType type) {
        super(message);
        this.type = type;
    }

    /**
     * Constructs a new exception with the specified detail message, the
     * exception type and cause.
     * <p>
     * Note that the detail message associated with {@code cause} is <i>not</i>
     * automatically incorporated in this exception's detail message.
     * </p>
     *
     * @param message
     *            the detail message (which is saved for later retrieval by the
     *            {@link #getMessage()} method).
     * @param type
     *            the exception type. The exception type is saved for later
     *            retrieval by the {@link #getType()} method.
     * @param cause
     *            the cause (which is saved for later retrieval by the
     *            {@link #getCause()} method). (A <tt>null</tt> value is
     *            permitted, and indicates that the cause is nonexistent or
     *            unknown.)
     */
    public OrderProcessException(final String message, final OrderProcessExceptionType type, final Throwable cause) {
        super(message, cause);
        this.type = type;
    }

    /**
     * Returns the type of the exception
     *
     * @return the type of the exception
     */
    public OrderProcessExceptionType getType() {

        return this.type;
    }

}
