/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.importer.exceptions;

import de.frankmuenster.jameica.depot.utils.I18N;

/**
 * Type enum for order process exceptions. Should be used to detailed any
 * exceptions thrown during order processing.
 *
 * @author Frank Mnster
 *
 */
public enum OrderProcessExceptionType {

    /**
     * Stock not found
     */
    STOCK_NOT_FOUND(I18N.tr("OrderProcessExceptionType.stockNotFound")), //$NON-NLS-1$

    /**
     *
     */
    ISIN_IS_NULL_OR_EMPTY(I18N.tr("OrderProcessExceptionType.isinIsNullOrEmpty")), //$NON-NLS-1$

    /**
     * Equal test failed
     */
    EQUAL_TEST_FAILED(I18N.tr("OrderProcessExceptionType.equalTestFailed")), //$NON-NLS-1$

    /**
     * Database connection problem
     */
    DB_CONNETION_PROBLEM(I18N.tr("OrderProcessExceptionType.dbConnectionProblem")), //$NON-NLS-1$

    /**
     * No importer for the given file/reader found
     */
    NO_IMPORTER_FOUND(I18N.tr("OrderProcessExceptionType.noImporterFound")); //$NON-NLS-1$

    /**
     * The detailed message for this exception type
     */
    private String detailMsg;

    /**
     * Internal constructor to create the enum list
     */
    OrderProcessExceptionType(final String detailMsg) {

        this.detailMsg = detailMsg;

    }

    /**
     * Returns the detailed message of an {@link OrderProcessExceptionType}
     *
     * @return the detailed message
     */
    public String getDetailMsg() {
        return this.detailMsg;
    }

}
