/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.importer.wizard;

import java.rmi.RemoteException;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.parts.ExpandPart;
import de.willuhn.jameica.gui.util.Color;
import de.willuhn.jameica.gui.util.SimpleContainer;

/**
 * @author Frank Mnster
 */
public class ImportWizardPage3Part implements Part {

    /**
     * The view controller
     */
    private final OrderImportWizardControl controller;

    /**
     * The parent composite of this page
     */
    private Composite parent;

    /**
     * Constructs the second page for the wizard with the controller, so tables
     * can be updated.
     *
     * @param controller
     */
    public ImportWizardPage3Part(final OrderImportWizardControl controller) {

        this.controller = controller;

    }

    /**
     * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {

        // Save parent
        this.parent = parent;

        final SimpleContainer headLine = new SimpleContainer(parent, true, 2);
        final SimpleContainer header = new SimpleContainer(parent, true, 4);
        final SimpleContainer errorTable = new SimpleContainer(parent, true, 2);

        headLine.addHeadline(I18N.tr("ImportWizardPage3Part.summary")); //$NON-NLS-1$
        header.addLabelPair(I18N.tr("ImportWizardPage3Part.readOrders"), this.controller.getPage3TotalOrders()); //$NON-NLS-1$
        header.addLabelPair(I18N.tr("ImportWizardPage3Part.existingOrders"), this.controller.getPage3ExistingOrders()); //$NON-NLS-1$
        header.addLabelPair(I18N.tr("ImportWizardPage3Part.savedOrders"), this.controller.getPage3SavedOrders()); //$NON-NLS-1$
        header.addLabelPair(I18N.tr("ImportWizardPage3Part.readErrors"), this.controller.getPage3LoadErrorOrders()); //$NON-NLS-1$
        header.addText(StringUtils.EMPTY, false);
        header.addLabelPair(I18N.tr("ImportWizardPage3Part.wrongOrders"), this.controller.getPage3FailureOrders()); //$NON-NLS-1$
        header.addText(StringUtils.EMPTY, false);
        errorTable.addHeadline(I18N.tr("ImportWizardPage3Part.saveErrors")); //$NON-NLS-1$
        final ExpandPart expand = new ExpandPart();
        expand.add(this.controller.getPage3FailedLoadedTable());
        errorTable.addPart(expand);

    }

    /**
     * Repaint the page with updated information about the boxes
     *
     * @throws RemoteException
     */
    public void repaint() throws RemoteException {

        if (this.parent != null) {
            // dispose all children
            for (final Control child : this.parent.getChildren()) {
                child.dispose();
            }

            // and repaint
            paint(this.parent);
            this.parent.layout(true);
        }

    }

    /**
     * Paints a major save error text
     */
    public void repaintSaveError() {

        if (this.parent != null) {

            // dispose all children
            for (final Control child : this.parent.getChildren()) {
                child.dispose();
            }

            final SimpleContainer conatiner = new SimpleContainer(this.parent, true, 2);
            conatiner.addHeadline(I18N.tr("ImportWizardPage3Part.majorSaveError")); //$NON-NLS-1$
            final String saveError = I18N.tr("ImportWizardPage3Part.majorSaveErrorTxt"); //$NON-NLS-1$
            conatiner.addText(StringUtils.EMPTY, true);
            conatiner.addText(saveError, true, Color.ERROR);
            conatiner.addText(StringUtils.EMPTY, true);

            this.parent.layout(true);
        }

    }

}
