/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.importer.wizard.parts;

import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.opencsv.exceptions.CsvException;

import de.frankmuenster.jameica.depot.internal.objects.Order;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.boxes.Box;
import de.willuhn.jameica.gui.formatter.Formatter;
import de.willuhn.jameica.gui.parts.TablePart;

/**
 * @author Frank Mnster
 */
public class OrderExceptionTablePart extends TablePart implements Box {

    /**
     * Constructs a new table to list all orders that have been loaded
     * successfully.
     *
     * @param list
     *            with successfully loaded {@link Order}
     * @param action
     *            that is executed when double-click an entry
     */
    public OrderExceptionTablePart(final List<CsvException> list, final Action action) {

        super(list, action);

        // Set basic behaviour
        this.setRememberColWidths(true);
        this.setRememberOrder(false);
        this.setRememberState(false);
        // Add columns
        this.addColumn(I18N.tr("OrderExceptionTablePart.lineNumber"), "lineNumber"); //$NON-NLS-1$//$NON-NLS-2$
        this.addColumn(I18N.tr("OrderExceptionTablePart.error"), "message"); //$NON-NLS-1$//$NON-NLS-2$
        this.addColumn(I18N.tr("OrderExceptionTablePart.line"), "line", new LineFormatter()); //$NON-NLS-1$//$NON-NLS-2$

    }

    /**
     * Always 0 as we do not sort
     */
    @Override
    public int compareTo(final Object object) {
        return 0;
    }

    /**
     * @see #getDefaultEnabled()
     */
    @Override
    public boolean getDefaultEnabled() {
        return false;
    }

    /**
     * Always 0
     */
    @Override
    public int getDefaultIndex() {
        return 0;
    }

    /**
     * @see #getHeight()
     */
    @Override
    public int getHeight() {
        return 170;
    }

    /**
     * Not used! getIndex will always return 0.
     */
    @Override
    public int getIndex() {
        return 0;
    }

    /**
     * @see Box#getName()
     */
    @Override
    public String getName() {
        return I18N.tr("OrderExceptionTablePart.tableName"); //$NON-NLS-1$
    }

    /**
     * Return if the box should be shown at all.
     *
     * @return <i>true</i> in case the TablePart is enabled, <i>false</i> else.
     *
     */
    @Override
    public boolean isActive() {
        return isEnabled();
    }

    /**
     * Not used! getIndex will always return 0.
     */
    @Override
    public void setIndex(final int index) {
        // Nothing to do here
    }

    /**
     * Nice formatter to present the line content
     *
     */
    private class LineFormatter implements Formatter {

        /**
         * The maximum line length for this formatter
         */
        private static final int MAX_LINE_LENGTH = 40;

        /**
         * Returns a new LineFormatter object
         */
        public LineFormatter() {
            // intentionally left blank
        }

        @Override
        public String format(final Object obj) {

            final StringBuilder lines = new StringBuilder(LineFormatter.MAX_LINE_LENGTH);

            if (obj instanceof String[]) {

                final String[] line = (String[]) obj;
                for (final String string : line) {
                    lines.append(string).append(';');
                    if (lines.length() > LineFormatter.MAX_LINE_LENGTH) {
                        break;
                    }
                }
            }
            return StringUtils.abbreviate(lines.toString(), "...", LineFormatter.MAX_LINE_LENGTH); //$NON-NLS-1$
        }

    }

}
