/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.importer.wizard.parts;

import java.util.List;

import de.frankmuenster.jameica.depot.internal.objects.Order;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.boxes.Box;
import de.willuhn.jameica.gui.formatter.DateFormatter;
import de.willuhn.jameica.gui.parts.TablePart;

/**
 * @author Frank Mnster
 */
public class OrderTablePart extends TablePart implements Box {

    /**
     * The height of the table. Deafault is 130
     */
    private int height = 130;

    /**
     * Constructs a new table to list all orders that have been loaded
     * successfully.
     *
     * @param list
     *            with successfully loaded {@link Order}
     * @param action
     */
    public OrderTablePart(final List<Order> list, final Action action) {

        super(list, action);

        // Set basic behaviour
        this.setRememberColWidths(true);
        this.setRememberOrder(false);
        this.setRememberState(false);
        // Add columns
        final DateFormatter daf = new DateFormatter();
        this.addColumn("ISIN", "isin"); //$NON-NLS-1$//$NON-NLS-2$
        this.addColumn(I18N.tr("OrderTablePart.date"), "orderDate", daf); //$NON-NLS-1$//$NON-NLS-2$
        this.addColumn(I18N.tr("OrderTablePart.amount"), "orderVolume"); //$NON-NLS-1$//$NON-NLS-2$
        this.addColumn(I18N.tr("OrderTablePart.rate"), "orderPrice"); //$NON-NLS-1$//$NON-NLS-2$
        this.addColumn(I18N.tr("OrderTablePart.currency"), "currency"); //$NON-NLS-1$//$NON-NLS-2$
        this.addColumn(I18N.tr("OrderTablePart.comment"), "comment"); //$NON-NLS-1$//$NON-NLS-2$

    }

    /**
     * Always 0 as we do not sort
     */
    @Override
    public int compareTo(final Object object) {
        return 0;
    }

    /**
     * @see #getDefaultEnabled()
     */
    @Override
    public boolean getDefaultEnabled() {
        return true;
    }

    /**
     * Always 0
     */
    @Override
    public int getDefaultIndex() {
        return 0;
    }

    /**
     * @see #getHeight()
     */
    @Override
    public int getHeight() {
        return this.height;
    }

    /**
     * Not used! getIndex will always return 0.
     */
    @Override
    public int getIndex() {
        return 0;
    }

    /**
     * @see Box#getName()
     */
    @Override
    public String getName() {
        return I18N.tr("OrderTablePart.tableName"); //$NON-NLS-1$
    }

    /**
     * Return if the box should be shown at all.
     *
     * @return <i>true</i> in case the TablePart is enabled, <i>false</i> else.
     *
     */
    @Override
    public boolean isActive() {
        return isEnabled();
    }

    /**
     * Sets the height of the table. default is 130
     *
     * @param height
     *            the height of the table
     */
    public void setHeight(final int height) {
        this.height = height;
    }

    /**
     * Not used! getIndex will always return 0.
     */
    @Override
    public void setIndex(final int index) {
        // Nothing to do here
    }
}
