package de.frankmuenster.jameica.depot.internal.objects;

import java.io.Serializable;
import java.time.LocalDate;

/**
 * This class offers a simple date range handling. It is composed out of a start
 * date and an end date.
 *
 * @author Frank Mnster
 */
public class DateRange implements Serializable {

    /**
     * The UID
     */
    private static final long serialVersionUID = 7413064455646018138L;

    /**
     * The start date
     */
    private LocalDate startDate;

    /**
     * The end date
     */
    private LocalDate endDate;

    /**
     * Creates a new DateRange object with the given start and end date.
     *
     * @param startDate
     * @param endDate
     */
    public DateRange(final LocalDate startDate, final LocalDate endDate) {
        this.startDate = LocalDate.from(startDate);
        this.endDate = LocalDate.from(endDate);
    }

    /**
     * Returns the end date
     *
     * @return the end date
     */
    public LocalDate getEndDate() {

        // make defence copy
        return LocalDate.from(this.endDate);
    }

    /**
     * Returns the start date
     *
     * @return the start date
     */
    public LocalDate getStartDate() {

        return LocalDate.from(this.startDate);
    }

    /**
     * Sets the end date
     *
     * @param endDate
     */
    public void setEndDate(final LocalDate endDate) {

        this.endDate = LocalDate.from(endDate);
    }

    /**
     * Sets the start date
     *
     * @param startDate
     */
    public void setStartDate(final LocalDate startDate) {

        this.startDate = LocalDate.from(startDate);
    }

}
