/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.internal.objects; // NOPMD

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;

/**
 * Interface das ein Order-Objekt beschreibt.
 *
 * @author Frank Mnster
 */
@SuppressWarnings("PMD.LongVariable")
public interface Order {

    /**
     * @return additionalCosts
     */
    BigDecimal getAdditionalCosts();

    /**
     * @return additionalCostsMainCurrency
     */
    String getAdditionalCostsCurrency();

    /**
     * @return additionalCostsInMainCurrency
     */
    BigDecimal getAdditionalCostsInMainCurrency();

    /**
     * @return ardId
     */
    String getArdId();

    /**
     * @return comment
     */
    String getComment();

    /**
     * @return currency
     */
    String getCurrency();

    /**
     * @return isin the isin of the order
     */
    String getIsin();

    /**
     * @return name
     */
    String getName();

    /**
     * @return orderDate
     */
    Date getOrderDate();

    /**
     * @param orderNumber
     *            the orderNumber to set
     */
    String getOrderNumber();

    /**
     * @return orderPrice
     */
    BigDecimal getOrderPrice();

    /**
     * @return orderPriceInMainCurrency
     */
    BigDecimal getOrderPriceInMainCurrency();

    /**
     * Implementations of this method should use {@link RoundingMode#HALF_EVEN}
     *
     * @return orderPriceTotal
     */
    BigDecimal getOrderPriceTotal();

    /**
     * @return orderVolume
     */
    BigDecimal getOrderVolume();

    /**
     * @param additionalCosts
     *            das zu setzende Objekt additionalCosts
     */
    void setAdditionalCosts(BigDecimal additionalCosts);

    /**
     * @param additionalCostsMainCurrency
     *            das zu setzende Objekt additionalCostsMainCurrency
     */
    void setAdditionalCostsCurrency(String additionalCostsMainCurrency);

    /**
     * @param additionalCostsInMainCurrency
     *            das zu setzende Objekt additionalCostsInMainCurrency
     */
    void setAdditionalCostsInMainCurrency(BigDecimal additionalCostsInMainCurrency);

    /**
     * @param ardId
     *            the ardId of the oder
     */
    void setArdId(String ardId);

    /**
     * @param comment
     *            das zu setzende Objekt comment
     */
    void setComment(String comment);

    /**
     * @param currency
     *            das zu setzende Objekt comment
     */
    void setCurrency(String currency);

    /**
     * @param isin
     *            the isin of the oder
     */
    void setIsin(String isin);

    /**
     * @param name
     *            das zu setzende Objekt name
     */
    void setName(String name);

    /**
     * @param date
     *            das zu setzende Objekt orderDate
     */
    void setOrderDate(Date date);

    /**
     * @return the orderNumber
     */
    void setOrderNumber(String orderNumber);

    /**
     * @param orderPrice
     *            das zu setzende Objekt orderPrice
     */
    void setOrderPrice(BigDecimal orderPrice);

    /**
     * @param orderPriceInMainCurrency
     *            das zu setzende Objekt orderPriceInMainCurrency
     */
    void setOrderPriceInMainCurrency(BigDecimal orderPriceInMainCurrency);

    /**
     * @param orderPriceTotal
     *            das zu setzende Objekt orderPriceTotal
     */
    void setOrderPriceTotal(BigDecimal orderPriceTotal);

    /**
     * @param orderVolume
     *            das zu setzende Objekt orderVolume
     */
    void setOrderVolume(BigDecimal orderVolume);

}
