/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.internal.objects;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;

/**
 * Implementierung einer Order-Bean
 *
 * @author Frank Mnster
 */
@SuppressWarnings("PMD.LongVariable")
public class OrderImpl implements Order {

    /**
     * Die zus&auml;tzlichen Kosten
     */
    private BigDecimal additionalCosts;

    /**
     * Die W&auml;hrung derzus&auml;tzlichen Kosten
     */
    private String additionalCostsCurrency;

    /**
     * Die zus&auml;tzlichen Kosten in der haupw&auml;hrung
     */
    private BigDecimal additionalCostsInMainCurrency;

    /**
     * Kommantar
     */
    private String comment;

    /**
     * The currency of the order. Should be same then the one of the stock
     */
    private String currency;

    /**
     * Der name des WP
     */
    private String name;

    /**
     * Das Order Datum
     */
    private Date orderDate;

    /**
     * The order number
     */
    private String orderNumber;

    /**
     * Der Order-Preis
     */
    private BigDecimal orderPrice;

    /**
     * Der Order-Preis in der Hauptw&auml;hrung
     */
    private BigDecimal orderPriceInMainCurrency;

    /**
     * Das Order Volumen
     */
    private BigDecimal orderVolume;

    /**
     * The ISIN of the security
     */
    private String isin;
    /**
     * The id from ARD Brse
     */
    private String ardId;

    /**
     * Standardkonstruktor
     */
    public OrderImpl() {
        // Absichlich leer
    }

    /**
     * @see Order#getAdditionalCosts()
     */
    @Override
    public BigDecimal getAdditionalCosts() {

        return this.additionalCosts;
    }

    /**
     * @see Order#getAdditionalCostsCurrency()
     */
    @Override
    public String getAdditionalCostsCurrency() {

        return this.additionalCostsCurrency;
    }

    /**
     * @see Order#getAdditionalCostsInMainCurrency()
     */
    @Override
    public BigDecimal getAdditionalCostsInMainCurrency() {

        return this.additionalCostsInMainCurrency;
    }

    /**
     * @see Order#getArdId()
     */
    @Override
    public String getArdId() {

        return this.ardId;
    }

    /**
     * @see Order#getComment()
     */
    @Override
    public String getComment() {

        return this.comment;
    }

    /**
     * @return the currency
     */
    @Override
    public String getCurrency() {
        return this.currency;
    }

    /**
     * @return the isin
     */
    @Override
    public String getIsin() {
        return this.isin;
    }

    /**
     * @see Order#getName()
     */
    @Override
    public String getName() {

        return this.name;
    }

    /**
     * @see Order#getOrderDate()
     */
    @Override
    public Date getOrderDate() {

        return this.orderDate;
    }

    /**
     * @see Order#getOrderNumber()
     */
    @Override
    public String getOrderNumber() {
        return this.orderNumber;
    }

    /**
     * @see Order#getOrderPrice()
     */
    @Override
    public BigDecimal getOrderPrice() {

        return this.orderPrice;
    }

    /**
     * @see Order#getOrderPriceInMainCurrency()
     */
    @Override
    public BigDecimal getOrderPriceInMainCurrency() {

        return this.orderPriceInMainCurrency;
    }

    /**
     * @see Order#getOrderPriceTotal()
     */
    @Override
    public BigDecimal getOrderPriceTotal() {

        return this.getOrderPrice().multiply(getOrderVolume()).setScale(2, RoundingMode.HALF_EVEN);
    }

    /**
     * @see Order#getOrderVolume()
     */
    @Override
    public BigDecimal getOrderVolume() {

        return this.orderVolume;
    }

    /**
     * @see Order#setAdditionalCosts(double)
     */
    @Override
    public void setAdditionalCosts(final BigDecimal additionalCosts) {

        this.additionalCosts = additionalCosts;
    }

    /**
     * @see Order#setAdditionalCostsCurrency(String)
     */
    @Override
    public void setAdditionalCostsCurrency(final String additionalCostsCurrency) {

        this.additionalCostsCurrency = additionalCostsCurrency;
    }

    /**
     * @see Order#setAdditionalCostsInMainCurrency(double)
     */
    @Override
    public void setAdditionalCostsInMainCurrency(final BigDecimal additionalCostsInMainCurrency) {

        this.additionalCostsInMainCurrency = additionalCostsInMainCurrency;
    }

    /**
     * @see Order#setArdId(String)
     */
    @Override
    public void setArdId(final String symbol) {

        this.ardId = symbol;
    }

    /**
     * @see Order#setComment(String)
     */
    @Override
    public void setComment(final String comment) {

        this.comment = comment;
    }

    /**
     * @see Order#setCurrency(String)
     */
    @Override
    public void setCurrency(final String currency) {
        this.currency = currency;
    }

    /**
     * @param isin
     *            the isin to set
     */
    @Override
    public void setIsin(final String isin) {
        this.isin = isin;
    }

    /**
     * @see Order#setName(String)
     */
    @Override
    public void setName(final String name) {

        this.name = name;
    }

    /**
     * @see Order#setOrderDate(Date)
     */
    @Override
    public void setOrderDate(final Date orderDate) {

        this.orderDate = orderDate;
    }

    /**
     * @see Order#setOrderNumber(String)
     */
    @Override
    public void setOrderNumber(final String orderNumber) {
        this.orderNumber = orderNumber;
    }

    /**
     * @see Order#setOrderPrice(double)
     */
    @Override
    public void setOrderPrice(final BigDecimal orderPrice) {

        this.orderPrice = orderPrice;
    }

    /**
     * @see Order#setOrderPriceInMainCurrency(double)
     */
    @Override
    public void setOrderPriceInMainCurrency(final BigDecimal orderPriceInMainCurrency) {

        this.orderPriceInMainCurrency = orderPriceInMainCurrency;
    }

    /**
     * @see Order#setOrderPriceTotal(double)
     */
    @Override
    public void setOrderPriceTotal(final BigDecimal orderPriceTotal) {

        // TODO: Mal schauen
    }

    /**
     * @see Order#setOrderVolume(double)
     */
    @Override
    public void setOrderVolume(final BigDecimal orderVolume) {

        this.orderVolume = orderVolume;
    }
}
