/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.kursanalyse;

import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

/**
 * Abstrakte Klassen zum Behandeln von Analysehinzuf&uuml;gen-Events.
 *
 * @author Frank Mnster
 *
 */
public abstract class AbstractHandleAddAction implements IHandleAddAction {

    /**
     * Die Listener fr einen AddMethode-Event
     */
    private final Set<MethodeAddRemoveListener> listeners = new CopyOnWriteArraySet<>();

    /**
     * Standardkonstruktor
     */
    public AbstractHandleAddAction() {
	// Absichtlich leer
    }

    /**
     * Fgt eine Listener hinzu
     *
     * @param listener
     *            der hinzugefgt werden soll
     */
    @Override
    public final void addListener(final MethodeAddRemoveListener listener) {
	this.listeners.add(listener);
    }

    /**
     * Benachrichtigen der registrierten Listeners mit bergabe des gefundenen
     * Strings
     *
     * @param methode
     */
    @Override
    public final void notifyListeners(final IKursAnalyse<?, ?> methode) {
	for (final MethodeAddRemoveListener listener : this.listeners) {
	    listener.notifyAddMethode(methode);
	}
    }

    /**
     * Entfernt einen Listener aus der Liste
     *
     * @param listener
     *            der entfernt werden soll
     */
    @Override
    public final void removeListener(final MethodeAddRemoveListener listener) {
	this.listeners.remove(listener);
    }

}
