/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/
package de.frankmuenster.jameica.depot.kursanalyse;

import org.eclipse.swt.graphics.Color;

import de.frankmuenster.jameica.depot.finanzen.WertpapierHistoryBean;
import de.frankmuenster.jameica.depot.internal.objects.DateRange;

/**
 * Interface einer Analysemethode.
 *
 * @param <P>
 *            Der Type des KursAnalyseParameters
 * @param <R>
 *            Der Ergebnistyp. Darf nur KursDaten[] oder XYPLot sein.
 * 
 * @author Frank Mnster
 *
 */
public interface IKursAnalyse<P, R> {

    /**
     * Liefert die Farben.
     *
     * @return die Farben
     */
    Color[] getColor();

    /**
     * Liefert den Chart-Legenden-Titel.
     *
     * @return den Chart-Legenden-Titel
     */
    String getLegendTitel();

    /**
     * Setzt die Anzahl der Linien im Kurstyp OVERLAY
     *
     * @return Anzahl Linien
     */
    int getLineCount();

    /**
     * Liefert den Linien-Style, <I>FULL</i> oder <i>DOTTED</i>.
     *
     * @return den KursAnalyseLineStyle
     */
    KursAnalyseLineStyle[] getLineStyle();

    /**
     * Liefert das Ergebnis der Analyse f&uuml;r den angegebenen Zeitraum.
     * Entweder ein Array aus <i>KursDaten[]</i> oder eine Chart als
     * <i>XYPlot</i>.
     *
     * @param period
     *            the <code>DateRange</code> object holding start and end date
     * @return entweder <i>KursDaten[]</i>, falls der Typ einer der
     *         <i>OVERLAY</i>-Typen ist, oder <i>XYPLot</i>, falls der Typ
     *         <i>SEPERATE_GRAPH</i> ist.
     */
    R getResult(DateRange period);

    /**
     * Liefert den Typ der Analyse (Overlay-Linie/-Bar, eigenst&auml;ndiger
     * Chart.
     *
     * @return der KursAnalyseTyp
     */
    KursAnalyseTyp getType();

    /**
     * Setzt die Farben.
     *
     * @param color
     *            das zu setzende Farb-Array
     */
    void setColor(Color... color);

    /**
     * Setzte den KursAnalyseParameter
     *
     * @param parameter
     */
    void setKursAnalyseParameter(KursAnalyseParameter<P> parameter);

    /**
     * Setzt die Anzahl an Kurslinien im Kurstyp OVERLAY
     *
     * @param count
     *            die Anzhal an Linien
     */
    void setLineCount(int count);

    /**
     * Setzt den Linien-Style, <I>FULL</i> oder <i>DOTTED</i>.
     *
     * @param lineStyle
     *            den KursAnalyseLineStyle
     */
    void setLineStyle(KursAnalyseLineStyle... lineStyle);

    /**
     * Setzt den Typ der Analyse (Overlay-Linie/-Bar, eigenst&auml;ndiger Chart.
     *
     * @param type
     *            der KursAnalyseTyp
     */
    void setType(KursAnalyseTyp type);

    /**
     * Setzt die historischen WP-Werte.
     *
     * @param wpHistoryBean
     */
    void setWertpapierHistory(WertpapierHistoryBean wpHistoryBean);
}
