/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.kursanalyse.bollingbaender; // NOPMD

import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.depot.kursanalyse.dialog.AbstractAddAnalyseDialog;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.input.SelectInput;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.util.ApplicationException;

/**
 * Bolling-B&auml;nder AddDialog
 *
 * @author Frank Mnster
 */
public class BollingBaenderAddDialog extends AbstractAddAnalyseDialog<Integer> {

    /**
     * Erzeugt einen neuen AddDialog
     */
    public BollingBaenderAddDialog() {
        super(I18N.tr("BollingBaenderAddDialog.dialogTitel")); //$NON-NLS-1$
        this.result = 20;
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(Composite
     *      parent)
     */
    @Override
    protected void paint(final Composite parent) {

        final GridLayout gridLayout = new GridLayout();
        gridLayout.numColumns = 1;
        parent.setLayout(gridLayout);

        // Erstelle einfachen Container
        final Container container = new SimpleContainer(parent, true, 2);

        final String[] list = { "10", "20", "50" }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
        final String preselected = Integer.toString(this.result);
        final SelectInput periode = new SelectInput(list, preselected);

        container.addLabelPair(I18N.tr("BollingBaenderAddDialog.period"), periode); //$NON-NLS-1$
        container.addText(" ", false); //$NON-NLS-1$

        final ButtonArea buttons = new ButtonArea();
        buttons.addButton(I18N.tr("_btnAdd"), new Action() { //$NON-NLS-1$
            /**
             * @see de.willuhn.jameica.gui.Action#handleAction(Object context)
             */
            @Override
            public void handleAction(final Object context) throws ApplicationException {
                BollingBaenderAddDialog.this.result = Integer.parseInt((String) periode.getValue());
                close();
            }
        }, null, true, "ok.png"); //$NON-NLS-1$

        buttons.addButton(I18N.tr("_cancelBtn"), new Action() { //$NON-NLS-1$
            /**
             * @see de.willuhn.jameica.gui.Action#handleAction(Object context)
             */
            @Override
            public void handleAction(final Object context) throws ApplicationException {
                BollingBaenderAddDialog.this.result = 0;
                close();
            }
        }, null, false, "window-close.png"); //$NON-NLS-1$

        container.addButtonArea(buttons);

    }

}
