/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.kursanalyse.bollingbaender; //NOPMD

import org.eclipse.swt.graphics.Color;

import de.frankmuenster.jameica.depot.gui.parts.grafiken.KursDaten;
import de.frankmuenster.jameica.depot.gui.utils.ColorUtils;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractHandleAddAction;
import de.frankmuenster.jameica.depot.kursanalyse.IHandleAddAction;
import de.frankmuenster.jameica.depot.kursanalyse.IKursAnalyse;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseLineStyle;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseParameter;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseTyp;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Bolling B&auml;nder AddAction
 *
 * @author Frank Mnster
 */
public class BollingBaenderHandleAddAction extends AbstractHandleAddAction implements IHandleAddAction {

    /**
     * Standardkonstruktor
     */
    public BollingBaenderHandleAddAction() {
	super();
    }

    /**
     * @see de.willuhn.jameica.gui.Action#handleAction(Object context)
     */
    @Override
    public void handleAction(final Object context) throws ApplicationException {
	Logger.info(I18N.tr("BollingBaenderHandleAddAction.logAdd")); //$NON-NLS-1$
	handleAdd();

    }

    /**
     * Methode hinzuf&uum,l;gen
     */
    private void handleAdd() {

	final BollingBaenderAddDialog dialog = new BollingBaenderAddDialog();
	int order = 0;
	try {
	    order = (Integer) dialog.open();
	} catch (final OperationCanceledException oce) {
	    Logger.info(I18N.tr("BollingBaenderHandleAddAction.logCancel")); //$NON-NLS-1$
	} catch (final Exception e) { // NOPMD
	    Logger.error(I18N.tr("BollingBaenderHandleAddAction.logAddError"), //$NON-NLS-1$
		    e);
	}

	if (order != 0) {

	    final KursAnalyseParameter<Integer> parameter = new KursAnalyseParameter<>(order);
	    final IKursAnalyse<Integer, KursDaten[]> bolling = new BollingBaender();
	    bolling.setKursAnalyseParameter(parameter);
	    bolling.setType(KursAnalyseTyp.OVERLAY_LINE);
	    bolling.setLineCount(3);
	    // drei Farben
	    final Color[] color = new Color[3];
	    color[0] = ColorUtils.getRandomColor();
	    color[1] = color[0];
	    color[2] = color[0];
	    bolling.setColor(color);
	    final KursAnalyseLineStyle[] lineStyle = new KursAnalyseLineStyle[3];
	    lineStyle[0] = KursAnalyseLineStyle.DOTTED;
	    lineStyle[1] = KursAnalyseLineStyle.FULL;
	    lineStyle[2] = KursAnalyseLineStyle.FULL;
	    bolling.setLineStyle(lineStyle);
	    // Call listener
	    notifyListeners(bolling);
	}
    }
}
