/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.kursanalyse.dialog; // NOPMD

import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.input.IntegerInput;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.util.ApplicationException;

/**
 * A dialog to request a single integer value *
 *
 * @author Frank Mnster
 */
public class SingleIntAddDialog extends AbstractAddAnalyseDialog<Integer> {

    /**
     * The label text of the value
     */
    private final String valueName;

    /**
     * The default value
     */
    private final Integer defaultValue;

    /**
     * Constructs a new AddDialog for a single integer
     *
     * @param title
     *            the dialog title
     * @param valueName
     *            the label text of the value
     * @param defaultVaule
     *            the default value in case the dialog is cancelled
     */
    public SingleIntAddDialog(final String title, final String valueName, final Integer defaultVaule) {
        super(title);
        this.defaultValue = defaultVaule;
        this.result = this.defaultValue;
        this.valueName = valueName;

    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(Composite
     *      parent)
     */
    @Override
    protected void paint(final Composite parent) {

        final GridLayout gridLayout = new GridLayout();
        gridLayout.numColumns = 1;
        parent.setLayout(gridLayout);

        // Erstelle einfachen Container
        final Container container = new SimpleContainer(parent, true, 2);

        final IntegerInput intInput = new IntegerInput();
        intInput.setHint(Integer.toString(this.result));
        intInput.setName(this.valueName);
        intInput.setComment(I18N.tr("SingleIntAddDialog.periodComment", Integer.toString(this.result))); //$NON-NLS-1$
        intInput.setValue(this.defaultValue);
        container.addLabelPair(this.valueName, intInput);
        container.addText(" ", false); //$NON-NLS-1$

        final ButtonArea buttons = new ButtonArea();
        buttons.addButton(I18N.tr("_btnAdd"), //$NON-NLS-1$
                new Action() {

                    /**
                     * @see de.willuhn.jameica.gui.Action#handleAction(Object
                     *      context)
                     */
                    @Override
                    public void handleAction(final Object context) throws ApplicationException {

                        if (intInput.getValue() == null) {
                            SingleIntAddDialog.this.result = SingleIntAddDialog.this.defaultValue;
                        } else {
                            SingleIntAddDialog.this.result = (Integer) intInput.getValue();
                        }
                        SingleIntAddDialog.this.close();
                    }
                }, null, true, "ok.png"); //$NON-NLS-1$

        buttons.addButton(I18N.tr("_cancelBtn"), new Action() { //$NON-NLS-1$

            /**
             * @see de.willuhn.jameica.gui.Action#handleAction(Object context)
             */
            @Override
            public void handleAction(final Object context) throws ApplicationException {

                SingleIntAddDialog.this.result = 0;
                SingleIntAddDialog.this.close();
            }
        }, null, false, "window-close.png"); //$NON-NLS-1$

        container.addButtonArea(buttons);

    }

}
