/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.kursanalyse.ema; // NOPMD

import org.eclipse.swt.graphics.Color;

import de.frankmuenster.jameica.depot.gui.parts.grafiken.KursDaten;
import de.frankmuenster.jameica.depot.gui.utils.ColorUtils;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractHandleAddAction;
import de.frankmuenster.jameica.depot.kursanalyse.IHandleAddAction;
import de.frankmuenster.jameica.depot.kursanalyse.IKursAnalyse;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseLineStyle;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseParameter;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseTyp;
import de.frankmuenster.jameica.depot.kursanalyse.dialog.SingleIntAddDialog;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * EMA AddAction.
 *
 * @author Frank Mnster
 */
public class EmaHandleAddAction extends AbstractHandleAddAction implements IHandleAddAction {

    /**
     * Standardkonstruktor
     */
    public EmaHandleAddAction() {
        super();
    }

    /**
     * @see de.willuhn.jameica.gui.Action#handleAction(Object context)
     */
    @Override
    public void handleAction(final Object context) throws ApplicationException {

        Logger.info(I18N.tr("EmaHandleAddAction.logAdd")); //$NON-NLS-1$
        this.handleAddEMA();

    }

    private void handleAddEMA() {

        final SingleIntAddDialog dialog = new SingleIntAddDialog(I18N.tr("EmaAddDialog.dialopTitle"), //$NON-NLS-1$
                I18N.tr("EmaAddDialog.periodName"), 50); //$NON-NLS-1$
        int order = 0;
        try {
            order = (Integer) dialog.open();
        } catch (final OperationCanceledException oce) {
            Logger.info(I18N.tr("EmaHandleAddAction.logAddCanceled")); //$NON-NLS-1$
        } catch (final Exception e) { // NOPMD
            Logger.error(I18N.tr("EmaHandleAddAction.logAddError"), e); //$NON-NLS-1$
        }

        if (order > 0) {

            final KursAnalyseParameter<Integer> parameter = new KursAnalyseParameter<>(order);
            final IKursAnalyse<Integer, KursDaten[]> ema = new ExponentialMovingAverage();
            ema.setKursAnalyseParameter(parameter);
            ema.setType(KursAnalyseTyp.OVERLAY_LINE);
            ema.setLineCount(1);
            final Color color = ColorUtils.getRandomColor();
            ema.setColor(color);
            final KursAnalyseLineStyle lineStyle = KursAnalyseLineStyle.FULL;
            ema.setLineStyle(lineStyle);

            // Call listener
            this.notifyListeners(ema);
        }
    }
}
