/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/
package de.frankmuenster.jameica.depot.kursanalyse.ema;

import java.time.LocalDate;

import de.frankmuenster.jameica.depot.finanzen.WertpapierHistoryBean;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.DateValuePair;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.KursDaten;
import de.frankmuenster.jameica.depot.gui.utils.DateUtils;
import de.frankmuenster.jameica.depot.internal.objects.DateRange;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractKursAnalyse;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseParameter;
import de.willuhn.jameica.gui.Action;

/**
 * ExponentialMovingAverage
 *
 * @author Frank Mnster
 */
public class ExponentialMovingAverage extends AbstractKursAnalyse<Integer, KursDaten[]> {

    /**
     * Methodenname
     */
    private static final String NAME = "EMA"; //$NON-NLS-1$

    /**
     * ExponentialMovingAverage Daten
     */
    private KursDaten ema;

    /**
     * Liefert die AddAction
     *
     * @return die EmaHandleAddAction
     */
    public static Action getHandleAction() {

        return new EmaHandleAddAction();
    }

    /**
     * Liefert den Methodennamen
     *
     * @return der Methodennamen
     */
    public static String getMethodName() {

        return ExponentialMovingAverage.NAME;
    }

    /**
     * Konstruktor der nurt super() aufruft.
     */
    public ExponentialMovingAverage() {
        super();
    }

    /**
     * Konstruktor mit Angabe der Kursdaten und des Parameters.
     *
     * @param wpHistoryBean
     * @param parameter
     */
    public ExponentialMovingAverage(final WertpapierHistoryBean wpHistoryBean,
            final KursAnalyseParameter<Integer> parameter) {
        super(wpHistoryBean, parameter);
    }

    /**
     * @see AbstractKursAnalyse#getLegendTitel ()
     */
    @Override
    public String getLegendTitel() {

        return "EMA" + this.parameter.getValue(); //$NON-NLS-1$
    }

    /**
     * @see AbstractKursAnalyse#getResult(DateRange)
     */
    @Override
    public KursDaten[] getResult(final DateRange period) {

        final LocalDate fromDate = period.getStartDate();
        final LocalDate toDate = period.getEndDate();

        if (this.ema == null) {
            this.calculate();
        }
        final KursDaten[] result = new KursDaten[1];
        final KursDaten emaSeries = new KursDaten();
        for (int i = 0; i < this.ema.size(); i++) {
            final DateValuePair orgValue = this.ema.get(i);
            if (DateUtils.isInRange(orgValue.getDate(), fromDate, toDate)) {
                emaSeries.add(orgValue);
            }
        }
        result[0] = emaSeries;
        return result;
    }

    /**
     * Hier wird gerechnet
     */
    private void calculate() {

        // Das Intervall
        final Integer interval = this.parameter.getValue();
        // EMA Daten
        this.ema = new KursDaten();
        double expWight;
        expWight = 2.0 / (interval + 1.0);
        this.ema.add(new DateValuePair(this.closeDaten.get(0).getDate(), this.closeDaten.get(0).getValue()));
        for (int i = 1; i < this.closeDaten.size(); i++) {
            final double emaValue = this.ema.get(i - 1).getValue()
                    + expWight * (this.closeDaten.get(i).getValue() - this.ema.get(i - 1).getValue());
            this.ema.add(new DateValuePair(this.closeDaten.get(i).getDate(), emaValue)); // NOPMD
        }
    }
}
