/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/
package de.frankmuenster.jameica.depot.kursanalyse.mom;

import java.time.LocalDate;

import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.SegmentedTimeline;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.data.time.Day;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;

import de.frankmuenster.jameica.depot.gui.utils.DateUtils;
import de.frankmuenster.jameica.depot.gui.utils.ItemUtils;
import de.frankmuenster.jameica.depot.internal.objects.DateRange;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractKursAnalyse;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.logging.Logger;

/**
 * Momentum Methode
 *
 * @author Frank Mnster
 */
public class Momentum extends AbstractKursAnalyse<MomentumParameter, XYPlot> {

    /**
     * Methodenname
     */
    private static final String NAME = "Momentum"; //$NON-NLS-1$

    /**
     * Werte der Serie
     */
    private TimeSeries serieMOM;

    /**
     * Der Rendere
     */
    private XYItemRenderer rendererMOM;

    /**
     * Liefert die AddAction
     *
     * @return die MomentumHandleAddAction
     */
    public static Action getHandleAction() {

	return new MomentumHandleAddAction();
    }

    /**
     * Liefert den Methodennamen
     *
     * @return der Methodennamen
     */
    public static String getMethodName() {

	return Momentum.NAME;
    }

    /**
     * Standardkonstruktor
     */
    public Momentum() {
	super();
    }

    /**
     * @see AbstractKursAnalyse#getLegendTitel()
     */
    @Override
    public String getLegendTitel() {

	final MomentumParameter para = this.parameter.getValue();
	final int period = para.getPeriod();
	return "Momentum (" + period + ")"; //$NON-NLS-1$ //$NON-NLS-2$
    }

    /**
     * @see AbstractKursAnalyse#getResult(DateRange)
     */
    @Override
    public XYPlot getResult(final DateRange period) {

	final LocalDate fromDate = period.getStartDate();
	final LocalDate toDate = period.getEndDate();

	if (this.anaG == null) {
	    final DateAxis dateAxisKurs = new DateAxis(I18N.tr("Momentum.axisDate")); //$NON-NLS-1$
	    final NumberAxis rangeAxisKurs = new NumberAxis(I18N.tr("Momentum.axisNumber")); //$NON-NLS-1$
	    rangeAxisKurs.setAutoRangeIncludesZero(false);
	    this.anaG = new XYPlot(null, dateAxisKurs, rangeAxisKurs, null);
	    ((DateAxis) this.anaG.getDomainAxis()).setTimeline(SegmentedTimeline.newMondayThroughFridayTimeline());
	    this.anaG.setRangeAxisLocation(AxisLocation.BOTTOM_OR_LEFT);
	    this.rendererMOM = new StandardXYItemRenderer();
	    this.rendererMOM.setSeriesPaint(0, java.awt.Color.BLACK);
	    ItemUtils.addLegendTitle(this.anaG);
	}
	if (this.serieMOM == null) {
	    this.calculate();
	}
	final Day fromDay = new Day(DateUtils.asDate(fromDate));
	final Day toDay = new Day(DateUtils.asDate(toDate));
	try {
	    final TimeSeriesCollection tmpMom = new TimeSeriesCollection(this.serieMOM.createCopy(fromDay, toDay));
	    this.anaG.setDataset(0, tmpMom);
	    this.anaG.setRenderer(0, this.rendererMOM);
	} catch (final CloneNotSupportedException e) {
	    Logger.error(I18N.tr("Momentum.createTsDataError"), //$NON-NLS-1$
		    e);
	}
	return this.anaG;
    }

    /**
     * Calculate momentum
     */
    private void calculate() {

	final int period = this.parameter.getValue().getPeriod();
	this.serieMOM = new TimeSeries(this.getLegendTitel());
	Day day = new Day(DateUtils.asDate(this.closeDaten.get(0).getDate()));
	this.serieMOM.add(day, 0.0);
	for (int i = 1; i < this.closeDaten.size(); i++) {
	    double newValue = Double.NaN;
	    final double value = this.closeDaten.get(i).getValue();
	    day = new Day(DateUtils.asDate(this.closeDaten.get(i).getDate())); // NOPMD
	    if (i >= period) {
		newValue = value - this.closeDaten.get(i - period).getValue();
	    }
	    this.serieMOM.add(day, newValue);
	}
    }
}
