/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.kursanalyse.mom;

import org.jfree.chart.plot.XYPlot;

import de.frankmuenster.jameica.depot.kursanalyse.AbstractHandleAddAction;
import de.frankmuenster.jameica.depot.kursanalyse.IKursAnalyse;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseParameter;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseTyp;
import de.frankmuenster.jameica.depot.kursanalyse.dialog.SingleIntAddDialog;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Momentum HandleAddAction
 *
 * @author Frank Mnster
 */
public class MomentumHandleAddAction extends AbstractHandleAddAction {

    /**
     * Standardkonstruktor
     */
    public MomentumHandleAddAction() {
        super();
    }

    /**
     * @see de.willuhn.jameica.gui.Action#handleAction(Object context)
     */
    @Override
    public void handleAction(final Object context) throws ApplicationException {

        Logger.info(I18N.tr("MomentumHandleAddAction.logAdd")); //$NON-NLS-1$
        this.handleAdd();
    }

    private void handleAdd() {

        final SingleIntAddDialog dialog = new SingleIntAddDialog(I18N.tr("MomentumAddDialog.dialogTitle"), //$NON-NLS-1$
                I18N.tr("MomentumAddDialog.periodName"), 14); //$NON-NLS-1$
        int period = 0;
        try {
            period = (Integer) dialog.open();
        } catch (final OperationCanceledException oce) {
            Logger.info(I18N.tr("MomentumHandleAddAction.logAddCanceled")); //$NON-NLS-1$
        } catch (final Exception e) { // NOPMD
            Logger.error(I18N.tr("MomentumHandleAddAction.logAddError"), //$NON-NLS-1$
                    e);
        }

        if (period > 0) {
            final MomentumParameter momPara = new MomentumParameter();
            momPara.setPeriod(period);

            final KursAnalyseParameter<MomentumParameter> para = new KursAnalyseParameter<>(momPara);

            final IKursAnalyse<MomentumParameter, XYPlot> mom = new Momentum();
            mom.setKursAnalyseParameter(para);
            mom.setType(KursAnalyseTyp.SEPERATE_GRAPH);

            // Call listener
            this.notifyListeners(mom);
        }
    }

}
