/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.kursanalyse.rsi;

import org.jfree.chart.plot.XYPlot;

import de.frankmuenster.jameica.depot.kursanalyse.AbstractHandleAddAction;
import de.frankmuenster.jameica.depot.kursanalyse.IKursAnalyse;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseParameter;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseTyp;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Logger;

/**
 * RelativeStrengthIndex HandleAddAction
 *
 * @author Frank Mnster
 */
public class RelativeStrengthIndexHandleAction extends AbstractHandleAddAction {

    /**
     * Standardkonstruktor
     */
    public RelativeStrengthIndexHandleAction() {
	super();
    }

    /**
     * @see de.willuhn.jameica.gui.Action#handleAction(Object context)
     */
    @Override
    public void handleAction(final Object context) throws de.willuhn.util.ApplicationException {

	Logger.info(I18N.tr("RelativeStrengthIndexHandleAction.logAdd")); //$NON-NLS-1$
	this.handleAddVolumen();
    }

    private void handleAddVolumen() {

	final RelativeStrengthIndexAddDialog dialog = new RelativeStrengthIndexAddDialog();
	int order = 0;
	try {
	    order = (Integer) dialog.open();
	} catch (final OperationCanceledException oce) {
	    Logger.info(I18N.tr("RelativeStrengthIndexHandleAction.logAddCancel")); //$NON-NLS-1$
	} catch (final Exception e) { // NOPMD
	    Logger.error(I18N.tr("RelativeStrengthIndexHandleAction.logAddError"), e); //$NON-NLS-1$
	}

	if (order != 0) {

	    final KursAnalyseParameter<Integer> parameter = new KursAnalyseParameter<>(order);
	    final IKursAnalyse<Integer, XYPlot> rsi = new RelativeStrengthIndex();
	    rsi.setKursAnalyseParameter(parameter);
	    rsi.setType(KursAnalyseTyp.SEPERATE_GRAPH);

	    // Call listener
	    this.notifyListeners(rsi);
	}
    }
}
