/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.utils;

import java.util.Locale;

import org.apache.commons.lang.StringUtils;

/**
 * @author Frank Mnster/ Amgad Hanafy
 */
public final class Utils {

    /**
     * Get all casses for the given extension Author: Amgad Hanafy <BR>
     * <a href="mailto:amgoody2002@yahoo.com"> amgoody2002@yahoo.com </a><BR>
     * <a href="mailto:amgad.mohamed@ams-europe.com">
     * amgad.mohamed@ams-europe.com </a> <BR>
     *
     * @param extension
     * @return String of all casses of the given extension
     */
    public static String getAllCassesForExtensionInString(final String extension) {

        StringBuilder allExtensions = new StringBuilder();
        final String[] allCasses = Utils.getAllCassesForExtension(extension);
        for (final String extensionCase : allCasses) {
            allExtensions = allExtensions.append("*.").append(extensionCase).append(';'); //$NON-NLS-1$
        }
        return allExtensions.toString();
    }

    /**
     * Get all casses for the given array of extensions Author: Amgad Hanafy
     * <BR>
     * <a href="mailto:amgoody2002@yahoo.com"> amgoody2002@yahoo.com </a><BR>
     * <a href="mailto:amgad.mohamed@ams-europe.com">
     * amgad.mohamed@ams-europe.com </a> <BR>
     *
     * @param extensions
     *            array of extensions (without *.)
     * @return String of all casses of the given extensions
     */
    public static String getAllCassesForExtensionInString(final String[] extensions) { // NOPMD

        final StringBuilder allExtensions = new StringBuilder();
        for (final String extension : extensions) {
            final String[] allCasses = Utils.getAllCassesForExtension(extension);
            for (final String extensionCase : allCasses) {
                allExtensions.append("*.").append(extensionCase).append(';'); //$NON-NLS-1$
            }
        }
        return allExtensions.toString();
    }

    /**
     * @param extension
     * @return all possible extension for a given lower case one
     */
    private static String[] getAllCassesForExtension(final String extension) {

        final int length = extension.trim().length();
        final int pow = (int) Math.pow(2, length);
        final String[] allExtensions = new String[pow];
        for (int i = 0; i < pow; i++) {
            for (int j = length - 1; j >= 0; j--) {
                final String string = new String(new char[] { extension.charAt(j) }); // NOPMD
                final int k = length - 1 - j; // NOPMD
                final int rotateRight = i >> k;
                final String toBinaryString = Integer.toBinaryString(rotateRight);
                final boolean isLowestZero = toBinaryString.charAt(toBinaryString.length() - 1) == '0';
                final String string2 = isLowestZero ? string.toLowerCase(Locale.getDefault())
                        : string.toUpperCase(Locale.getDefault());
                if (allExtensions[i] == null) {
                    allExtensions[i] = StringUtils.EMPTY;
                }
                allExtensions[i] = string2 + allExtensions[i]; // NOPMD
            }
        }
        return allExtensions;
    }

    /**
     * Private constructor to avoid instantiation of utility class
     */
    private Utils() {
        // intentionally left blank
    }

}
